@@ -28,11 +28,10 @@
 #include <string.h>
 #include <glib.h>
 #include <fcntl.h>
-/* libgnome.h needed for _() */
-#include <libgnome/libgnome.h>
 
 #include "libbalsa.h"
 #include "misc.h"
+#include "i18n.h"
 
 LibBalsaMessageBody *
 libbalsa_message_body_new(LibBalsaMessage * message)
@@ -109,7 +108,12 @@ libbalsa_message_body_extract_embedded_headers(GMimeMessage* msg)
 
     subj = g_mime_message_get_subject(msg);
     if (subj) {
-	ehdr->subject = g_mime_utils_header_decode_text(subj);
+	ehdr->subject =
+#if HAVE_GMIME_2_2_5
+	    g_mime_utils_header_decode_text(subj);
+#else
+	    g_mime_utils_header_decode_text((const unsigned char *) subj);
+#endif
 	libbalsa_utf8_sanitize(&ehdr->subject, TRUE, NULL);
     } else 
 	ehdr->subject = g_strdup(_("(No subject)"));
@@ -249,7 +253,7 @@ gchar *
 libbalsa_message_body_get_parameter(LibBalsaMessageBody * body,
 				    const gchar * param)
 {
-    gchar *res;
+    gchar *res = NULL;
 
     g_return_val_if_fail(body != NULL, NULL);
 
@@ -257,7 +261,7 @@ libbalsa_message_body_get_parameter(LibBalsaMessageBody * body,
 	const GMimeContentType *type =
 	    g_mime_object_get_content_type(body->mime_part);
 	res = g_strdup(g_mime_content_type_get_parameter(type, param));
-    } else {
+    } else if (body->content_type) {
 	GMimeContentType *type =
 	    g_mime_content_type_new_from_string(body->content_type);
 	res = g_strdup(g_mime_content_type_get_parameter(type, param));
@@ -272,8 +276,14 @@ libbalsa_message_body_get_parameter(LibBalsaMessageBody * body,
    allocates a temporary file name and saves the body there.
 */
 gboolean
-libbalsa_message_body_save_temporary(LibBalsaMessageBody * body)
+libbalsa_message_body_save_temporary(LibBalsaMessageBody * body, GError **err)
 {
+    if (!body) {
+        g_set_error(err, LIBBALSA_MAILBOX_ERROR, LIBBALSA_MAILBOX_ACCESS_ERROR,
+                    "NULL message body");
+        return FALSE;
+    }
+
     if (body->temp_filename == NULL) {
 	gint count = 100; /* Magic number, same as in g_mkstemp. */
 
@@ -285,17 +295,17 @@ libbalsa_message_body_save_temporary(LibBalsaMessageBody * body)
 	    gint fd;
 	    gchar *tmp_file_name;
 	    gchar *dotpos = NULL;
-	    GError *err = NULL;
 
 	    fd = g_file_open_tmp("balsa-body-XXXXXX", &tmp_file_name,
-				 &err);
-	    if (err) {
-		printf("libbalsa_message_body_save_temporary:\n %s\n",
-		       err->message);
-		g_error_free(err);
-	    }
+				 err);
 	    if (fd < 0)
 		return FALSE;
+	    if (err && *err  &&  /* We should have returned by now! */
+                (*err)->message) {
+		printf("libbalsa_message_body_save_temporary:\n %s\n",
+		       (*err)->message);
+		g_clear_error(err);
+	    }
 	    close(fd);
 	    unlink(tmp_file_name);
 
@@ -310,9 +320,10 @@ libbalsa_message_body_save_temporary(LibBalsaMessageBody * body)
 		g_free(tmp_file_name);
 	    } else
 		body->temp_filename = tmp_file_name;
-	    fd = open(body->temp_filename, O_WRONLY | O_EXCL | O_CREAT, 0600);
+	    fd = open(body->temp_filename, O_WRONLY | O_EXCL | O_CREAT,
+                      LIBBALSA_MESSAGE_BODY_SAFE);
 	    if (fd >= 0)
-		return libbalsa_message_body_save_fd(body, fd);
+		return libbalsa_message_body_save_fd(body, fd, FALSE, err);
 	} while (errno == EEXIST && --count > 0);
 
 	/* Either we hit a real error, or we used up 100 attempts. */
@@ -327,7 +338,9 @@ libbalsa_message_body_save_temporary(LibBalsaMessageBody * body)
 	    s.st_uid == getuid())
 	    return TRUE;
 	else
-	    return libbalsa_message_body_save(body, body->temp_filename);
+	    return libbalsa_message_body_save(body, body->temp_filename,
+                                              LIBBALSA_MESSAGE_BODY_SAFE,
+                                              FALSE, err);
     }
 }
 
@@ -337,7 +350,9 @@ libbalsa_message_body_save_temporary(LibBalsaMessageBody * body)
 */
 gboolean
 libbalsa_message_body_save(LibBalsaMessageBody * body,
-			   const gchar * filename)
+			   const gchar * filename, mode_t mode,
+                           gboolean filter_crlf,
+                           GError **err)
 {
     int fd;
     int flags = O_CREAT | O_EXCL | O_WRONLY;
@@ -346,113 +361,168 @@ libbalsa_message_body_save(LibBalsaMessageBody * body,
     flags |= O_NOFOLLOW;
 #endif
 
-    if ((fd=libbalsa_safe_open(filename, flags)) < 0)
+    if ((fd=libbalsa_safe_open(filename, flags, mode)) < 0)
 	return FALSE;
-    return libbalsa_message_body_save_fd(body, fd);
+    return libbalsa_message_body_save_fd(body, fd, filter_crlf, err);
 }
 
 static GMimeStream *
-libbalsa_message_body_assure_stream_filter(GMimeStream * stream)
+libbalsa_message_body_stream_add_filter(GMimeStream * stream,
+                                        GMimeFilter * filter)
 {
-    GMimeStream *filtered_stream;
+    if (!GMIME_IS_STREAM_FILTER(stream)) {
+        GMimeStream *filtered_stream =
+            g_mime_stream_filter_new_with_stream(stream);
+        g_object_unref(stream);
+        stream = filtered_stream;
+    }
 
-    if (GMIME_IS_STREAM_FILTER(stream))
-        return stream;
+    g_mime_stream_filter_add(GMIME_STREAM_FILTER(stream), filter);
+    g_object_unref(filter);
 
-    filtered_stream = g_mime_stream_filter_new_with_stream(stream);
-    g_object_unref(stream);
-
-    return filtered_stream;
+    return stream;
 }
 
 GMimeStream *
-libbalsa_message_body_get_stream(LibBalsaMessageBody * body)
+libbalsa_message_body_get_stream(LibBalsaMessageBody * body, GError **err)
 {
     GMimeStream *stream;
     GMimeFilter *filter;
     gchar *mime_type = NULL;
     const gchar *charset;
 
-    if (libbalsa_mailbox_get_message_part(body->message, body)) {
+    g_return_val_if_fail(body != NULL, NULL);
+    g_return_val_if_fail(body->message != NULL, NULL);
+
+    if (body->message->mailbox) {
         GMimeDataWrapper *wrapper;
+        GMimePartEncodingType encoding;
+
+        if(!libbalsa_mailbox_get_message_part(body->message, body, err))
+            return NULL;
 
         wrapper =
             g_mime_part_get_content_object(GMIME_PART(body->mime_part));
         stream = g_mime_data_wrapper_get_stream(wrapper);
+        encoding = g_mime_data_wrapper_get_encoding(wrapper);
+        g_object_unref(wrapper);
 
-        filter = NULL;
-        switch (g_mime_data_wrapper_get_encoding(wrapper)) {
+        switch (encoding) {
         case GMIME_PART_ENCODING_BASE64:
             filter =
-                g_mime_filter_basic_new_type(GMIME_FILTER_BASIC_BASE64_DEC);
+                g_mime_filter_basic_new_type
+                (GMIME_FILTER_BASIC_BASE64_DEC);
+            stream =
+                libbalsa_message_body_stream_add_filter(stream, filter);
             break;
         case GMIME_PART_ENCODING_QUOTEDPRINTABLE:
             filter =
                 g_mime_filter_basic_new_type(GMIME_FILTER_BASIC_QP_DEC);
+            stream =
+                libbalsa_message_body_stream_add_filter(stream, filter);
             break;
         case GMIME_PART_ENCODING_UUENCODE:
             filter =
                 g_mime_filter_basic_new_type(GMIME_FILTER_BASIC_UU_DEC);
+            stream =
+                libbalsa_message_body_stream_add_filter(stream, filter);
             break;
         default:
             break;
         }
-        g_object_unref(wrapper);
-
-        if (filter) {
-	    stream = libbalsa_message_body_assure_stream_filter(stream);
-            g_mime_stream_filter_add(GMIME_STREAM_FILTER(stream), filter);
-            g_object_unref(filter);
-        }
     } else if (body->mime_part) {
         /* Not a GMimePart... */
+        GMimeObject *object = body->mime_part;
+        if (GMIME_IS_MESSAGE_PART(object))
+            object = GMIME_OBJECT(g_mime_message_part_get_message
+                                  ((GMimeMessagePart *) object));
+        else
+            g_object_ref(object);
         stream = g_mime_stream_mem_new();
-        g_mime_object_write_to_stream(body->mime_part, stream);
-    } else
+        libbalsa_mailbox_lock_store(body->message->mailbox);
+        g_mime_object_write_to_stream(object, stream);
+        libbalsa_mailbox_unlock_store(body->message->mailbox);
+        g_object_unref(object);
+    } else {
+        g_set_error(err, LIBBALSA_MAILBOX_ERROR, LIBBALSA_MAILBOX_ACCESS_ERROR,
+                    "Internal error in get_stream");
         return NULL;
+    }
 
     /* convert text bodies but HTML - gtkhtml does conversion on its own. */
     if (libbalsa_message_body_type(body) == LIBBALSA_MESSAGE_BODY_TYPE_TEXT
-	&& strcmp(mime_type = libbalsa_message_body_get_mime_type(body),
-			                  "text/html") != 0
-	&& (charset = libbalsa_message_body_charset(body))
-	&& g_ascii_strcasecmp(charset, "unknown-8bit") != 0
-	&& (filter = g_mime_filter_charset_new(charset, "UTF-8")) != NULL) {
-	stream = libbalsa_message_body_assure_stream_filter(stream);
-	g_mime_stream_filter_add(GMIME_STREAM_FILTER(stream), filter);
-	g_object_unref(filter);
+        && strcmp(mime_type = libbalsa_message_body_get_mime_type(body),
+                  "text/html") != 0
+        && (charset = libbalsa_message_body_charset(body)) != NULL
+        && g_ascii_strcasecmp(charset, "unknown-8bit") != 0) {
+        GMimeStream *stream_null;
+        GMimeStream *stream_filter;
+        GMimeFilter *filter_windows;
+
+        stream_null = g_mime_stream_null_new();
+        stream_filter = g_mime_stream_filter_new_with_stream(stream_null);
+        g_object_unref(stream_null);
+
+        filter_windows = g_mime_filter_windows_new(charset);
+        g_mime_stream_filter_add(GMIME_STREAM_FILTER(stream_filter),
+                                 filter_windows);
+
+        libbalsa_mailbox_lock_store(body->message->mailbox);
+        g_mime_stream_reset(stream);
+        g_mime_stream_write_to_stream(stream, stream_filter);
+        libbalsa_mailbox_unlock_store(body->message->mailbox);
+        g_object_unref(stream_filter);
+
+        charset = g_mime_filter_windows_real_charset(GMIME_FILTER_WINDOWS
+                                                     (filter_windows));
+        if ((filter = g_mime_filter_charset_new(charset, "UTF-8"))) {
+            stream =
+                libbalsa_message_body_stream_add_filter(stream, filter);
+            g_free(body->charset);
+            body->charset = g_strdup(charset);
+        }
+        g_object_unref(filter_windows);
     }
+
     g_free(mime_type);
 
-    g_mime_stream_reset(stream);
     return stream;
 }
 
 gssize
-libbalsa_message_body_get_content(LibBalsaMessageBody * body, gchar ** buf)
+libbalsa_message_body_get_content(LibBalsaMessageBody * body, gchar ** buf,
+                                  GError **err)
 {
     GMimeStream *stream, *stream_mem;
     GByteArray *array;
     gssize len;
 
+    g_return_val_if_fail(body != NULL, -1);
+    g_return_val_if_fail(body->message != NULL, -1);
+    g_return_val_if_fail(buf != NULL, -1);
+
     *buf = NULL;
-    stream = libbalsa_message_body_get_stream(body);
+    stream = libbalsa_message_body_get_stream(body, err);
     if (!stream)
         return -1;
 
     array = g_byte_array_new();
     stream_mem = g_mime_stream_mem_new_with_byte_array(array);
     g_mime_stream_mem_set_owner(GMIME_STREAM_MEM(stream_mem), FALSE);
+
+    libbalsa_mailbox_lock_store(body->message->mailbox);
+    g_mime_stream_reset(stream);
     len = g_mime_stream_write_to_stream(stream, stream_mem);
-    g_object_unref(stream_mem);
+    libbalsa_mailbox_unlock_store(body->message->mailbox);
     g_object_unref(stream);
+    g_object_unref(stream_mem);
 
     if (len >= 0) {
+	guint8 zero = 0;
         len = array->len;
 	/* NULL-terminate, in case it is used as a string. */
-	g_byte_array_append(array, "", 1);
-        *buf = array->data;
-        g_byte_array_free(array, FALSE);
+	g_byte_array_append(array, &zero, 1);
+        *buf = (gchar *) g_byte_array_free(array, FALSE);
     } else
         g_byte_array_free(array, TRUE);
 
@@ -469,20 +539,23 @@ libbalsa_message_body_get_pixbuf(LibBalsaMessageBody * body, GError ** err)
     gchar buf[4096];
     GdkPixbuf *pixbuf = NULL;
 
-    stream = libbalsa_message_body_get_stream(body);
+    stream = libbalsa_message_body_get_stream(body, err);
     if (!stream)
         return pixbuf;
+    libbalsa_mailbox_lock_store(body->message->mailbox);
+    g_mime_stream_reset(stream);
 
     loader = gdk_pixbuf_loader_new();
     while ((count = g_mime_stream_read(stream, buf, sizeof(buf))) > 0) {
-        if (!gdk_pixbuf_loader_write(loader, buf, count, err)) {
+        if (!gdk_pixbuf_loader_write(loader, (guchar *) buf, count, err)) {
             ok = FALSE;
             break;
         }
     }
+    libbalsa_mailbox_unlock_store(body->message->mailbox);
     g_object_unref(stream);
 
-    if (ok && !gdk_pixbuf_loader_close(loader, err))
+    if (!gdk_pixbuf_loader_close(loader, ok ? err : NULL))
 	ok = FALSE;
 
     if (ok) {
@@ -495,20 +568,57 @@ libbalsa_message_body_get_pixbuf(LibBalsaMessageBody * body, GError ** err)
 }
 
 gboolean
-libbalsa_message_body_save_fd(LibBalsaMessageBody * body, int fd)
+libbalsa_message_body_save_fd(LibBalsaMessageBody * body, int fd,
+                              gboolean filter_crlf, GError ** err)
 {
     GMimeStream *stream, *stream_fs;
-    gboolean retval = TRUE;
+    ssize_t len;
+
+    stream = libbalsa_message_body_get_stream(body, err);
+    if (!body->mime_part)
+        return FALSE;
 
-    stream = libbalsa_message_body_get_stream(body);
     stream_fs = g_mime_stream_fs_new(fd);
+    libbalsa_mailbox_lock_store(body->message->mailbox);
 
-    if (g_mime_stream_write_to_stream(stream, stream_fs) < 0)
-	retval = FALSE;
-    g_object_unref(stream);
+    if (stream) {
+        g_mime_stream_reset(stream);
+
+        if (filter_crlf) {
+            GMimeFilter *filter;
+
+            if (!GMIME_IS_STREAM_FILTER(stream)) {
+                GMimeStream *stream_filter =
+                    g_mime_stream_filter_new_with_stream(stream);
+                g_object_unref(stream);
+                stream = stream_filter;
+            }
+
+            filter = g_mime_filter_crlf_new(GMIME_FILTER_CRLF_DECODE,
+                                            GMIME_FILTER_CRLF_MODE_CRLF_ONLY);
+            g_mime_stream_filter_add(GMIME_STREAM_FILTER(stream), filter);
+            g_object_unref(filter);
+        }
+
+        len = g_mime_stream_write_to_stream(stream, stream_fs);
+        g_object_unref(stream);
+    } else {
+        /* body->mime_part is not a GMimePart. */
+        GMimeObject *mime_part = body->mime_part;
+
+        if (GMIME_IS_MESSAGE_PART(mime_part))
+            /* The user probably wants the message without the part's
+             * mime headers. */
+            mime_part =
+                GMIME_OBJECT(GMIME_MESSAGE_PART(mime_part)->message);
+
+        len = g_mime_object_write_to_stream(mime_part, stream_fs);
+    }
+
+    libbalsa_mailbox_unlock_store(body->message->mailbox);
     g_object_unref(stream_fs);
 
-    return retval;
+    return len >= 0;
 }
 
 gchar *
@@ -634,3 +744,28 @@ libbalsa_message_body_get_by_id(LibBalsaMessageBody * body,
 
     return libbalsa_message_body_get_by_id(body->next, id);
 }
+
+
+#ifdef HAVE_GPGME
+LibBalsaMsgProtectState
+libbalsa_message_body_protect_state(LibBalsaMessageBody *body)
+{
+    if (!body || !body->sig_info ||
+	body->sig_info->status == GPG_ERR_NOT_SIGNED ||
+	body->sig_info->status == GPG_ERR_CANCELED)
+	return LIBBALSA_MSG_PROTECT_NONE;
+
+    if (body->sig_info->status == GPG_ERR_NO_ERROR) {
+	/* good signature, check if the validity and trust (OpenPGP only) are
+	   at least marginal */
+	if (body->sig_info->validity >= GPGME_VALIDITY_MARGINAL &&
+	    (body->sig_info->protocol == GPGME_PROTOCOL_CMS ||
+	     body->sig_info->trust >= GPGME_VALIDITY_MARGINAL))
+	    return LIBBALSA_MSG_PROTECT_SIGN_GOOD;
+	else
+	    return LIBBALSA_MSG_PROTECT_SIGN_NOTRUST;
+    }
+    
+    return LIBBALSA_MSG_PROTECT_SIGN_BAD;
+}
+#endif
