@@ -27,6 +27,7 @@
 
 #include <glib.h>
 #include <gmime/gmime.h>
+#include <gdk/gdk.h>
 
 #include "config.h"
 
@@ -48,6 +49,17 @@ enum _LibBalsaMessageBodyType {
     LIBBALSA_MESSAGE_BODY_TYPE_VIDEO
 };
 
+typedef enum _LibBalsaAttachMode LibBalsaAttachMode;
+
+enum _LibBalsaAttachMode {
+    LIBBALSA_ATTACH_AS_ATTACHMENT = 1,
+    LIBBALSA_ATTACH_AS_INLINE,
+    LIBBALSA_ATTACH_AS_EXTBODY
+};
+
+#define LIBBALSA_MESSAGE_BODY_SAFE (S_IRUSR | S_IWUSR)
+#define LIBBALSA_MESSAGE_BODY_UNSAFE \
+    (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH)
 struct _LibBalsaMessageBody {
     LibBalsaMessage *message;	/* The message of which this is a part */
     /* FIXME: remove buffer and buf_len to decrease memory usage. */
@@ -61,7 +73,7 @@ struct _LibBalsaMessageBody {
     const gchar *content_dsp;	/* content-disposition */ 
     const gchar *content_id;    /* content-id */
     gchar *filename;		/* holds filename for attachments and such (used mostly for sending) */
-    gboolean attach_as_extbody; /* if an attachment shall be appended as external-body (sending) */
+    LibBalsaAttachMode attach_mode; /* attachment mode for sending */
     gchar *temp_filename;	/* Holds the filename of a the temporary file where this part is saved */
     gchar *charset;		/* the charset, used for sending, replying. */
     GMimeObject *mime_part;	/* mime body */
@@ -84,16 +96,20 @@ LibBalsaMessageBodyType libbalsa_message_body_type(LibBalsaMessageBody *
 void libbalsa_message_body_set_mime_body(LibBalsaMessageBody * body,
 					 GMimeObject * mime_part);
 
-GMimeStream *libbalsa_message_body_get_stream(LibBalsaMessageBody * body);
+GMimeStream *libbalsa_message_body_get_stream(LibBalsaMessageBody * body,
+                                              GError **err);
 gssize libbalsa_message_body_get_content(LibBalsaMessageBody * body,
-                                         gchar ** buf);
+                                         gchar ** buf, GError **err);
 GdkPixbuf *libbalsa_message_body_get_pixbuf(LibBalsaMessageBody * body,
                                             GError ** err);
 
-gboolean libbalsa_message_body_save_fd(LibBalsaMessageBody * body, int fd);
+gboolean libbalsa_message_body_save_fd(LibBalsaMessageBody * body, int fd,
+                                       gboolean filter_crlf, GError **err);
 gboolean libbalsa_message_body_save(LibBalsaMessageBody * body,
-                                    const gchar * filename);
-gboolean libbalsa_message_body_save_temporary(LibBalsaMessageBody * body);
+                                    const gchar * filename, mode_t mode,
+                                    gboolean filter_crlf, GError **err);
+gboolean libbalsa_message_body_save_temporary(LibBalsaMessageBody * body,
+                                              GError **err);
 
 gchar *libbalsa_message_body_get_parameter(LibBalsaMessageBody * body,
 					   const gchar * param);
@@ -108,5 +124,9 @@ LibBalsaMessageBody *libbalsa_message_body_get_by_id(LibBalsaMessageBody *
                                                      body,
                                                      const gchar * id);
 
+#ifdef HAVE_GPGME
+LibBalsaMsgProtectState libbalsa_message_body_protect_state(LibBalsaMessageBody *body);
+#endif
+
 
 #endif				/* __LIBBALSA_BODY_H__ */
