@@ -29,10 +29,14 @@
  * Primary author: Emmanuel ALLAUD
  */
 
+#include "config.h"
+
 #include <ctype.h>
 #include <string.h>
 
-#include <libgnome/libgnome.h>
+#ifdef HAVE_GNOME
+#include <libgnome/gnome-sound.h>
+#endif
 
 #include "libbalsa.h"
 #include "libbalsa_private.h"
@@ -41,6 +45,7 @@
 #include "filter-funcs.h"
 #include "filter-private.h"
 #include "misc.h"
+#include "i18n.h"
 
 /* from libmutt */
 #define REGCOMP(X,Y,Z) regcomp(X, Y, REG_WORDS|REG_EXTENDED|(Z))
@@ -70,47 +75,6 @@ libbalsa_filters_set_filter_list(GSList** list)
     filter_list = list;
 }
 
-/* FIXME : filtering takes time, we should notify progress to user */
-
-/* in_string_utf8 returns TRUE if s2 is a substring of s1
- * in_string_utf8 is case insensitive
- * this functions understands utf8 strings (as you might have guessed ;-)
- */
-static gboolean
-in_string_utf8(const gchar * s1,const gchar * s2)
-{
-    const gchar * p,* q;
-
-    /* convention : NULL string is contained in anything */
-    if (!s2) return TRUE;
-    /* s2 is non-NULL, so if s1==NULL we return FALSE :)*/
-    if (!s1) return FALSE;
-    /* OK both are non-NULL now*/
-    /* If s2 is the empty string return TRUE */
-    if (!*s2) return TRUE;
-    while (*s1) {
-	/* We look for the first char of s2*/
-	for (;*s1 &&
-		 g_unichar_toupper(g_utf8_get_char(s2))!=g_unichar_toupper(g_utf8_get_char(s1));
-	     s1 = g_utf8_next_char(s1));
-	if (*s1) {
-	    /* We found the first char let see if this potential match is an actual one */
-	    s1 = g_utf8_next_char(s1);
-	    q = s1;
-	    p = g_utf8_next_char(s2);
-	    while (*q && *p && 
-		   g_unichar_toupper(g_utf8_get_char(p))
-		   ==g_unichar_toupper(g_utf8_get_char(q))) {
-		p = g_utf8_next_char(p);
-		q = g_utf8_next_char(q);
-	    }
-	    /* We have a match if p has reached the end of s2, ie *p==0 */
-	    if (!*p) return TRUE;
-	}
-    }
-    return FALSE;
-}
-
 /*------------------------------------------------------------------------
   ---- Helper functions (also exported to have a fine-grained API) -------
 */
@@ -165,60 +129,66 @@ libbalsa_condition_matches(LibBalsaCondition* cond,
                 libbalsa_information(LIBBALSA_INFORMATION_ERROR,
                                      _("Unable to load message body to "
                                        "match filter"));
-                return FALSE;  /* We don't want to match if an error occured */
+                return FALSE;  /* We don't want to match if an error occurred */
             }
         }
         /* do the work */
 	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_TO)) {
-	    str=libbalsa_make_string_from_list(message->headers->to_list);
-	    match=in_string_utf8(str,cond->match.string.string);
+            str =
+                internet_address_list_to_string(message->headers->to_list,
+                                                FALSE);
+	    match=libbalsa_utf8_strstr(str,cond->match.string.string);
 	    g_free(str);
             if(match) break;
 	}
 	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_FROM)
             && message->headers->from) {
-	    str=libbalsa_address_to_gchar(message->headers->from,0);
-	    match=in_string_utf8(str,cond->match.string.string);
+            str =
+                internet_address_list_to_string(message->headers->from,
+                                                FALSE);
+	    match=libbalsa_utf8_strstr(str,cond->match.string.string);
 	    g_free(str);
 	    if (match) break;
 	}
 	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_SUBJECT)) {
-	    if (in_string_utf8(LIBBALSA_MESSAGE_GET_SUBJECT(message),
-                          cond->match.string.string)) { 
+	    if (libbalsa_utf8_strstr(LIBBALSA_MESSAGE_GET_SUBJECT(message),
+                                     cond->match.string.string)) { 
                 match = TRUE;
                 break;
             }
 	}
 	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_CC)) {
-	    str=libbalsa_make_string_from_list(message->headers->cc_list);
-	    match=in_string_utf8(str,cond->match.string.string);
+            str =
+                internet_address_list_to_string(message->headers->cc_list,
+                                                FALSE);
+	    match=libbalsa_utf8_strstr(str,cond->match.string.string);
 	    g_free(str);
 	    if (match) break;
 	}
 	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_US_HEAD)) {
-	    if (cond->match.string.user_header) {
-		GList * header =
-		    libbalsa_message_find_user_hdr(message,
-                                                   cond->match.string
-                                                   .user_header);
+            if (cond->match.string.user_header) {
+                const gchar *header =
+                    libbalsa_message_get_user_header(message,
+                                                     cond->match.string.
+                                                     user_header);
 
-		if (header) {
-		    gchar ** tmp = header->data;
-		    if (in_string_utf8(tmp[1],cond->match.string.string)) {
-			match = TRUE;
-			break;
-		    }
-		}
-	    }
+                if (libbalsa_utf8_strstr(header,
+                                         cond->match.string.string)) {
+                    match = TRUE;
+                    break;
+                }
+            }
 	}
 	if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_BODY)) {
 	    if (!message->mailbox)
-		return FALSE; /* We don't want to match if an error occured */
-	    body = content2reply(message,NULL,0,FALSE,FALSE);
+		return FALSE; /* We don't want to match if an error occurred */
+            body =
+                content2reply(message->body_list, NULL, 0,
+                              FALSE, FALSE, NULL, NULL);
 	    if (body) {
 		if (body->str)
-                    match = in_string_utf8(body->str,
-                                           cond->match.string.string);
+                    match = libbalsa_utf8_strstr(body->str,
+                                                 cond->match.string.string);
 		g_string_free(body,TRUE);
 	    }
 	}
@@ -287,8 +257,10 @@ libbalsa_filter_mailbox_messages(LibBalsaFilter * filt,
     if (msgnos->len == 0)
 	return FALSE;
 
+#ifdef HAVE_GNOME
     if (filt->sound)
 	gnome_sound_play(filt->sound);
+#endif /* HAVE_GNOME */
     if (filt->popup_text)
 	libbalsa_information(LIBBALSA_INFORMATION_MESSAGE,
 			     filt->popup_text);
@@ -422,10 +394,10 @@ libbalsa_condition_is_flag_only(LibBalsaCondition * cond,
         retval =
             libbalsa_condition_is_flag_only(cond->match.andor.left,
                                             mailbox, msgno,
-					    &left_match)
+					    match ? &left_match : NULL)
             && libbalsa_condition_is_flag_only(cond->match.andor.right,
                                                mailbox, msgno,
-                                               &right_match);
+                                               match ? &right_match : NULL);
         if (retval && match)
             *match = left_match && right_match;
         break;
@@ -433,10 +405,10 @@ libbalsa_condition_is_flag_only(LibBalsaCondition * cond,
         retval =
             libbalsa_condition_is_flag_only(cond->match.andor.left,
                                             mailbox, msgno,
-					    &left_match)
+					    match ? &left_match : NULL)
             && libbalsa_condition_is_flag_only(cond->match.andor.right,
                                                mailbox, msgno,
-                                               &right_match);
+                                               match ? &right_match : NULL);
         if (retval && match)
             *match = left_match || right_match;
         break;
