@@ -114,6 +114,7 @@ libbalsa_identity_init(LibBalsaIdentity* ident)
     ident->always_trust = FALSE;
     ident->crypt_protocol = LIBBALSA_PROTECT_OPENPGP;
 #endif
+    ident->request_mdn = FALSE;
     /*
     ident->face = NULL;
     ident->x_face = NULL;
@@ -305,6 +306,72 @@ libbalsa_identity_set_sig_prepend(LibBalsaIdentity* ident, gboolean prepend)
     ident->sig_prepend = prepend;
 }
 
+/** Returns a signature for given identity, adding a signature prefix
+    if needed. parent can be NULL. */
+gchar*
+libbalsa_identity_get_signature(LibBalsaIdentity* identity, GtkWindow *parent)
+{
+    FILE *fp = NULL;
+    size_t len = 0;
+    gchar *ret = NULL, *path;
+
+    if (identity->signature_path == NULL ||
+        *identity->signature_path == '\0')
+	return NULL;
+
+    path = libbalsa_expand_path(identity->signature_path);
+    if(identity->sig_executable){
+        /* signature is executable */
+	fp = popen(path,"r");
+        if (fp) {
+            len = libbalsa_readfile_nostat(fp, &ret);
+            pclose(fp);
+        } else
+            libbalsa_information_parented
+                (parent, LIBBALSA_INFORMATION_ERROR,
+                 _("Error executing signature generator %s"),
+                 identity->signature_path);
+    } else {
+        /* sign is normal file */
+        fp = fopen(path, "r");
+        if (fp) {
+            len = libbalsa_readfile_nostat(fp, &ret);
+            fclose(fp);
+        } else
+            libbalsa_information_parented(parent, LIBBALSA_INFORMATION_ERROR,
+                                          _("Cannot open signature file '%s' "
+                                            "for reading"),
+                                          identity->signature_path);
+    }
+    if(!ret)
+        libbalsa_information_parented(parent, LIBBALSA_INFORMATION_ERROR,
+                                   _("Error reading signature from %s"), path);
+    else {
+        if(!libbalsa_utf8_sanitize(&ret, FALSE, NULL))
+            libbalsa_information_parented
+                (parent, LIBBALSA_INFORMATION_ERROR,
+                 _("Signature in %s is not a UTF-8 text."),
+                 identity->signature_path);
+    }
+    g_free(path);
+
+    if(ret == NULL) return NULL;
+
+    /* Prepend the separator if needed... */
+
+    if (identity->sig_separator
+        && strncmp(ret, "--\n", 3)
+        && strncmp(ret, "-- \n", 4)) {
+        gchar *sig_tmp = g_strconcat("\n-- \n", ret, NULL);
+        g_free(ret);
+        ret = sig_tmp;
+    } else {
+        gchar *sig_tmp = g_strconcat("\n", ret, NULL);
+        g_free(ret);
+        ret = sig_tmp;
+    }
+    return ret;
+}
 
 #if ENABLE_ESMTP
 void
@@ -511,10 +578,11 @@ static void config_frame_button_select_cb(GtkTreeSelection * selection,
 static void ident_dialog_add_checkbutton(GtkWidget *, gint, GtkDialog *,
                                          const gchar *, const gchar *,
 					 gboolean sensitive);
+static void ident_dialog_add_check_and_entry(GtkWidget *, gint, GtkDialog *,
+                                             const gchar *, const gchar *);
 static void ident_dialog_add_entry(GtkWidget *, gint, GtkDialog *,
                                    const gchar *, const gchar *);
 typedef enum LibBalsaIdentityPathType_ {
-    LBI_PATH_TYPE_SIG,
     LBI_PATH_TYPE_FACE,
     LBI_PATH_TYPE_XFACE
 } LibBalsaIdentityPathType;
@@ -541,7 +609,7 @@ static void display_frame_set_field(GObject * dialog, const gchar* key,
 static void display_frame_set_boolean(GObject * dialog, const gchar* key, 
                                       gboolean value);
 static void display_frame_set_path(GObject * dialog, const gchar * key,
-                                   const gchar * value);
+                                   const gchar * value, gboolean use_chooser);
 
 
 static void identity_list_update(GtkTreeView * tree);
@@ -859,11 +927,6 @@ struct {
     const gchar *basename;
     const gchar *info;
 } static const path_info[] = {
-    {N_("Signature _Path"),
-     "identity-sigpath",
-     NULL,
-     ".signature",
-     N_("Signature")},
         /* Translators: please do not translate Face. */
     {N_("_Face Path"),
      "identity-facepath",
@@ -878,6 +941,9 @@ struct {
      "X-Face"}
 };
 
+#define LIBBALSA_IDENTITY_CHECK "libbalsa-identity-check"
+static void md_sig_path_changed_cb(GtkWidget *sig_button, GObject *dialog);
+
 #if ENABLE_ESMTP
 static GtkWidget*
 setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree,
@@ -891,9 +957,10 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
     GtkWidget *table;
     gint row;
     GObject *name;
+    gpointer path;
 
     /* create the "General" tab */
-    table = append_ident_notebook_page(notebook, 8, _("General"));
+    table = append_ident_notebook_page(notebook, 5, _("General"));
     row = 0;
     ident_dialog_add_entry(table, row++, dialog, _("_Identity Name:"), 
 		           "identity-name");
@@ -905,12 +972,26 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
                            "identity-replyto");
     ident_dialog_add_entry(table, row++, dialog, _("_Domain:"), 
                            "identity-domain");
+
+    /* create the "Messages" tab */
+    table = append_ident_notebook_page(notebook, 8, _("Messages"));
+    row = 0;
     ident_dialog_add_entry(table, row++, dialog, _("_Bcc:"), 
                            "identity-bcc");
     ident_dialog_add_entry(table, row++, dialog, _("Reply _String:"), 
                            "identity-replystring");
     ident_dialog_add_entry(table, row++, dialog, _("F_orward String:"), 
                            "identity-forwardstring");
+    ident_dialog_add_checkbutton(table, row++, dialog, 
+                                 _("request _Message Disposition Notification by default"),
+                                 "identity-requestmdn", TRUE);
+    ident_dialog_add_file_chooser_button(table, row++, dialog,
+                                         LBI_PATH_TYPE_FACE);
+    ident_dialog_add_file_chooser_button(table, row++, dialog,
+                                         LBI_PATH_TYPE_XFACE);
+    ident_dialog_add_boxes(table, row++, dialog,
+                           path_info[LBI_PATH_TYPE_FACE].box_key,
+                           path_info[LBI_PATH_TYPE_XFACE].box_key);
 #if ENABLE_ESMTP
     ident_dialog_add_smtp_menu(table, row++, dialog, _("SMT_P Server:"),
                                "identity-smtp-server", smtp_servers);
@@ -919,8 +1000,9 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
     /* create the "Signature" tab */
     table = append_ident_notebook_page(notebook, 7, _("Signature"));
     row = 0;
-    ident_dialog_add_file_chooser_button(table, row++, dialog,
-                                         LBI_PATH_TYPE_SIG);
+    ident_dialog_add_check_and_entry(table, row++, dialog,
+                                     _("Signature _Path"),
+                                     "identity-sigpath");
     ident_dialog_add_checkbutton(table, row++, dialog,
                                 _("_Execute Signature"),
 				 "identity-sigexecutable", FALSE);
@@ -939,13 +1021,6 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
     ident_dialog_add_checkbutton(table, row++, dialog,
                                  _("Prepend Si_gnature"),
                                  "identity-sigprepend", FALSE);
-    ident_dialog_add_file_chooser_button(table, row++, dialog,
-                                         LBI_PATH_TYPE_FACE);
-    ident_dialog_add_file_chooser_button(table, row++, dialog,
-                                         LBI_PATH_TYPE_XFACE);
-    ident_dialog_add_boxes(table, row++, dialog,
-                           path_info[LBI_PATH_TYPE_FACE].box_key,
-                           path_info[LBI_PATH_TYPE_XFACE].box_key);
 
 #ifdef HAVE_GPGME
     /* create the "Security" tab */
@@ -969,6 +1044,12 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
     g_signal_connect(name, "changed",
                      G_CALLBACK(md_name_changed), tree);
 
+    path = g_object_get_data(G_OBJECT(dialog), "identity-sigpath");
+    g_signal_connect(g_object_get_data(G_OBJECT(path),
+                                       LIBBALSA_IDENTITY_CHECK),
+                     "toggled",
+                     G_CALLBACK(md_sig_path_changed_cb), dialog);
+
     gtk_notebook_set_current_page(notebook, 0);
 
     return GTK_WIDGET(notebook);
@@ -1000,6 +1081,32 @@ ident_dialog_add_checkbutton(GtkWidget * table, gint row,
     gtk_widget_set_sensitive(check, sensitive);
 }
 
+/*
+ * Create and add a GtkCheckButton to the given dialog with caption
+ * and add a pointer to it stored under the given key, which is
+ * followed by a text entry.  The check button is initialized to the
+ * given value.
+ */
+static void
+ident_dialog_add_check_and_entry(GtkWidget * table, gint row,
+                                 GtkDialog * dialog, const gchar * check_label,
+                                 const gchar * entry_key)
+{
+    GtkWidget *check, *entry;
+
+    check = gtk_check_button_new_with_mnemonic(check_label);
+    entry = gtk_entry_new();
+
+
+    gtk_table_attach(GTK_TABLE(table), check, 0, 1, row, row + 1,
+                     GTK_FILL, 0, 0, 0);
+    gtk_table_attach(GTK_TABLE(table), entry, 1, 2, row, row + 1,
+                     GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+
+    g_object_set_data(G_OBJECT(dialog), entry_key, entry);
+    g_object_set_data(G_OBJECT(entry), LIBBALSA_IDENTITY_CHECK, check);
+}
+
 
 /*
  * Add a GtkEntry to the given dialog with a label next to it
@@ -1078,7 +1185,7 @@ md_face_path_changed(const gchar * filename, gboolean active,
         libbalsa_information(LIBBALSA_INFORMATION_WARNING,
                 /* Translators: please do not translate Face. */
                              _("Face header file %s is too long "
-                               "(%d bytes)."), filename, size);
+                               "(%lu bytes)."), filename, (unsigned long)size);
         g_free(content);
         gtk_widget_hide(face_box);
         return;
@@ -1124,10 +1231,8 @@ md_face_path_changed(const gchar * filename, gboolean active,
 /* Callback for the "selection-changed" signal of the signature path
  * file chooser; sets sensitivity of the signature-related buttons. */
 
-#define LIBBALSA_IDENTITY_CHECK "libbalsa-identity-check"
 static void
-md_sig_path_changed(const gchar * filename, gboolean active,
-                    GObject * dialog)
+md_sig_path_changed(gboolean active, GObject * dialog)
 {
     guint i;
     static gchar *button_key[] = {
@@ -1137,15 +1242,22 @@ md_sig_path_changed(const gchar * filename, gboolean active,
         "identity-whenreply",
         "identity-sigseparator",
         "identity-sigprepend",
+        "identity-sigpath",
     };
 
-
     for (i = 0; i < ELEMENTS(button_key); i++) {
         GtkWidget *button = g_object_get_data(dialog, button_key[i]);
         gtk_widget_set_sensitive(button, active);
     }
 }
 
+static void
+md_sig_path_changed_cb(GtkWidget *sig_button, GObject *dialog)
+{
+    md_sig_path_changed(GTK_TOGGLE_BUTTON(sig_button)->active,
+                        dialog);
+}
+
 #define LIBBALSA_IDENTITY_INFO "libbalsa-identity-info"
 static void
 file_chooser_cb(GtkWidget * chooser, gpointer data)
@@ -1170,9 +1282,11 @@ file_chooser_cb(GtkWidget * chooser, gpointer data)
     check = g_object_get_data(G_OBJECT(chooser), LIBBALSA_IDENTITY_CHECK);
     active = gtk_toggle_button_get_active(check);
 
+#if 0
     if (type == LBI_PATH_TYPE_SIG)
         md_sig_path_changed(filename, active, data);
     else
+#endif
         md_face_path_changed(filename, active, type, data);
     g_free(filename);
 }
@@ -1337,13 +1451,13 @@ ident_dialog_update(GObject * dlg)
     g_free(id->forward_string);
     id->forward_string  = ident_dialog_get_text(dlg, "identity-forwardstring");
 #if ENABLE_ESMTP
-    g_object_unref(id->smtp_server);
+    if(id->smtp_server) g_object_unref(id->smtp_server);
     id->smtp_server = ident_dialog_get_value(dlg, "identity-smtp-server");
     g_object_ref(id->smtp_server);
 #endif /* ENABLE_ESMTP */
 
     g_free(id->signature_path);
-    id->signature_path  = ident_dialog_get_path(dlg, "identity-sigpath");
+    id->signature_path  = ident_dialog_get_text(dlg, "identity-sigpath");
     
     id->sig_executable  = ident_dialog_get_bool(dlg, "identity-sigexecutable");
     id->sig_sending     = ident_dialog_get_bool(dlg, "identity-sigappend");
@@ -1356,7 +1470,8 @@ ident_dialog_update(GObject * dlg)
     id->face            = ident_dialog_get_path(dlg, "identity-facepath");
     g_free(id->x_face);
     id->x_face          = ident_dialog_get_path(dlg, "identity-xfacepath");
-    
+    id->request_mdn     = ident_dialog_get_bool(dlg, "identity-requestmdn");
+
 #ifdef HAVE_GPGME
     id->gpg_sign        = ident_dialog_get_bool(dlg, "identity-gpgsign");
     id->gpg_encrypt     = ident_dialog_get_bool(dlg, "identity-gpgencrypt");
@@ -1377,8 +1492,12 @@ static gchar*
 ident_dialog_get_text(GObject * dialog, const gchar * key)
 {
     GtkEditable *entry;
+    GtkToggleButton *check;
     
     entry = g_object_get_data(dialog, key);
+    check = g_object_get_data(G_OBJECT(entry), LIBBALSA_IDENTITY_CHECK);
+    if (check && !gtk_toggle_button_get_active(check))
+        return NULL;
     return gtk_editable_get_chars(entry, 0, -1);
 }
 
@@ -1702,7 +1821,8 @@ display_frame_update(GObject * dialog, LibBalsaIdentity* ident)
                              ident->smtp_server);
 #endif /* ENABLE_ESMTP */
 
-    display_frame_set_path(dialog, "identity-sigpath", ident->signature_path);
+    display_frame_set_path(dialog, "identity-sigpath", 
+                           ident->signature_path, FALSE);
     display_frame_set_boolean(dialog, "identity-sigexecutable", ident->sig_executable);
 
     display_frame_set_boolean(dialog, "identity-sigappend", ident->sig_sending);
@@ -1719,13 +1839,15 @@ display_frame_update(GObject * dialog, LibBalsaIdentity* ident)
                                  path_info[LBI_PATH_TYPE_FACE].box_key);
     gtk_widget_hide(face_box);
     display_frame_set_path(dialog, path_info[LBI_PATH_TYPE_FACE].path_key,
-                           ident->face);
+                           ident->face, TRUE);
 
     face_box = g_object_get_data(G_OBJECT(dialog),
                                  path_info[LBI_PATH_TYPE_XFACE].box_key);
     gtk_widget_hide(face_box);
     display_frame_set_path(dialog, path_info[LBI_PATH_TYPE_XFACE].path_key,
-                           ident->x_face);
+                           ident->x_face, TRUE);
+    display_frame_set_boolean(dialog, "identity-requestmdn", 
+                              ident->request_mdn);    
 
 #ifdef HAVE_GPGME
     display_frame_set_boolean(dialog, "identity-gpgsign", 
@@ -1763,19 +1885,19 @@ display_frame_set_boolean(GObject * dialog,
 static void
 display_frame_set_path(GObject * dialog,
                        const gchar* key,
-                       const gchar* value)
+                       const gchar* value, gboolean use_chooser)
 {
     gboolean set = (value && *value);
     GtkWidget *chooser = g_object_get_data(dialog, key);
     GtkToggleButton *check =
         g_object_get_data(G_OBJECT(chooser), LIBBALSA_IDENTITY_CHECK);
     
-    if (set)
-#if GTK_CHECK_VERSION(2, 6, 0)
-        gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(chooser), value);
-#else                           /* GTK_CHECK_VERSION(2, 6, 0) */
-        gtk_entry_set_text(GTK_ENTRY(chooser), value);
-#endif                          /* GTK_CHECK_VERSION(2, 6, 0) */
+    if (set) {
+        if(use_chooser)
+            gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(chooser), value);
+        else
+            gtk_entry_set_text(GTK_ENTRY(chooser), value);
+    }
     gtk_widget_set_sensitive(GTK_WIDGET(chooser), set);
     gtk_toggle_button_set_active(check, set);
 }
@@ -1828,6 +1950,7 @@ libbalsa_identity_new_config(const gchar* name)
     ident->sig_prepend = libbalsa_conf_get_bool("SigPrepend");
     ident->face = libbalsa_conf_get_string("FacePath");
     ident->x_face = libbalsa_conf_get_string("XFacePath");
+    ident->request_mdn = libbalsa_conf_get_bool("RequestMDN");
 
 #ifdef HAVE_GPGME
     ident->gpg_sign = libbalsa_conf_get_bool("GpgSign");
@@ -1872,6 +1995,7 @@ libbalsa_identity_save(LibBalsaIdentity* ident, const gchar* group)
         libbalsa_conf_set_string("FacePath", ident->face);
     if (ident->x_face)
         libbalsa_conf_set_string("XFacePath", ident->x_face);
+    libbalsa_conf_set_bool("RequestMDN", ident->request_mdn);
 
 #ifdef HAVE_GPGME
     libbalsa_conf_set_bool("GpgSign", ident->gpg_sign);
