@@ -41,8 +41,6 @@ typedef struct _LibBalsaCondition LibBalsaCondition;
 
 #include "information.h"
 
-#include "server.h"
-
 #include "address-book.h"
 #include "address-book-vcard.h"
 #include "address-book-ldif.h"
@@ -67,7 +65,7 @@ typedef struct _LibBalsaCondition LibBalsaCondition;
 #define ELEMENTS(x) (sizeof (x) / sizeof (x[0]))
 
 /*
- * Error domains for GError: only one for now, more to come.
+ * Error domains for GError:
  */
 enum {
     LIBBALSA_SCANNER_ERROR,
@@ -75,7 +73,7 @@ enum {
 };
 
 /*
- * Error codes for GError: only one for now, more to come.
+ * Error codes for GError:
  */
 enum {
     LIBBALSA_SCANNER_ERROR_IMAP,
@@ -89,7 +87,8 @@ enum {
     LIBBALSA_MAILBOX_NETWORK_ERROR,
     LIBBALSA_MAILBOX_OPEN_ERROR,
     LIBBALSA_MAILBOX_TOOMANYOPEN_ERROR,
-    LIBBALSA_MAILBOX_ACCESS_ERROR
+    LIBBALSA_MAILBOX_ACCESS_ERROR,
+    LIBBALSA_MAILBOX_DUPLICATES_ERROR
 };
 
 
@@ -115,14 +114,7 @@ gchar *libbalsa_guess_ldap_server(void);
 
 gchar *libbalsa_guess_imap_inbox(void);
 
-gchar *libbalsa_guess_ldap_base(void);
-gchar *libbalsa_guess_ldap_name(void);
 
-gchar *libbalsa_guess_ldif_file(void);
-
-gboolean libbalsa_ldap_exists(const gchar *server);
-
-void libbalsa_assure_balsa_dir(void);
 gchar* libbalsa_date_to_utf8(const time_t *date, const gchar *date_string);
 LibBalsaMessageStatus libbalsa_get_icon_from_flags(LibBalsaMessageFlag flags);
 
@@ -132,6 +124,8 @@ gboolean libbalsa_is_cert_known(X509* cert, long vfy_result);
 void libbalsa_certs_destroy(void);
 #endif
 
+gboolean libbalsa_abort_on_timeout(const char *host);
+
 #ifdef BALSA_USE_THREADS
 #include <pthread.h>
 pthread_t libbalsa_get_main_thread(void);
@@ -143,7 +137,11 @@ gboolean libbalsa_threads_has_lock(void);
 #define libbalsa_am_i_subthread() FALSE
 #define libbalsa_threads_has_lock() TRUE
 #endif /* BALSA_USE_THREADS */
-void libbalsa_message(const char *fmt, ...);
+void libbalsa_message(const char *fmt, ...)
+#ifdef __GNUC__
+    __attribute__ ((format (printf, 1, 2)))
+#endif
+;
 gchar * libbalsa_rot(const gchar * pass);
 
 typedef enum {
@@ -163,4 +161,33 @@ extern void (*libbalsa_progress_set_text) (LibBalsaProgress * progress,
 extern void (*libbalsa_progress_set_fraction) (LibBalsaProgress * progress,
                                                gdouble fraction);
 
+/*
+ * Face and X-Face header support.
+ */
+gchar *libbalsa_get_header_from_path(const gchar * header,
+                                     const gchar * path, gsize * size,
+                                     GError ** err);
+GtkWidget *libbalsa_get_image_from_face_header(const gchar * content,
+                                               GError ** err);
+#if HAVE_COMPFACE
+GtkWidget *libbalsa_get_image_from_x_face_header(const gchar * content,
+                                                 GError ** err);
+#endif                          /* HAVE_COMPFACE */
+
+GQuark libbalsa_image_error_quark(void);
+#define LIBBALSA_IMAGE_ERROR libbalsa_image_error_quark()
+enum LibBalsaImageError {
+    LIBBALSA_IMAGE_ERROR_NO_DATA
+#if HAVE_COMPFACE
+        ,
+    LIBBALSA_IMAGE_ERROR_FORMAT,
+    LIBBALSA_IMAGE_ERROR_BUFFER,
+    LIBBALSA_IMAGE_ERROR_BAD_DATA
+#endif                          /* HAVE_COMPFACE */
+};
+
+#if HAVE_GTKSOURCEVIEW
+GtkWidget *libbalsa_source_view_new(gboolean highlight_phrases, GdkColor *q_colour);
+#endif                          /* HAVE_GTKSOURCEVIEW */
+
 #endif                          /* __LIBBALSA_H__ */
