@@ -47,19 +47,20 @@
 #include "i18n.h"
 
 struct message_info {
+    LibBalsaMailboxLocalMessageInfo local_info;
+    LibBalsaMessageFlag orig_flags;     /* Has only real flags */
     off_t start;
     off_t status;		/* Offset of the "Status:" header. */
     off_t x_status;		/* Offset of the "X-Status:" header. */
     off_t mime_version;		/* Offset of the "MIME-Version:" header. */
     off_t end;
     size_t from_len;
-    LibBalsaMessage *message; /* registers only referenced messages
-			       * to avoid having two objects refering 
-			       * to a single physical message. */
-    LibBalsaMessageFlag flags;
-    LibBalsaMessageFlag orig_flags;
 };
 
+#define REAL_FLAGS(flags) ((flags) & LIBBALSA_MESSAGE_FLAGS_REAL)
+#define FLAGS_REALLY_DIFFER(orig_flags, flags) \
+    ((((orig_flags) ^ (flags)) & LIBBALSA_MESSAGE_FLAGS_REAL) != 0)
+
 static LibBalsaMailboxLocalClass *parent_class = NULL;
 
 static void libbalsa_mailbox_mbox_class_init(LibBalsaMailboxMboxClass *klass);
@@ -78,11 +79,11 @@ static void libbalsa_mailbox_mbox_close_mailbox(LibBalsaMailbox * mailbox,
 static void libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox);
 static gboolean libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox,
                                            gboolean expunge);
-static LibBalsaMessage *lbm_mbox_get_message(LibBalsaMailbox * mailbox,
-                                             guint msgno);
-static LibBalsaMessageFlag lbm_mbox_load_message(LibBalsaMailboxLocal * local,
-                                                 guint msgno,
-                                                 LibBalsaMessage **msg);
+
+/* LibBalsaMailboxLocal class methods */
+static LibBalsaMailboxLocalMessageInfo
+    *lbm_mbox_get_info(LibBalsaMailboxLocal * local, guint msgno);
+
 static gboolean
 libbalsa_mailbox_mbox_fetch_message_structure(LibBalsaMailbox * mailbox,
                                               LibBalsaMessage * message,
@@ -92,16 +93,6 @@ static gboolean libbalsa_mailbox_mbox_add_message(LibBalsaMailbox *
                                                   GMimeStream * stream,
                                                   LibBalsaMessageFlag
                                                   flags, GError ** err);
-static gboolean
-libbalsa_mailbox_mbox_messages_change_flags(LibBalsaMailbox * mailbox,
-                                            GArray * msgnos,
-                                            LibBalsaMessageFlag set,
-                                            LibBalsaMessageFlag clear);
-static gboolean
-libbalsa_mailbox_mbox_msgno_has_flags(LibBalsaMailbox * mailbox,
-                                      guint msgno,
-                                      LibBalsaMessageFlag set,
-                                      LibBalsaMessageFlag unset);
 static guint
 libbalsa_mailbox_mbox_total_messages(LibBalsaMailbox * mailbox);
 #if BALSA_USE_THREADS
@@ -116,7 +107,7 @@ struct _LibBalsaMailboxMboxClass {
 struct _LibBalsaMailboxMbox {
     LibBalsaMailboxLocal parent;
 
-    GArray* messages_info;
+    GPtrArray *msgno_2_msg_info;
     GMimeStream *gmime_stream;
     gint size;
     gboolean messages_info_changed;
@@ -170,14 +161,9 @@ libbalsa_mailbox_mbox_class_init(LibBalsaMailboxMboxClass * klass)
     libbalsa_mailbox_class->sync = libbalsa_mailbox_mbox_sync;
     libbalsa_mailbox_class->close_mailbox =
 	libbalsa_mailbox_mbox_close_mailbox;
-    libbalsa_mailbox_class->get_message = lbm_mbox_get_message;
     libbalsa_mailbox_class->fetch_message_structure =
 	libbalsa_mailbox_mbox_fetch_message_structure;
     libbalsa_mailbox_class->add_message = libbalsa_mailbox_mbox_add_message;
-    libbalsa_mailbox_class->messages_change_flags =
-	libbalsa_mailbox_mbox_messages_change_flags;
-    libbalsa_mailbox_class->msgno_has_flags =
-	libbalsa_mailbox_mbox_msgno_has_flags;
     libbalsa_mailbox_class->total_messages =
 	libbalsa_mailbox_mbox_total_messages;
 #if BALSA_USE_THREADS
@@ -188,7 +174,7 @@ libbalsa_mailbox_mbox_class_init(LibBalsaMailboxMboxClass * klass)
     libbalsa_mailbox_local_class->remove_files = 
 	libbalsa_mailbox_mbox_remove_files;
 
-    libbalsa_mailbox_local_class->load_message = lbm_mbox_load_message;
+    libbalsa_mailbox_local_class->get_info = lbm_mbox_get_info;
 }
 
 
@@ -255,7 +241,7 @@ lbm_mbox_stream_seek_to_message(GMimeStream * stream, off_t offset)
     gboolean retval;
 
     retval = g_mime_stream_seek(stream, offset, GMIME_STREAM_SEEK_SET) >= 0
-        && g_mime_stream_read(stream, buffer, sizeof(buffer)) >= 0
+        && g_mime_stream_read(stream, buffer, sizeof buffer) == sizeof buffer
         && strncmp("From ", buffer, 5) == 0;
 
     g_mime_stream_seek(stream, offset, GMIME_STREAM_SEEK_SET);
@@ -275,28 +261,34 @@ lbm_mbox_seek_to_message(LibBalsaMailboxMbox * mbox, off_t offset)
     return retval;
 }
 
+static struct message_info *
+message_info_from_msgno(LibBalsaMailboxMbox * mbox, guint msgno)
+{
+    g_assert(msgno > 0 && msgno <= mbox->msgno_2_msg_info->len);
+
+    return (struct message_info *) g_ptr_array_index(mbox->
+                                                     msgno_2_msg_info,
+                                                     msgno - 1);
+}
+
 static GMimeStream *
-libbalsa_mailbox_mbox_get_message_stream(LibBalsaMailbox *mailbox,
+libbalsa_mailbox_mbox_get_message_stream(LibBalsaMailbox * mailbox,
                                          guint msgno)
 {
-	GMimeStream *stream = NULL;
-	struct message_info *msg_info;
-	LibBalsaMailboxMbox *mbox;
+    LibBalsaMailboxMbox *mbox;
+    struct message_info *msg_info;
 
-	g_return_val_if_fail (LIBBALSA_IS_MAILBOX_MBOX(mailbox), NULL);
+    g_return_val_if_fail(LIBBALSA_IS_MAILBOX_MBOX(mailbox), NULL);
 
-	mbox = LIBBALSA_MAILBOX_MBOX(mailbox);
+    mbox = LIBBALSA_MAILBOX_MBOX(mailbox);
+    msg_info = message_info_from_msgno(mbox, msgno);
 
-	msg_info = &g_array_index(mbox->messages_info,
-				  struct message_info, msgno - 1);
-	if (!lbm_mbox_seek_to_message(mbox, msg_info->start))
-	    return NULL;
-	stream = g_mime_stream_substream(mbox->gmime_stream,
-					 msg_info->start
-					 + msg_info->from_len,
-					 msg_info->end);
+    if (!msg_info || !lbm_mbox_seek_to_message(mbox, msg_info->start))
+        return NULL;
 
-	return stream;
+    return g_mime_stream_substream(mbox->gmime_stream,
+                                   msg_info->start + msg_info->from_len,
+                                   msg_info->end);
 }
 
 static void
@@ -388,10 +380,25 @@ lbm_mbox_save(LibBalsaMailboxMbox * mbox)
 
     filename = lbm_mbox_get_cache_filename(mbox);
 
-    if (mbox->messages_info->len > 0) {
+    if (mbox->msgno_2_msg_info->len > 0) {
+        GArray *messages_info =
+            g_array_sized_new(FALSE, FALSE, sizeof(struct message_info), 
+                              mbox->msgno_2_msg_info->len);
+        guint msgno;
+#if !GLIB_CHECK_VERSION(2, 8, 0) || defined(__APPLE__)
+        gchar *template;
+        gint fd;
+#endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
+
+        for (msgno = 1; msgno <= mbox->msgno_2_msg_info->len; msgno++) {
+            struct message_info *msg_info =
+                message_info_from_msgno(mbox, msgno);
+            g_array_append_val(messages_info, *msg_info);
+        }
+
 #if GLIB_CHECK_VERSION(2, 8, 0) && !defined(__APPLE__)
-        if (!g_file_set_contents(filename, mbox->messages_info->data,
-                                 mbox->messages_info->len
+        if (!g_file_set_contents(filename, messages_info->data,
+                                 messages_info->len
                                  * sizeof(struct message_info), &err)) {
             libbalsa_information(LIBBALSA_INFORMATION_WARNING,
                                  _("Could not write file %s: %s"),
@@ -399,21 +406,19 @@ lbm_mbox_save(LibBalsaMailboxMbox * mbox)
             g_error_free(err);
         }
 #else                           /* GLIB_CHECK_VERSION(2, 8, 0) */
-        gchar *template;
-        gint fd;
-
         template = g_strconcat(filename, ":XXXXXX", NULL);
         fd = g_mkstemp(template);
-        if (fd < 0 || write(fd, mbox->messages_info->data,
-                            mbox->messages_info->len *
+        if (fd < 0 || write(fd, messages_info->data,
+                            messages_info->len *
                             sizeof(struct message_info)) <
-            (ssize_t) mbox->messages_info->len) {
+            (ssize_t) messages_info->len) {
             libbalsa_information(LIBBALSA_INFORMATION_WARNING,
                                  _("Failed to create temporary file "
                                    "\"%s\": %s"), template,
                                  strerror(errno));
             g_free(template);
             g_free(filename);
+            g_array_free(messages_info, TRUE);
             return;
         }
         if (close(fd) != 0
@@ -425,6 +430,7 @@ lbm_mbox_save(LibBalsaMailboxMbox * mbox)
                                  filename, strerror(errno), template);
         g_free(template);
 #endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
+        g_array_free(messages_info, TRUE);
     } else if (unlink(filename) < 0)
         libbalsa_information(LIBBALSA_INFORMATION_WARNING,
                              _("Could not unlink file %s: %s"),
@@ -434,7 +440,7 @@ lbm_mbox_save(LibBalsaMailboxMbox * mbox)
 #ifdef DEBUG
     g_print("%s:    %s    saved %d messages\n", __func__, 
             LIBBALSA_MAILBOX(mbox)->name,
-            mbox->messages_info->len);
+            mbox->msgno_2_msg_info->len);
 #endif
 }
 
@@ -448,7 +454,7 @@ parse_mailbox(LibBalsaMailboxMbox * mbox)
     GMimeParser *gmime_parser;
     struct message_info msg_info;
     struct message_info * msg_info_p = &msg_info;
-    unsigned msgno = mbox->messages_info->len;
+    unsigned msgno = mbox->msgno_2_msg_info->len;
 
     gmime_parser = g_mime_parser_new_with_stream(mbox->gmime_stream);
     g_mime_parser_set_scan_from(gmime_parser, TRUE);
@@ -459,14 +465,17 @@ parse_mailbox(LibBalsaMailboxMbox * mbox)
 				   lbm_mbox_header_cb, &msg_info_p);
 
     libbalsa_mailbox_local_set_threading_info(local);
-    msg_info.message = NULL;
+    msg_info.local_info.message = NULL;
     while (!g_mime_parser_eos(gmime_parser)) {
 	GMimeMessage *mime_message;
         LibBalsaMessage *msg;
         gchar *from;
+        off_t offset;
 
         msg_info.status = msg_info.x_status = msg_info.mime_version = -1;
         mime_message   = g_mime_parser_construct_message(gmime_parser);
+        if (!mime_message)
+            continue;
         msg_info.start = g_mime_parser_get_from_offset(gmime_parser);
         msg_info.end   = g_mime_parser_tell(gmime_parser);
         if (msg_info.end <= msg_info.start
@@ -493,18 +502,26 @@ parse_mailbox(LibBalsaMailboxMbox * mbox)
         if (!msg)
             continue;
 
-        msg_info.flags = msg_info.orig_flags;
-        g_array_append_val(mbox->messages_info, msg_info);
+        msg_info.local_info.flags = msg_info.orig_flags;
+        g_ptr_array_add(mbox->msgno_2_msg_info,
+                        g_memdup(&msg_info, sizeof(msg_info)));
         mbox->messages_info_changed = TRUE;
 
         msg->flags = msg_info.orig_flags;
         msg->length = msg_info.end - (msg_info.start + msg_info.from_len);
         msg->mailbox = LIBBALSA_MAILBOX(mbox);
         msg->msgno = ++msgno;
-        g_ptr_array_add(LIBBALSA_MAILBOX(mbox)->mindex, 
-                        libbalsa_mailbox_index_entry_new_from_msg(msg));
-
+	/* We must drop the mime-stream lock to call
+	 * libbalsa_mailbox_cache_message(), as it may grab the
+	 * gdk lock to emit gtk signals; we save and restore the current
+	 * stream position, in case someone changes it while we're not
+	 * holding the lock. */
+        offset = g_mime_stream_tell(mbox->gmime_stream);
+        libbalsa_mime_stream_shared_unlock(mbox->gmime_stream);
+        libbalsa_mailbox_cache_message(LIBBALSA_MAILBOX(mbox), msgno, msg);
         libbalsa_mailbox_local_cache_message(local, msgno, msg);
+        libbalsa_mime_stream_shared_lock(mbox->gmime_stream);
+        g_mime_stream_seek(mbox->gmime_stream, offset, GMIME_STREAM_SEEK_SET);
 
         g_object_unref(msg);
     }
@@ -513,39 +530,30 @@ parse_mailbox(LibBalsaMailboxMbox * mbox)
     lbm_mbox_save(mbox);
 }
 
-static void
-mbox_msg_unref(gpointer data, GObject *msg)
-{
-    LibBalsaMailboxMbox *mbox = LIBBALSA_MAILBOX_MBOX(data);
-    unsigned msgno = LIBBALSA_MESSAGE(msg)->msgno;
-    struct message_info *msg_info =
-	&g_array_index(mbox->messages_info, struct message_info, msgno - 1);
-    msg_info->message = NULL;
-}
-
 static void
 free_message_info(struct message_info *msg_info)
 {
-    if (msg_info->message) {
-	g_object_weak_unref(G_OBJECT(msg_info->message), mbox_msg_unref,
-			    msg_info->message->mailbox);
-	msg_info->message->mailbox = NULL;
-	msg_info->message->msgno   = 0;
-	msg_info->message = NULL;
+    if (msg_info->local_info.message) {
+	msg_info->local_info.message->mailbox = NULL;
+	msg_info->local_info.message->msgno   = 0;
+        g_object_remove_weak_pointer(G_OBJECT(msg_info->local_info.message),
+                                     (gpointer) & msg_info->local_info.message);
+	msg_info->local_info.message = NULL;
     }
 }
 
 static void
-free_messages_info(GArray * messages_info)
+free_messages_info(GPtrArray * msgno_2_msg_info)
 {
     guint i;
 
-    for (i = 0; i < messages_info->len; i++) {
-	struct message_info *msg_info =
-	    &g_array_index(messages_info, struct message_info, i);
-	free_message_info(msg_info);
+    for (i = 0; i < msgno_2_msg_info->len; i++) {
+        struct message_info *msg_info =
+            g_ptr_array_index(msgno_2_msg_info, i);
+        free_message_info(msg_info);
+        g_free(msg_info);
     }
-    g_array_free(messages_info, TRUE);
+    g_ptr_array_free(msgno_2_msg_info, TRUE);
 }
 
 static void
@@ -589,7 +597,7 @@ lbm_mbox_restore(LibBalsaMailboxMbox * mbox)
 
     end = 0;
     do {
-        msg_info->message = NULL;
+        msg_info->local_info.message = NULL;
         if (msg_info->start != end)
             /* Error: this message doesn't start at the end of the
              * previous one. */
@@ -604,12 +612,10 @@ lbm_mbox_restore(LibBalsaMailboxMbox * mbox)
             && !lbm_mbox_seek_to_message(mbox, msg_info->end))
             /* Error: no message following this one. */
             break;
-        /* dummy entry in mindex for now */
-        g_ptr_array_add(LIBBALSA_MAILBOX(mbox)->mindex, NULL);
+        g_ptr_array_add(mbox->msgno_2_msg_info,
+                        g_memdup(msg_info, sizeof *msg_info));
     } while (++msg_info < (struct message_info *) (contents + length));
 
-    g_array_append_vals(mbox->messages_info, contents,
-                        msg_info - (struct message_info *) contents);
 #ifdef DEBUG
     g_print("%s: %s restored %zd messages\n", __func__,
             LIBBALSA_MAILBOX(mbox)->name,
@@ -694,8 +700,7 @@ libbalsa_mailbox_mbox_open(LibBalsaMailbox * mailbox, GError **err)
     libbalsa_mailbox_set_mtime(mailbox, st.st_mtime);
     mbox->gmime_stream = gmime_stream;
 
-    mbox->messages_info =
-	g_array_new(FALSE, FALSE, sizeof(struct message_info));
+    mbox->msgno_2_msg_info = g_ptr_array_new();
 
     mailbox->unread_messages = 0;
     time(&t0);
@@ -1003,7 +1008,7 @@ libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox)
      * we first check one byte beyond the end of the last message: */
     start = mbox->size + 1;
 
-    while ((msgno = mbox->messages_info->len) > 0) {
+    while ((msgno = mbox->msgno_2_msg_info->len) > 0) {
 	off_t offset;
         struct message_info *msg_info;
 
@@ -1013,12 +1018,11 @@ libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox)
             break;
 
 	/* Back up over this message and try again. */
-        msg_info = &g_array_index(mbox->messages_info, struct message_info,
-                                  msgno - 1);
+        msg_info = message_info_from_msgno(mbox, msgno);
         start = msg_info->start;
 
-        if ((msg_info->flags & LIBBALSA_MESSAGE_FLAG_NEW)
-            && !(msg_info->flags & LIBBALSA_MESSAGE_FLAG_DELETED))
+        if ((msg_info->local_info.flags & LIBBALSA_MESSAGE_FLAG_NEW)
+            && !(msg_info->local_info.flags & LIBBALSA_MESSAGE_FLAG_DELETED))
             --mailbox->unread_messages;
 
 	/* We must drop the mime-stream lock to call
@@ -1033,14 +1037,14 @@ libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox)
         g_mime_stream_seek(mbox_stream, offset, GMIME_STREAM_SEEK_SET);
 
         free_message_info(msg_info);
-        g_array_remove_index(mbox->messages_info, msgno - 1);
+        g_ptr_array_remove_index(mbox->msgno_2_msg_info, msgno - 1);
         mbox->messages_info_changed = TRUE;
     }
     if(msgno == 0)
         g_mime_stream_seek(mbox_stream, 0, GMIME_STREAM_SEEK_SET);
 #ifdef DEBUG
     g_print("%s: start parsing at msgno %d of %d\n", __func__, msgno,
-            mbox->messages_info->len);
+            mbox->msgno_2_msg_info->len);
 #endif
     parse_mailbox(mbox);
     mbox->size = g_mime_stream_tell(mbox_stream);
@@ -1056,9 +1060,9 @@ libbalsa_mailbox_mbox_close_mailbox(LibBalsaMailbox * mailbox,
     LibBalsaMailboxMbox *mbox = LIBBALSA_MAILBOX_MBOX(mailbox);
     guint len;
 
-    len = mbox->messages_info->len;
+    len = mbox->msgno_2_msg_info->len;
     libbalsa_mailbox_mbox_sync(mailbox, expunge);
-    if (mbox->messages_info->len != len)
+    if (mbox->msgno_2_msg_info->len != len)
         libbalsa_mailbox_changed(mailbox);
 
     if (mbox->gmime_stream) {
@@ -1071,8 +1075,8 @@ libbalsa_mailbox_mbox_close_mailbox(LibBalsaMailbox * mailbox,
                                                             expunge);
 
     /* Now it's safe to free the message info. */
-    free_messages_info(mbox->messages_info);
-    mbox->messages_info = NULL;
+    free_messages_info(mbox->msgno_2_msg_info);
+    mbox->msgno_2_msg_info = NULL;
 }
 
 static GMimeMessage *
@@ -1218,7 +1222,7 @@ lbm_mbox_rewrite_in_place(struct message_info *msg_info,
 
     /* Get the flags for the "Status" header. */
     header = g_string_new(NULL);
-    lbm_mbox_status_hdr(msg_info->flags, 0, header);
+    lbm_mbox_status_hdr(msg_info->local_info.flags, 0, header);
     g_assert(header->len <= 3);
 
     if (!lbm_mbox_rewrite_helper(msg_info->status, stream, header,
@@ -1230,7 +1234,7 @@ lbm_mbox_rewrite_in_place(struct message_info *msg_info,
 
     /* Get the flags for the "X-Status" header. */
     g_string_truncate(header, 0);
-    lbm_mbox_x_status_hdr(msg_info->flags, 0, header);
+    lbm_mbox_x_status_hdr(msg_info->local_info.flags, 0, header);
     g_assert(header->len <= 3);
 
     if (!lbm_mbox_rewrite_helper(msg_info->x_status, stream, header,
@@ -1251,7 +1255,7 @@ lbm_mbox_rewrite_in_place(struct message_info *msg_info,
 	g_mime_stream_seek(stream, msg_info->x_status, GMIME_STREAM_SEEK_SET);
 	g_mime_stream_write(stream, x_status_buf, sizeof(x_status_buf));
     }
-    msg_info->orig_flags = msg_info->flags & LIBBALSA_MESSAGE_FLAGS_REAL;
+    msg_info->orig_flags = REAL_FLAGS(msg_info->local_info.flags);
     return TRUE;
 }
 
@@ -1364,7 +1368,7 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
      */
     libbalsa_mailbox_mbox_check(mailbox);
     mbox = LIBBALSA_MAILBOX_MBOX(mailbox);
-    if (mbox->messages_info->len == 0)
+    if (mbox->msgno_2_msg_info->len == 0)
 	return TRUE;
     mbox_stream = mbox->gmime_stream;
 
@@ -1388,20 +1392,19 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
      * missing either status header, to reduce the chances of multiple
      * rewrites.
      */
-    messages = mbox->messages_info->len;
+    messages = mbox->msgno_2_msg_info->len;
     first = -1;
     for (i = j = 0; i < messages; i++)
     {
-	msg_info = &g_array_index(mbox->messages_info,
-		       struct message_info, i);
+	msg_info = message_info_from_msgno(mbox, i + 1);
 	if (mailbox->state == LB_MAILBOX_STATE_CLOSING)
-	    msg_info->flags &= ~LIBBALSA_MESSAGE_FLAG_RECENT;
-	if (expunge && (msg_info->flags & LIBBALSA_MESSAGE_FLAG_DELETED))
+	    msg_info->local_info.flags &= ~LIBBALSA_MESSAGE_FLAG_RECENT;
+	if (expunge && (msg_info->local_info.flags & LIBBALSA_MESSAGE_FLAG_DELETED))
 	    break;
 	if (first < 0 && (msg_info->status < 0 || msg_info->x_status < 0))
 	    first = i;
-        if ((msg_info->orig_flags ^ msg_info->flags)
-            & LIBBALSA_MESSAGE_FLAGS_REAL) {
+        if (FLAGS_REALLY_DIFFER(msg_info->orig_flags,
+                                msg_info->local_info.flags)) {
 	    gboolean can_rewrite_in_place;
 
 	    libbalsa_mime_stream_shared_lock(mbox_stream);
@@ -1437,8 +1440,7 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     if (first < 0)
 	first = i; 
     /* where to start overwriting */
-    offset =
-	g_array_index(mbox->messages_info, struct message_info, first).start;
+    offset = message_info_from_msgno(mbox, first + 1)->start;
 
     /* Create a temporary file to write the new version of the mailbox in. */
     i = g_file_open_tmp("balsa-tmp-mbox-XXXXXX", &tempfile, &error);
@@ -1454,9 +1456,8 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     for (i = first; i < messages; i++) {
 	guint status_len, x_status_len;
 
-	msg_info = &g_array_index(mbox->messages_info,
-				  struct message_info, i);
-	if (expunge && (msg_info->flags & LIBBALSA_MESSAGE_FLAG_DELETED))
+	msg_info = message_info_from_msgno(mbox, i + 1);
+	if (expunge && (msg_info->local_info.flags & LIBBALSA_MESSAGE_FLAG_DELETED))
 	    continue;
 
 	if (msg_info->status >= 0) {
@@ -1483,12 +1484,12 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	if (msg_info->status <= msg_info->x_status) {
 	    if (!lbm_mbox_copy_stream(mbox, msg_info->start,
 				      msg_info->status, temp_stream)
-		|| !lbm_mbox_write_status_hdr(temp_stream, msg_info->flags)
+		|| !lbm_mbox_write_status_hdr(temp_stream, msg_info->local_info.flags)
 		|| !lbm_mbox_copy_stream(mbox,
 					 msg_info->status + status_len,
 					 msg_info->x_status, temp_stream)
 		|| !lbm_mbox_write_x_status_hdr(temp_stream,
-						msg_info->flags)
+						msg_info->local_info.flags)
 		|| !lbm_mbox_copy_stream(mbox,
 					 msg_info->x_status +
 					 x_status_len, msg_info->end,
@@ -1498,12 +1499,12 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	    if (!lbm_mbox_copy_stream(mbox, msg_info->start,
 				      msg_info->x_status, temp_stream)
 		|| !lbm_mbox_write_x_status_hdr(temp_stream,
-						msg_info->flags)
+						msg_info->local_info.flags)
 		|| !lbm_mbox_copy_stream(mbox,
 					 msg_info->x_status +
 					 x_status_len,
 					 msg_info->status, temp_stream)
-		|| !lbm_mbox_write_status_hdr(temp_stream, msg_info->flags)
+		|| !lbm_mbox_write_status_hdr(temp_stream, msg_info->local_info.flags)
 		|| !lbm_mbox_copy_stream(mbox,
 					 msg_info->status + status_len,
 					 msg_info->end, temp_stream))
@@ -1580,14 +1581,13 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 
     if (mailbox->state == LB_MAILBOX_STATE_CLOSING) {
 	/* Just shorten the msg_info array. */
-	for (j = first; j < mbox->messages_info->len; ) {
-	    msg_info = &g_array_index(mbox->messages_info,
-				      struct message_info, j);
+	for (j = first; j < mbox->msgno_2_msg_info->len; ) {
+	    msg_info = message_info_from_msgno(mbox, j + 1);
 	    if (expunge &&
-		(msg_info->flags & LIBBALSA_MESSAGE_FLAG_DELETED)) {
+		(msg_info->local_info.flags & LIBBALSA_MESSAGE_FLAG_DELETED)) {
 	        libbalsa_mailbox_local_msgno_removed(mailbox, j + 1);
 		free_message_info(msg_info);
-		g_array_remove_index(mbox->messages_info, j);
+		g_ptr_array_remove_index(mbox->msgno_2_msg_info, j);
                 mbox->messages_info_changed = TRUE;
 	    } else
 		j++;
@@ -1611,13 +1611,12 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
                                    "^Status|^X-Status|^MIME-Version",
 				   (GMimeParserHeaderRegexFunc)
 				   lbm_mbox_header_cb, &msg_info);
-    for (j = first; j < mbox->messages_info->len; ) {
+    for (j = first; j < mbox->msgno_2_msg_info->len; ) {
 	GMimeMessage *mime_msg;
         gchar *from;
 
-	msg_info =
-	    &g_array_index(mbox->messages_info, struct message_info, j);
-	if (expunge && (msg_info->flags & LIBBALSA_MESSAGE_FLAG_DELETED)) {
+	msg_info = message_info_from_msgno(mbox, j + 1);
+	if (expunge && (msg_info->local_info.flags & LIBBALSA_MESSAGE_FLAG_DELETED)) {
 	    /* We must drop the mime-stream lock to call
 	     * libbalsa_mailbox_local_msgno_removed(), as it will grab
 	     * the gdk lock to emit gtk signals; we save and restore the
@@ -1629,12 +1628,12 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	    libbalsa_mime_stream_shared_lock(mbox_stream);
 	    g_mime_stream_seek(mbox_stream, offset, GMIME_STREAM_SEEK_SET);
 	    free_message_info(msg_info);
-	    g_array_remove_index(mbox->messages_info, j);
+	    g_ptr_array_remove_index(mbox->msgno_2_msg_info, j);
             mbox->messages_info_changed = TRUE;
 	    continue;
 	}
-	if (msg_info->message)
-	    msg_info->message->msgno = j + 1;
+	if (msg_info->local_info.message)
+	    msg_info->local_info.message->msgno = j + 1;
 	j++;
 
 	msg_info->status = msg_info->x_status = msg_info->mime_version = -1;
@@ -1653,66 +1652,36 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
         msg_info->from_len = strlen(from) + 1;
         g_free(from);
 	msg_info->end = g_mime_parser_tell(gmime_parser);
-	msg_info->orig_flags = msg_info->flags & LIBBALSA_MESSAGE_FLAGS_REAL;
+	msg_info->orig_flags = REAL_FLAGS(msg_info->local_info.flags);
 	g_assert(mime_msg != NULL);
 	g_assert(mime_msg->mime_part != NULL);
-	if (!msg_info->message || !msg_info->message->mime_msg)
+	if (!msg_info->local_info.message || !msg_info->local_info.message->mime_msg)
 	    g_object_unref(mime_msg);
 	else {
-	    g_object_unref(msg_info->message->mime_msg);
-	    msg_info->message->mime_msg = mime_msg;
+	    g_object_unref(msg_info->local_info.message->mime_msg);
+	    msg_info->local_info.message->mime_msg = mime_msg;
 	    /*
 	     * reinit the message parts info
 	     */
-	    libbalsa_message_body_set_mime_body(msg_info->message->body_list,
+	    libbalsa_message_body_set_mime_body(msg_info->local_info.message->body_list,
 						mime_msg->mime_part);
 	}
     }
     libbalsa_mime_stream_shared_unlock(mbox_stream);
-    mbox->messages_info->len = j;
+    mbox->msgno_2_msg_info->len = j;
     g_object_unref(gmime_parser);
     lbm_mbox_save(mbox);
 
     return TRUE;
 }
 
-static LibBalsaMessage *
-lbm_mbox_get_message(LibBalsaMailbox * mailbox, guint msgno)
-{
-    LibBalsaMailboxMbox *mbox = LIBBALSA_MAILBOX_MBOX(mailbox);
-    struct message_info *msg_info;
-    LibBalsaMessage *message;
-
-    msg_info = &g_array_index(mbox->messages_info, struct message_info,
-                              msgno - 1);
-
-    message = msg_info->message;
-    if (message)
-        return g_object_ref(message);
-
-    msg_info->message = message = libbalsa_message_new();
-    g_object_weak_ref(G_OBJECT(message), mbox_msg_unref, mbox);
-
-    message->flags = msg_info->flags & LIBBALSA_MESSAGE_FLAGS_REAL;
-    message->mailbox = mailbox;
-    message->msgno = msgno;
-    libbalsa_message_load_envelope(message);
-
-    return message;
-}
-
-static LibBalsaMessageFlag
-lbm_mbox_load_message(LibBalsaMailboxLocal * local, guint msgno,
-                      LibBalsaMessage ** msg)
+static LibBalsaMailboxLocalMessageInfo *
+lbm_mbox_get_info(LibBalsaMailboxLocal * local, guint msgno)
 {
     LibBalsaMailboxMbox *mbox = LIBBALSA_MAILBOX_MBOX(local);
-    struct message_info *msg_info =
-        &g_array_index(mbox->messages_info, struct message_info, msgno - 1);
+    struct message_info *msg_info = message_info_from_msgno(mbox, msgno);
 
-    if (msg_info->message)
-        *msg = g_object_ref(msg_info->message);
-
-    return msg_info->flags & LIBBALSA_MESSAGE_FLAGS_REAL;
+    return &msg_info->local_info;
 }
 
 static gboolean
@@ -1988,73 +1957,12 @@ libbalsa_mailbox_mbox_add_message(LibBalsaMailbox * mailbox,
     return retval >= 0;
 }
 
-static gboolean
-libbalsa_mailbox_mbox_messages_change_flags(LibBalsaMailbox * mailbox,
-                                            GArray * msgnos,
-                                            LibBalsaMessageFlag set,
-                                            LibBalsaMessageFlag clear)
-{
-    guint i;
-    guint changed = 0;
-
-    for (i = 0; i < msgnos->len; i++) {
-        guint msgno = g_array_index(msgnos, guint, i);
-        struct message_info *msg_info =
-            &g_array_index(LIBBALSA_MAILBOX_MBOX(mailbox)->messages_info,
-                           struct message_info, msgno - 1);
-        LibBalsaMessageFlag old_flags = msg_info->flags;
-        gboolean was_unread_undeleted, is_unread_undeleted;
-
-        msg_info->flags |= set;
-        msg_info->flags &= ~clear;
-        if (!((old_flags ^ msg_info->flags) & LIBBALSA_MESSAGE_FLAGS_REAL))
-	    /* No real flags changed. */
-            continue;
-        ++changed;
-
-        if (msg_info->message)
-            msg_info->message->flags =
-		msg_info->flags & LIBBALSA_MESSAGE_FLAGS_REAL;
-
-        libbalsa_mailbox_index_set_flags(mailbox, msgno, msg_info->flags);
-
-        was_unread_undeleted = (old_flags & LIBBALSA_MESSAGE_FLAG_NEW)
-            && !(old_flags & LIBBALSA_MESSAGE_FLAG_DELETED);
-        is_unread_undeleted = (msg_info->flags & LIBBALSA_MESSAGE_FLAG_NEW)
-            && !(msg_info->flags & LIBBALSA_MESSAGE_FLAG_DELETED);
-        mailbox->unread_messages +=
-            is_unread_undeleted - was_unread_undeleted;
-    }
-
-    if (changed > 0) {
-        libbalsa_mailbox_set_unread_messages_flag(mailbox,
-                                                  mailbox->unread_messages
-                                                  > 0);
-        libbalsa_mailbox_local_queue_sync(LIBBALSA_MAILBOX_LOCAL(mailbox));
-    }
-
-    return TRUE;
-}
-
-static gboolean
-libbalsa_mailbox_mbox_msgno_has_flags(LibBalsaMailbox * mailbox,
-                                      guint msgno, LibBalsaMessageFlag set,
-                                      LibBalsaMessageFlag unset)
-{
-    struct message_info *msg_info =
-        &g_array_index(LIBBALSA_MAILBOX_MBOX(mailbox)->messages_info,
-                       struct message_info, msgno - 1);
-
-    return (msg_info->flags & set) == set
-        && (msg_info->flags & unset) == 0;
-}
-
 static guint
 libbalsa_mailbox_mbox_total_messages(LibBalsaMailbox * mailbox)
 {
     LibBalsaMailboxMbox *mbox = (LibBalsaMailboxMbox *) mailbox;
 
-    return mbox->messages_info ? mbox->messages_info->len : 0;
+    return mbox->msgno_2_msg_info ? mbox->msgno_2_msg_info->len : 0;
 }
 
 #if BALSA_USE_THREADS
