@@ -22,11 +22,11 @@
 
 #ifdef HAVE_GPGME
 
-#include <gnome.h>
 #include <string.h>
 #include <gpgme.h>
 #include <pwd.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 
 #include "libbalsa.h"
 #include "libbalsa_private.h"
@@ -36,12 +36,17 @@
 #include "gmime-gpgme-signature.h"
 #include "gmime-part-rfc2440.h"
 
+#ifdef HAVE_SMIME
+#  include "gmime-application-pkcs7.h"
+#endif
+
 #ifdef BALSA_USE_THREADS
 #  include <pthread.h>
 #  include "misc.h"
 #endif
 
 #include "padlock-keyhole.xpm"
+#include "i18n.h"
 
 
 /* local prototypes */
@@ -54,7 +59,12 @@ static gpgme_key_t select_key_from_list(const gchar * name,
 					gboolean is_secret,
 					GMimeGpgmeContext * ctx,
 					GList * keys);
+static gboolean accept_low_trust_key(const gchar * name,
+				     gpgme_user_id_t uid,
+				     GMimeGpgmeContext * ctx);
 static gboolean gpg_updates_trustdb(void);
+static gchar *fix_EMail_info(gchar * str);
+
 
 /* ==== public functions =================================================== */
 gboolean
@@ -95,7 +105,7 @@ libbalsa_check_crypto_engine(gpgme_protocol_t protocol)
 	} else
 	    g_string_append_printf(message,
 				   _
-				   ("%s: could not retreive crypto engine information: %s."),
+				   ("%s: could not retrieve crypto engine information: %s."),
 				   gpgme_strsource(err),
 				   gpgme_strerror(err));
 	g_string_append_printf(message,
@@ -150,10 +160,14 @@ libbalsa_message_body_protection(LibBalsaMessageBody * body)
 
 	result = LIBBALSA_PROTECT_SIGN;
 	if (protocol && body->parts && body->parts->next) {
-	    if (!g_ascii_strcasecmp
-		("application/pkcs7-signature", protocol)
-		&& body_is_type(body->parts->next, "application",
-				"pkcs7-signature")) {
+	    if ((!g_ascii_strcasecmp("application/pkcs7-signature",
+				     protocol)
+		 && body_is_type(body->parts->next, "application", 
+				 "pkcs7-signature")) ||
+		(!g_ascii_strcasecmp("application/x-pkcs7-signature",
+				     protocol)
+		 && body_is_type(body->parts->next, "application",
+				 "x-pkcs7-signature"))) {
 		result |= LIBBALSA_PROTECT_SMIMEV3;
 		if (!micalg)
 		    result |= LIBBALSA_PROTECT_ERROR;
@@ -189,10 +203,9 @@ libbalsa_message_body_protection(LibBalsaMessageBody * body)
 	    libbalsa_message_body_get_parameter(body, "smime-type");
 
 	result = LIBBALSA_PROTECT_SMIMEV3;
-	if (!g_ascii_strcasecmp("enveloped-data", smime_type))
+	if (!g_ascii_strcasecmp("enveloped-data", smime_type) ||
+	    !g_ascii_strcasecmp("signed-data", smime_type))
 	    result |= LIBBALSA_PROTECT_ENCRYPT;
-	else if (!g_ascii_strcasecmp("signed-data", smime_type))
-	    result |= LIBBALSA_PROTECT_SIGN;
 	else
 	    result |= LIBBALSA_PROTECT_ERROR;
 	g_free(smime_type);
@@ -207,19 +220,25 @@ libbalsa_message_body_protection(LibBalsaMessageBody * body)
  * Signs the MIME object *content with the private key of rfc822_for using
  * protocol. Return TRUE on success (in which case *content is replaced by the
  * new MIME object).
+ * Note: In RFC 2633 mode (GPGME_PROTOCOL_CMS), this function creates a
+ * multipart/signed instead of an application/pkcs7-mime, as the latter one
+ * doesn't contain a cleartext also readable for MUA's without S/MIME support.
  */
 gboolean
 libbalsa_sign_mime_object(GMimeObject ** content, const gchar * rfc822_for,
-			  gpgme_protocol_t protocol, GtkWindow * parent)
+			  gpgme_protocol_t protocol, GtkWindow * parent,
+			  GError ** error)
 {
     GMimeSession *session;
     GMimeGpgmeContext *ctx;
     GMimeMultipartSigned *mps;
-    GError *error = NULL;
 
     /* paranoia checks */
     g_return_val_if_fail(rfc822_for != NULL, FALSE);
     g_return_val_if_fail(content != NULL, FALSE);
+#ifndef HAVE_SMIME
+    g_return_val_if_fail(protocol == GPGME_PROTOCOL_OpenPGP, FALSE);
+#endif
 
     /* check if gpg is currently available */
     if (protocol == GPGME_PROTOCOL_OpenPGP && gpg_updates_trustdb())
@@ -228,17 +247,8 @@ libbalsa_sign_mime_object(GMimeObject ** content, const gchar * rfc822_for,
     /* create a session and a GMimeGpgmeContext */
     session = g_object_new(g_mime_session_get_type(), NULL, NULL);
     ctx = GMIME_GPGME_CONTEXT(g_mime_gpgme_context_new(session, protocol,
-						       &error));
+						       error));
     if (ctx == NULL) {
-	if (error) {
-	    libbalsa_information(LIBBALSA_INFORMATION_ERROR, "%s: %s",
-				 _("creating a gpgme context failed"),
-				 error->message);
-	    g_error_free(error);
-	} else
-	    libbalsa_information(LIBBALSA_INFORMATION_ERROR,
-				 _("creating a gpgme context failed"),
-				 error->message);
 	g_object_unref(session);
 	return FALSE;
     }
@@ -251,9 +261,9 @@ libbalsa_sign_mime_object(GMimeObject ** content, const gchar * rfc822_for,
 	g_object_set_data(G_OBJECT(ctx), "passphrase-info",
 			  _
 			  ("Enter passphrase to unlock the secret key for signing"));
-	ctx->key_select_cb = select_key_from_list;
-	g_object_set_data(G_OBJECT(ctx), "parent-window", parent);
     }
+    ctx->key_select_cb = select_key_from_list;
+    g_object_set_data(G_OBJECT(ctx), "parent-window", parent);
 
     /* call gpgme to create the signature */
     if (!(mps = g_mime_multipart_signed_new())) {
@@ -264,18 +274,10 @@ libbalsa_sign_mime_object(GMimeObject ** content, const gchar * rfc822_for,
 
     if (g_mime_multipart_signed_sign
 	(mps, *content, GMIME_CIPHER_CONTEXT(ctx), rfc822_for,
-	 GMIME_CIPHER_HASH_DEFAULT, &error) != 0) {
+	 GMIME_CIPHER_HASH_DEFAULT, error) != 0) {
 	g_object_unref(mps);
 	g_object_unref(ctx);
 	g_object_unref(session);
-	if (error) {
-	    if (error->code != GPG_ERR_CANCELED)
-		libbalsa_information(LIBBALSA_INFORMATION_ERROR, "%s: %s",
-				     _("signing failed"), error->message);
-	    g_error_free(error);
-	} else
-	    libbalsa_information(LIBBALSA_INFORMATION_ERROR,
-				 _("signing failed"));
 	return FALSE;
     }
 
@@ -296,17 +298,21 @@ libbalsa_sign_mime_object(GMimeObject ** content, const gchar * rfc822_for,
  */
 gboolean
 libbalsa_encrypt_mime_object(GMimeObject ** content, GList * rfc822_for,
-			     gpgme_protocol_t protocol, GtkWindow * parent)
+			     gpgme_protocol_t protocol, gboolean always_trust,
+			     GtkWindow * parent, GError ** error)
 {
     GMimeSession *session;
     GMimeGpgmeContext *ctx;
+    GMimeObject *encrypted_obj = NULL;
     GPtrArray *recipients;
-    GMimeMultipartEncrypted *mpe;
-    GError *error = NULL;
+    int result = -1;
 
     /* paranoia checks */
     g_return_val_if_fail(rfc822_for != NULL, FALSE);
     g_return_val_if_fail(content != NULL, FALSE);
+#ifndef HAVE_SMIME
+    g_return_val_if_fail(protocol == GPGME_PROTOCOL_OpenPGP, FALSE);
+#endif
 
     /* check if gpg is currently available */
     if (protocol == GPGME_PROTOCOL_OpenPGP && gpg_updates_trustdb())
@@ -315,22 +321,17 @@ libbalsa_encrypt_mime_object(GMimeObject ** content, GList * rfc822_for,
     /* create a session and a GMimeGpgmeContext */
     session = g_object_new(g_mime_session_get_type(), NULL, NULL);
     ctx = GMIME_GPGME_CONTEXT(g_mime_gpgme_context_new(session, protocol,
-						       &error));
+						       error));
     if (ctx == NULL) {
-	if (error) {
-	    libbalsa_information(LIBBALSA_INFORMATION_ERROR, "%s: %s",
-				 _("creating a gpgme context failed"),
-				 error->message);
-	    g_error_free(error);
-	} else
-	    libbalsa_information(LIBBALSA_INFORMATION_ERROR,
-				 _("creating a gpgme context failed"));
 	g_object_unref(session);
 	return FALSE;
     }
 
     /* set the callback for the key selection (no secret needed here) */
     ctx->key_select_cb = select_key_from_list;
+    if (!always_trust)
+	ctx->key_trust_cb = accept_low_trust_key;
+    ctx->always_trust_uid = always_trust;
     g_object_set_data(G_OBJECT(ctx), "parent-window", parent);
 
     /* convert the key list to a GPtrArray */
@@ -340,30 +341,43 @@ libbalsa_encrypt_mime_object(GMimeObject ** content, GList * rfc822_for,
 	rfc822_for = g_list_next(rfc822_for);
     }
 
-    /* encrypt */
-    mpe = g_mime_multipart_encrypted_new();
-    if (g_mime_multipart_encrypted_encrypt(mpe, *content,
-					   GMIME_CIPHER_CONTEXT(ctx),
-					   recipients, &error) != 0) {
-	if (error) {
-	    if (error->code != GPG_ERR_CANCELED)
-		libbalsa_information(LIBBALSA_INFORMATION_ERROR, "%s: %s",
-				     _("encryption failed"),
-				     error->message);
-	    g_error_free(error);
-	} else
-	    libbalsa_information(LIBBALSA_INFORMATION_ERROR,
-				 _("encryption failed"));
+    /* encrypt: multipart/encrypted for RFC 3156, application/pkcs7-mime for
+       RFC 2633 */
+    if (protocol == GPGME_PROTOCOL_OpenPGP) {
+	GMimeMultipartEncrypted *mpe = g_mime_multipart_encrypted_new();
+
+	encrypted_obj = GMIME_OBJECT(mpe);
+	result = 
+	    g_mime_multipart_encrypted_encrypt(mpe, *content,
+					       GMIME_CIPHER_CONTEXT(ctx),
+					       recipients, error);
+    }
+#ifdef HAVE_SMIME
+    else {
+	GMimePart *pkcs7 =
+	    g_mime_part_new_with_type("application", "pkcs7-mime");
+
+	encrypted_obj = GMIME_OBJECT(pkcs7);
+	ctx->singlepart_mode = TRUE;
+	result = 
+	    g_mime_application_pkcs7_encrypt(pkcs7, *content,
+					     GMIME_CIPHER_CONTEXT(ctx),
+					     recipients, error);
+    }
+#endif
+
+    /* error checking */
+    if (result != 0) {
 	g_ptr_array_free(recipients, FALSE);
 	g_object_unref(ctx);
 	g_object_unref(session);
-	g_object_unref(mpe);
+	g_object_unref(encrypted_obj);
 	return FALSE;
     }
 
     g_ptr_array_free(recipients, FALSE);
     g_object_unref(G_OBJECT(*content));
-    *content = GMIME_OBJECT(mpe);
+    *content = GMIME_OBJECT(encrypted_obj);
     g_object_unref(ctx);
     g_object_unref(session);
 
@@ -382,7 +396,9 @@ libbalsa_sign_encrypt_mime_object(GMimeObject ** content,
 				  const gchar * rfc822_signer,
 				  GList * rfc822_for,
 				  gpgme_protocol_t protocol,
-				  GtkWindow * parent)
+				  gboolean always_trust,
+				  GtkWindow * parent,
+				  GError ** error)
 {
     GMimeObject *signed_object;
 
@@ -390,6 +406,9 @@ libbalsa_sign_encrypt_mime_object(GMimeObject ** content,
     g_return_val_if_fail(rfc822_signer != NULL, FALSE);
     g_return_val_if_fail(rfc822_for != NULL, FALSE);
     g_return_val_if_fail(content != NULL, FALSE);
+#ifndef HAVE_SMIME
+    g_return_val_if_fail(protocol == GPGME_PROTOCOL_OpenPGP, FALSE);
+#endif
 
     /* check if gpg is currently available */
     if (protocol == GPGME_PROTOCOL_OpenPGP && gpg_updates_trustdb())
@@ -400,11 +419,11 @@ libbalsa_sign_encrypt_mime_object(GMimeObject ** content,
     g_object_ref(G_OBJECT(signed_object));
 
     if (!libbalsa_sign_mime_object(&signed_object, rfc822_signer, protocol,
-				   parent))
+				   parent, error))
 	return FALSE;
 
     if (!libbalsa_encrypt_mime_object(&signed_object, rfc822_for, protocol,
-				      parent)) {
+				      always_trust, parent, error)) {
 	g_object_unref(G_OBJECT(signed_object));
 	return FALSE;
     }
@@ -432,26 +451,27 @@ libbalsa_body_check_signature(LibBalsaMessageBody * body,
     GMimeCipherContext *ctx;
     GMimeSignatureValidity *valid;
     GError *error = NULL;
+
+    /* paranoia checks */
     g_return_val_if_fail(body, FALSE);
-    g_return_val_if_fail(body->parts, FALSE);
-    g_return_val_if_fail(body->parts->next, FALSE);
+    g_return_val_if_fail(body->mime_part != NULL, FALSE);
     g_return_val_if_fail(body->message, FALSE);
+#ifndef HAVE_SMIME
+    g_return_val_if_fail(protocol == GPGME_PROTOCOL_OpenPGP, FALSE);
+#endif
 
     /* check if gpg is currently available */
     if (protocol == GPGME_PROTOCOL_OpenPGP && gpg_updates_trustdb())
 	return FALSE;
 
     /* check if the body is really a multipart/signed */
-    if (!GMIME_IS_MULTIPART_SIGNED(body->mime_part))
-	return FALSE;
+    if (!GMIME_IS_MULTIPART_SIGNED(body->mime_part)
+        || (g_mime_multipart_get_number
+            (((GMimeMultipart *) body->mime_part))) < 2)
+        return FALSE;
     if (body->parts->next->sig_info)
 	g_object_unref(G_OBJECT(body->parts->next->sig_info));
 
-    /* fix the protocol of the multipart/signed if protocol is S/MIME */
-    if (protocol == GPGME_PROTOCOL_CMS)
-	g_mime_object_set_content_type_parameter(GMIME_OBJECT (body->mime_part),
-						 "protocol", "application/pkcs7-signature");
-
     /* try to create GMimeGpgMEContext */
     session = g_object_new(g_mime_session_get_type(), NULL, NULL);
     ctx = g_mime_gpgme_context_new(session, protocol, &error);
@@ -470,9 +490,24 @@ libbalsa_body_check_signature(LibBalsaMessageBody * body,
 	return FALSE;
     }
 
+    /* S/MIME uses the protocol application/pkcs7-signature, but some ancient
+       mailers, not yet knowing RFC 2633, use application/x-pkcs7-signature,
+       so tweak the context if necessary... */
+    if (protocol == GPGME_PROTOCOL_CMS) {
+	const char * cms_protocol = 
+	    g_mime_object_get_content_type_parameter(GMIME_OBJECT (body->mime_part),
+						     "protocol");
+	if (!g_ascii_strcasecmp(cms_protocol, "application/x-pkcs7-signature"))
+	    ctx->sign_protocol = cms_protocol;
+    }
+
     /* verify the signature */
+
+    libbalsa_mailbox_lock_store(body->message->mailbox);
     valid = g_mime_multipart_signed_verify(GMIME_MULTIPART_SIGNED
 					   (body->mime_part), ctx, &error);
+    libbalsa_mailbox_unlock_store(body->message->mailbox);
+
     if (valid == NULL) {
 	if (error) {
 	    libbalsa_information(LIBBALSA_INFORMATION_ERROR, "%s: %s",
@@ -483,8 +518,10 @@ libbalsa_body_check_signature(LibBalsaMessageBody * body,
 	    libbalsa_information(LIBBALSA_INFORMATION_ERROR,
 				 _("signature verification failed"));
     }
-    body->parts->next->sig_info = GMIME_GPGME_CONTEXT(ctx)->sig_state;
-    g_object_ref(G_OBJECT(body->parts->next->sig_info));
+    if (GMIME_GPGME_CONTEXT(ctx)->sig_state) {
+	body->parts->next->sig_info = GMIME_GPGME_CONTEXT(ctx)->sig_state;
+	g_object_ref(G_OBJECT(body->parts->next->sig_info));
+    }
     g_mime_signature_validity_free(valid);
     g_object_unref(ctx);
     g_object_unref(session);
@@ -493,7 +530,7 @@ libbalsa_body_check_signature(LibBalsaMessageBody * body,
 
 
 /*
- * body points to an application/pgp-encrypted body. If decryption is
+ * Body points to an application/pgp-encrypted body. If decryption is
  * successful, it is freed, and the routine returns a pointer to the chain of
  * decrypted bodies. Otherwise, the original body is returned.
  */
@@ -503,21 +540,46 @@ libbalsa_body_decrypt(LibBalsaMessageBody * body,
 {
     GMimeSession *session;
     GMimeGpgmeContext *ctx;
-    GMimeObject *mime_obj;
+    GMimeObject *mime_obj = NULL;
     GError *error = NULL;
     LibBalsaMessage *message;
+#ifdef HAVE_SMIME
+    gboolean smime_signed = FALSE;
+#endif
 
     /* paranoia checks */
     g_return_val_if_fail(body != NULL, body);
+    g_return_val_if_fail(body->mime_part != NULL, body);
     g_return_val_if_fail(body->message != NULL, body);
+#ifndef HAVE_SMIME
+    g_return_val_if_fail(protocol == GPGME_PROTOCOL_OpenPGP, FALSE);
+#endif
 
     /* check if gpg is currently available */
     if (protocol == GPGME_PROTOCOL_OpenPGP && gpg_updates_trustdb())
 	return body;
 
-    /* check if the body is really a multipart/signed */
-    if (!GMIME_IS_MULTIPART_ENCRYPTED(body->mime_part))
-	return body;
+    /* sanity checks... */
+    if (protocol == GPGME_PROTOCOL_OpenPGP) {
+	if (!GMIME_IS_MULTIPART_ENCRYPTED(body->mime_part))
+	    return body;
+    }
+#ifdef HAVE_SMIME
+    else {
+	const char * smime_type = 
+	    g_mime_object_get_content_type_parameter(body->mime_part,
+						     "smime-type");
+
+	if (!smime_type || !GMIME_IS_PART(body->mime_part))
+	    return body;
+	if (!g_ascii_strcasecmp(smime_type, "signed-data"))
+	    smime_signed = TRUE;
+	else if (!g_ascii_strcasecmp(smime_type, "enveloped-data"))
+	    smime_signed = FALSE;
+	else
+	    return body;
+    }
+#endif
 
     /* create a session and a GMimeGpgmeContext */
     session = g_object_new(g_mime_session_get_type(), NULL, NULL);
@@ -546,11 +608,32 @@ libbalsa_body_decrypt(LibBalsaMessageBody * body,
 			  _("Enter passphrase to decrypt message"));
     }
 
-    mime_obj =
-	g_mime_multipart_encrypted_decrypt(GMIME_MULTIPART_ENCRYPTED
-					   (body->mime_part),
-					   GMIME_CIPHER_CONTEXT(ctx),
-					   &error);
+    libbalsa_mailbox_lock_store(body->message->mailbox);
+    if (protocol == GPGME_PROTOCOL_OpenPGP)
+	mime_obj =
+	    g_mime_multipart_encrypted_decrypt(GMIME_MULTIPART_ENCRYPTED(body->mime_part),
+					       GMIME_CIPHER_CONTEXT(ctx),
+					       &error);
+#ifdef HAVE_SMIME
+    else if (smime_signed) {
+	GMimeSignatureValidity *valid;
+
+	ctx->singlepart_mode = TRUE;
+	mime_obj =
+	    g_mime_application_pkcs7_verify(GMIME_PART(body->mime_part),
+					    &valid,
+					    GMIME_CIPHER_CONTEXT(ctx),
+					    &error);
+	g_mime_signature_validity_free(valid);
+    } else
+	mime_obj =
+	    g_mime_application_pkcs7_decrypt(GMIME_PART(body->mime_part),
+					       GMIME_CIPHER_CONTEXT(ctx),
+					       &error);
+#endif
+    libbalsa_mailbox_unlock_store(body->message->mailbox);
+
+    /* check the result */
     if (mime_obj == NULL) {
 	if (error) {
 	    if (error->code != GPG_ERR_CANCELED)
@@ -568,7 +651,15 @@ libbalsa_body_decrypt(LibBalsaMessageBody * body,
     message = body->message;
     libbalsa_message_body_free(body);
     body = libbalsa_message_body_new(message);
-    body->was_encrypted = TRUE;  /* remember that is was encrypted */
+
+    /* remember that is was encrypted */
+    if (protocol == GPGME_PROTOCOL_OpenPGP)
+	body->was_encrypted = TRUE;
+#ifdef HAVE_SMIME
+    else
+	body->was_encrypted = !smime_signed;
+#endif
+
     libbalsa_message_body_set_mime_body(body, mime_obj);
     g_object_unref(G_OBJECT(mime_obj));
     if (ctx->sig_state && ctx->sig_state->status != GPG_ERR_NOT_SIGNED) {
@@ -586,12 +677,12 @@ libbalsa_body_decrypt(LibBalsaMessageBody * body,
 /* routines dealing with RFC2440 */
 gboolean
 libbalsa_rfc2440_sign_encrypt(GMimePart * part, const gchar * sign_for,
-			      GList * encrypt_for, GtkWindow * parent)
+			      GList * encrypt_for, gboolean always_trust,
+			      GtkWindow * parent, GError ** error)
 {
     GMimeSession *session;
     GMimeGpgmeContext *ctx;
     GPtrArray *recipients;
-    GError *error = NULL;
     gint result;
 
     /* paranoia checks */
@@ -606,16 +697,8 @@ libbalsa_rfc2440_sign_encrypt(GMimePart * part, const gchar * sign_for,
     session = g_object_new(g_mime_session_get_type(), NULL, NULL);
     ctx = GMIME_GPGME_CONTEXT(g_mime_gpgme_context_new(session,
 						       GPGME_PROTOCOL_OpenPGP,
-						       &error));
+						       error));
     if (ctx == NULL) {
-	if (error) {
-	    libbalsa_information(LIBBALSA_INFORMATION_ERROR, "%s: %s",
-				 _("creating a gpgme context failed"),
-				 error->message);
-	    g_error_free(error);
-	} else
-	    libbalsa_information(LIBBALSA_INFORMATION_ERROR,
-				 _("creating a gpgme context failed"));
 	g_object_unref(session);
 	return FALSE;
     }
@@ -632,6 +715,9 @@ libbalsa_rfc2440_sign_encrypt(GMimePart * part, const gchar * sign_for,
 	}
     }
     ctx->key_select_cb = select_key_from_list;
+    if (!always_trust)
+	ctx->key_trust_cb = accept_low_trust_key;
+    ctx->always_trust_uid = always_trust;
     g_object_set_data(G_OBJECT(ctx), "parent-window", parent);
 
     /* convert the key list to a GPtrArray */
@@ -647,36 +733,7 @@ libbalsa_rfc2440_sign_encrypt(GMimePart * part, const gchar * sign_for,
     /* sign and/or encrypt */
     result =
 	g_mime_part_rfc2440_sign_encrypt(part, ctx, recipients, sign_for,
-					 &error);
-    if (result != 0) {
-	if (error) {
-	    if (error->code != GPG_ERR_CANCELED) {
-		if (sign_for && recipients)
-		    libbalsa_information(LIBBALSA_INFORMATION_ERROR, "%s: %s",
-					 _("signing and encryption failed"),
-					 error->message);
-		else if (sign_for)
-		    libbalsa_information(LIBBALSA_INFORMATION_ERROR, "%s: %s",
-					 _("signing failed"), error->message);
-		else
-		    libbalsa_information(LIBBALSA_INFORMATION_ERROR, "%s :%s",
-					 _("encryption failed"),
-					 error->message);
-	    }
-	    g_error_free(error);
-	} else {
-	    if (sign_for && recipients)
-		libbalsa_information(LIBBALSA_INFORMATION_ERROR,
-				     _("signing and encryption failed"));
-	    else if (sign_for)
-		libbalsa_information(LIBBALSA_INFORMATION_ERROR,
-				     _("signing failed: %s"));
-	    else
-		libbalsa_information(LIBBALSA_INFORMATION_ERROR,
-				     _("encryption failed: %s"));
-	}
-    }
-
+					 error);
     /* clean up */
     if (recipients)
 	g_ptr_array_free(recipients, FALSE);
@@ -874,7 +931,7 @@ libbalsa_gpgme_sig_stat_to_gchar(gpgme_error_t stat)
 	    ("The signature is valid but the key used to verify the signature has been revoked.");
     case GPG_ERR_BAD_SIGNATURE:
 	return _
-	    ("The signature is invalid (Note: might be caused by gmime bug!).");
+	    ("The signature is invalid.");
     case GPG_ERR_NO_PUBKEY:
 	return
 	    _("The signature could not be verified due to a missing key.");
@@ -953,11 +1010,11 @@ libbalsa_signature_info_to_gchar(GMimeGpgmeSigstat * info,
 	strftime(buf, sizeof(buf), date_string, &date);
 	g_string_append_printf(msg, _("\nSigned on: %s"), buf);
     }
-    g_string_append_printf(msg, _("\nValidity: %s"),
+    g_string_append_printf(msg, _("\nUser ID validity: %s"),
 			   libbalsa_gpgme_validity_to_gchar(info->
 							    validity));
     if (info->protocol == GPGME_PROTOCOL_OpenPGP)
-	g_string_append_printf(msg, _("\nOwner trust: %s"),
+	g_string_append_printf(msg, _("\nKey owner trust: %s"),
 			       libbalsa_gpgme_validity_to_gchar_short
 			       (info->trust));
     if (info->fingerprint)
@@ -966,7 +1023,12 @@ libbalsa_signature_info_to_gchar(GMimeGpgmeSigstat * info,
     if (info->key_created) {
 	localtime_r(&info->key_created, &date);
 	strftime(buf, sizeof(buf), date_string, &date);
-	g_string_append_printf(msg, _("\nKey created on: %s"), buf);
+	g_string_append_printf(msg, _("\nSubkey created on: %s"), buf);
+    }
+    if (info->key_expires) {
+	localtime_r(&info->key_expires, &date);
+	strftime(buf, sizeof(buf), date_string, &date);
+	g_string_append_printf(msg, _("\nSubkey expires on: %s"), buf);
     }
     if (info->key_revoked || info->key_expired || info->key_disabled ||
        info->key_invalid) {
@@ -992,15 +1054,20 @@ libbalsa_signature_info_to_gchar(GMimeGpgmeSigstat * info,
                attrs = g_string_append_c(attrs, ',');
            attrs = g_string_append(attrs, _(" invalid"));
        }
-       if (count > 1)
-           g_string_append_printf(msg, _("\nKey attributes:%s"), attrs->str);
-       else
-           g_string_append_printf(msg, _("\nKey attribute:%s"), attrs->str);
+       /* ngettext: string begins with a single space, so no space after
+        * the colon is correct punctuation (in English). */
+       g_string_append_printf(msg, ngettext("\nSubkey attribute:%s",
+                                            "\nSubkey attributes:%s",
+                                            count),
+                              attrs->str);
        g_string_free(attrs, TRUE);
     }
-    if (info->issuer_name)
-	g_string_append_printf(msg, _("\nIssuer name: %s"),
-			       info->issuer_name);
+    if (info->issuer_name) {
+	gchar * issuer = fix_EMail_info(g_strdup(info->issuer_name));
+
+	g_string_append_printf(msg, _("\nIssuer name: %s"), issuer);
+	g_free(issuer);
+    }
     if (info->issuer_serial)
 	g_string_append_printf(msg, _("\nIssuer serial number: %s"),
 			       info->issuer_serial);
@@ -1072,7 +1139,8 @@ static gboolean
 check_gpg_child(gpointer data)
 {
     spawned_gpg_T *spawned_gpg = (spawned_gpg_T *) data;
-    int status, bytes_read;
+    int status;
+    ssize_t bytes_read;
     gchar buffer[1024], *gpg_message;
     GtkWidget *dialog;
 
@@ -1093,13 +1161,13 @@ check_gpg_child(gpointer data)
 
     gpg_message =
 	g_locale_to_utf8(spawned_gpg->stderr_buf->str, -1, NULL,
-			 &bytes_read, NULL);
+			 NULL, NULL);
     gdk_threads_enter();
     if (WEXITSTATUS(status) > 0)
 	dialog =
 	    gtk_message_dialog_new(spawned_gpg->parent,
 				   GTK_DIALOG_DESTROY_WITH_PARENT,
-				   GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
+				   GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
 				   _
 				   ("Running gpg failed with return value %d:\n%s"),
 				   WEXITSTATUS(status), gpg_message);
@@ -1149,40 +1217,44 @@ libbalsa_gpgme_validity_to_gchar_short(gpgme_validity_t validity)
 }
 
 
+#define OID_EMAIL       "1.2.840.113549.1.9.1=#"
+#define OID_EMAIL_LEN   22
 static gchar *
 fix_EMail_info(gchar * str)
 {
-    gchar *p = strstr(str, "1.2.840.113549.1.9.1=#");
+    gchar *p;
     GString *result;
+
+    /* check for any EMail info */
+    p = strstr(str, OID_EMAIL);
     if (!p)
 	return str;
 
     *p = '\0';
-    p += 22;
+    p += OID_EMAIL_LEN;
     result = g_string_new(str);
-    result = g_string_append(result, "EMail=");
-    while (*p != '\0' && *p != ',') {
-	gchar x = 0;
+    while (p) {
+	gchar *next;
 
-	if (*p >= 'A' && *p <= 'F')
-	    x = (*p - 'A' + 10) << 4;
-	else if (*p >= 'a' && *p <= 'f')
-	    x = (*p - 'a' + 10) << 4;
-	else if (*p >= '0' && *p <= '9')
-	    x = (*p - '0') << 4;
-	p++;
-	if (*p != '\0' && *p != ',') {
-	    if (*p >= 'A' && *p <= 'F')
-		x += *p - 'A' + 10;
-	    else if (*p >= 'a' && *p <= 'f')
-		x += *p - 'a' + 10;
-	    else if (*p >= '0' && *p <= '9')
-		x += *p - '0';
-	    p++;
+	result = g_string_append(result, "EMail=");
+	/* convert the info from hex until we reach some other char */
+	while (g_ascii_isxdigit(*p)) {
+	    gchar c = g_ascii_xdigit_value(*p++) << 4;
+
+	    if (g_ascii_isxdigit(*p))
+		result =
+		    g_string_append_c(result, c + g_ascii_xdigit_value(*p++));
 	}
-	result = g_string_append_c(result, x);
+	
+	/* find more */
+	next = strstr(p, OID_EMAIL);
+	if (next) {
+	    *next = '\0';
+	    next += OID_EMAIL_LEN;
+	}
+	result = g_string_append(result, p);
+	p = next;
     }
-    result = g_string_append(result, p);
     g_free(str);
     p = result->str;
     g_string_free(result, FALSE);
@@ -1192,19 +1264,16 @@ fix_EMail_info(gchar * str)
 
 /* stuff to get a key fingerprint from a selection list */
 enum {
-    GPG_KEY_PTR_COLUMN = 0,
-    GPG_KEY_USER_ID_COLUMN,
+    GPG_KEY_USER_ID_COLUMN = 0,
     GPG_KEY_ID_COLUMN,
     GPG_KEY_LENGTH_COLUMN,
     GPG_KEY_VALIDITY_COLUMN,
-    GPG_KEY_TRUST_COLUMN,
+    GPG_KEY_PTR_COLUMN,
     GPG_KEY_NUM_COLUMNS
 };
 
 static gchar *col_titles[] =
-    { NULL, N_("User ID"), N_("Key ID"), N_("Length"), N_("Validity"),
-    N_("Owner trust")
-};
+    { N_("User ID"), N_("Key ID"), N_("Length"), N_("Validity") };
 
 /* callback function if a new row is selected in the list */
 static void
@@ -1237,6 +1306,7 @@ select_key_from_list(const gchar * name, gboolean is_secret,
     GtkTreeIter iter;
     gint i, last_col;
     gchar *prompt;
+    gchar *upcase_name;
     gpgme_protocol_t protocol;
     GtkWindow *parent;
     gpgme_key_t use_key = NULL;
@@ -1278,9 +1348,12 @@ select_key_from_list(const gchar * name, gboolean is_secret,
     gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);
 
     model =
-	gtk_tree_store_new(GPG_KEY_NUM_COLUMNS, G_TYPE_POINTER,
-			   G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT,
-			   G_TYPE_STRING, G_TYPE_STRING);
+	gtk_tree_store_new(GPG_KEY_NUM_COLUMNS,
+			   G_TYPE_STRING,   /* user ID */
+			   G_TYPE_STRING,   /* key ID */
+			   G_TYPE_INT,      /* length */
+			   G_TYPE_STRING,   /* validity (gpg encrypt only) */
+			   G_TYPE_POINTER); /* key */
 
     tree_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(model));
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
@@ -1289,42 +1362,69 @@ select_key_from_list(const gchar * name, gboolean is_secret,
 		     G_CALLBACK(key_selection_changed_cb), &use_key);
 
     /* add the keys */
+    upcase_name = g_ascii_strup(name, -1);
     while (keys) {
 	gpgme_key_t key = (gpgme_key_t) keys->data;
-	if (key->subkeys) {
-	    gchar *uid_info;
-	    if (key->uids && key->uids->uid)
-		uid_info = fix_EMail_info(g_strdup(key->uids->uid));
-	    else
-		uid_info = g_strdup("");
+	gpgme_subkey_t subkey = key->subkeys;
+	gpgme_user_id_t uid = key->uids;
+	gchar *uid_info = NULL;
+	gboolean uid_found;
+
+	/* find the relevant subkey */
+	while (subkey && ((is_secret && !subkey->can_sign) ||
+			  (!is_secret && !subkey->can_encrypt)))
+	    subkey = subkey->next;
+
+	/* find the relevant uid */
+	uid_found = FALSE;
+	while (uid && !uid_found) {
+	    g_free(uid_info);
+	    uid_info = fix_EMail_info(g_strdup(uid->uid));
+
+	    /* check the email field which may or may not be present */
+	    if (uid->email && !g_ascii_strcasecmp(uid->email, name))
+		uid_found = TRUE;
+	    else {
+		/* no email or no match, check the uid */
+		gchar * upcase_uid = g_ascii_strup(uid_info, -1);
+		
+		if (strstr(upcase_uid, upcase_name))
+		    uid_found = TRUE;
+		else
+		    uid = uid->next;
+		g_free(upcase_uid);
+	    }
+	}
+
+	/* append the element */
+	if (subkey && uid) {
 	    gtk_tree_store_append(GTK_TREE_STORE(model), &iter, NULL);
 	    gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
-			       GPG_KEY_PTR_COLUMN, key,
 			       GPG_KEY_USER_ID_COLUMN, uid_info,
-			       GPG_KEY_ID_COLUMN, key->subkeys->keyid,
-			       GPG_KEY_LENGTH_COLUMN,
-			       key->subkeys->length,
+			       GPG_KEY_ID_COLUMN, subkey->keyid,
+			       GPG_KEY_LENGTH_COLUMN, subkey->length,
 			       GPG_KEY_VALIDITY_COLUMN,
 			       libbalsa_gpgme_validity_to_gchar_short
-			       (key->uids->validity),
-			       GPG_KEY_TRUST_COLUMN,
-			       libbalsa_gpgme_validity_to_gchar_short
-			       (key->owner_trust), -1);
-	    g_free(uid_info);
+			       (uid->validity),
+			       GPG_KEY_PTR_COLUMN, key,
+			       -1);
 	}
+	g_free(uid_info);
 	keys = g_list_next(keys);
     }
+    g_free(upcase_name);
 
     g_object_unref(G_OBJECT(model));
-    last_col = (protocol == GPGME_PROTOCOL_CMS) ? GPG_KEY_LENGTH_COLUMN :
-	GPG_KEY_TRUST_COLUMN;
-    for (i = 1; i <= last_col; i++) {
+    /* show the validity only if we are asking for a gpg public key */
+    last_col = (protocol == GPGME_PROTOCOL_CMS || is_secret) ?
+	GPG_KEY_LENGTH_COLUMN :	GPG_KEY_VALIDITY_COLUMN;
+    for (i = 0; i <= last_col; i++) {
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
 
 	renderer = gtk_cell_renderer_text_new();
 	column =
-	    gtk_tree_view_column_new_with_attributes(col_titles[i],
+	    gtk_tree_view_column_new_with_attributes(_(col_titles[i]),
 						     renderer, "text", i,
 						     NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), column);
@@ -1343,6 +1443,49 @@ select_key_from_list(const gchar * name, gboolean is_secret,
 }
 
 
+/*
+ * Display a dialog to select whether a key with a low trust level shall be accepted
+ */
+static gboolean
+accept_low_trust_key(const gchar * name, gpgme_user_id_t uid,
+		     GMimeGpgmeContext * ctx)
+{
+    GtkWidget *dialog;
+    GtkWindow *parent;
+    gint result;
+    gchar *message1;
+    gchar *message2;
+
+    /* paranoia checks */
+    g_return_val_if_fail(ctx != NULL, FALSE);
+    g_return_val_if_fail(uid != NULL, FALSE);
+    parent = GTK_WINDOW(g_object_get_data(G_OBJECT(ctx), "parent-window"));
+    
+    /* build the message */
+    message1 =
+	g_strdup_printf(_("Insufficient trust for recipient %s"), name);
+    message2 =
+	g_strdup_printf(_("The validity of the key with user ID \"%s\" is \"%s\"."),
+			uid->uid,
+			libbalsa_gpgme_validity_to_gchar_short(uid->validity));
+    dialog = 
+	gtk_message_dialog_new_with_markup(parent,
+					   GTK_DIALOG_DESTROY_WITH_PARENT,
+					   GTK_MESSAGE_WARNING,
+					   GTK_BUTTONS_YES_NO,
+					   "<b>%s</b>\n\n%s\n%s",
+					   message1,
+					   message2,
+					   _("Use this key anyway?"));
+			      
+    /* ask the user */
+    result = gtk_dialog_run(GTK_DIALOG(dialog));
+    gtk_widget_destroy(dialog);
+
+    return result == GTK_RESPONSE_YES;
+}
+
+
 /*
  * display a dialog to read the passphrase
  */
@@ -1456,9 +1599,10 @@ get_passphrase_cb(void *opaque, const char *uid_hint,
 {
     GMimeGpgmeContext *context;
     gchar *passwd = NULL;
+    int foo;
 
     if (!opaque || !GMIME_IS_GPGME_CONTEXT(opaque)) {
-	write(fd, "\n", 1);
+	foo = write(fd, "\n", 1);
 	return GPG_ERR_USER_1;
     }
     context = GMIME_GPGME_CONTEXT(opaque);
@@ -1469,8 +1613,8 @@ get_passphrase_cb(void *opaque, const char *uid_hint,
 
     /* check if we have the passphrase already cached... */
     if ((passwd = check_cache(pcache, uid_hint, prev_was_bad))) {
-	write(fd, passwd, strlen(passwd));
-	write(fd, "\n", 1);
+	foo = write(fd, passwd, strlen(passwd));
+	foo = write(fd, "\n", 1);
 	wipe_string(passwd);
 	g_free(passwd);
 	return GPG_ERR_NO_ERROR;
@@ -1508,15 +1652,15 @@ get_passphrase_cb(void *opaque, const char *uid_hint,
 #endif				/* BALSA_USE_THREADS */
 
     if (!passwd) {
-	write(fd, "\n", 1);
+	foo = write(fd, "\n", 1);
 	return GPG_ERR_CANCELED;
     }
 
     /* send the passphrase and erase the string */
-    write(fd, passwd, strlen(passwd));
+    foo = write(fd, passwd, strlen(passwd));
     wipe_string(passwd);
     g_free(passwd);
-    write(fd, "\n", 1);
+    foo = write(fd, "\n", 1);
     return GPG_ERR_NO_ERROR;
 }
 
@@ -1538,7 +1682,7 @@ gpg_updates_trustdb(void)
 		g_strdup_printf("%s/.gnupg/trustdb.gpg.lock",
 				pwent->pw_dir);
     if (!stat(lockname, &stat_buf)) {
-	libbalsa_information(LIBBALSA_INFORMATION_ERROR,
+	libbalsa_information(LIBBALSA_INFORMATION_ERROR, "%s%s",
 			     _
 			     ("GnuPG is rebuilding the trust database and is currently unavailable."),
 			     _("Try again later."));
