@@ -1,4 +1,4 @@
-/* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
+/* -*-mode:c; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
  * Copyright (C) 1997-2002 Stuart Parmenter and others,
@@ -39,6 +39,7 @@
 #include "libbalsa.h"
 #include "libbalsa_private.h"
 
+#include "server.h"
 #include "send.h"
 #include "misc.h"
 #include "information.h"
@@ -192,20 +193,21 @@ send_message_info_destroy(SendMessageInfo *smi)
 
 
 #if HAVE_GPGME
-static gint
+static LibBalsaMsgCreateResult
 libbalsa_create_rfc2440_buffer(LibBalsaMessageBody *body, GMimePart *mime_part,
-			       GtkWindow * parent);
+			       GtkWindow * parent, GError ** error);
 static LibBalsaMsgCreateResult
 do_multipart_crypto(LibBalsaMessage * message, GMimeObject ** mime_root,
-		    GtkWindow * parent);
+		    GtkWindow * parent, GError ** error);
 #endif
 
 static guint balsa_send_message_real(SendMessageInfo* info);
 static LibBalsaMsgCreateResult
 libbalsa_message_create_mime_message(LibBalsaMessage* message,
-				     gboolean flow, gboolean postponing);
+				     gboolean flow, gboolean postponing,
+				     GError ** error);
 static LibBalsaMsgCreateResult libbalsa_create_msg(LibBalsaMessage * message,
-				    gboolean flow);
+						   gboolean flow, GError ** error);
 static LibBalsaMsgCreateResult
 libbalsa_fill_msg_queue_item_from_queu(LibBalsaMessage * message,
                                        MessageQueueItem *mqi);
@@ -390,7 +392,7 @@ libbalsa_message_queue(LibBalsaMessage * message, LibBalsaMailbox * outbox,
 #if ENABLE_ESMTP
                        LibBalsaSmtpServer * smtp_server,
 #endif /* ESMTP */
-		       gboolean flow)
+		       gboolean flow, GError ** error)
 {
     LibBalsaMsgCreateResult result;
 #if ENABLE_ESMTP
@@ -401,7 +403,7 @@ libbalsa_message_queue(LibBalsaMessage * message, LibBalsaMailbox * outbox,
 
     g_return_val_if_fail(message, LIBBALSA_MESSAGE_CREATE_ERROR);
 
-    if ((result = libbalsa_create_msg(message, flow)) !=
+    if ((result = libbalsa_create_msg(message, flow, error)) !=
 	LIBBALSA_MESSAGE_CREATE_OK)
         return result;
 
@@ -458,18 +460,6 @@ libbalsa_message_queue(LibBalsaMessage * message, LibBalsaMailbox * outbox,
     return rc ?  LIBBALSA_MESSAGE_CREATE_OK : LIBBALSA_MESSAGE_QUEUE_ERROR;
 }
 
-static void
-lbs_change_flags(LibBalsaMessage * msg, LibBalsaMessageFlag set,
-                 LibBalsaMessageFlag clear)
-{
-    GArray *array = g_array_new(FALSE, FALSE, sizeof(guint));
-
-    g_array_append_val(array, msg->msgno);
-    libbalsa_mailbox_messages_change_flags(msg->mailbox, array, set,
-                                           clear);
-    g_array_free(array, TRUE);
-}
-
 /* libbalsa_message_send:
    send the given messsage (if any, it can be NULL) and all the messages
    in given outbox.
@@ -485,7 +475,8 @@ libbalsa_message_send(LibBalsaMessage * message, LibBalsaMailbox * outbox,
                       LibBalsaMailbox * fccbox,
                       LibBalsaFccboxFinder finder,
                       LibBalsaSmtpServer * smtp_server,
-                      gboolean flow, gboolean debug)
+                      gboolean flow, gboolean debug,
+		      GError ** error)
 {
     LibBalsaMsgCreateResult result = LIBBALSA_MESSAGE_CREATE_OK;
 
@@ -494,7 +485,7 @@ libbalsa_message_send(LibBalsaMessage * message, LibBalsaMailbox * outbox,
 
     if (message != NULL)
         result = libbalsa_message_queue(message, outbox, fccbox,
-                                        smtp_server, flow);
+                                        smtp_server, flow, error);
 
     if (result == LIBBALSA_MESSAGE_CREATE_OK
         && !lbs_process_queue(outbox, finder, smtp_server, debug))
@@ -506,12 +497,13 @@ libbalsa_message_send(LibBalsaMessage * message, LibBalsaMailbox * outbox,
 LibBalsaMsgCreateResult
 libbalsa_message_send(LibBalsaMessage* message, LibBalsaMailbox* outbox,
 		      LibBalsaMailbox* fccbox, LibBalsaFccboxFinder finder,
-                      gboolean flow, gboolean debug)
+                      gboolean flow, gboolean debug,
+		      GError ** error)
 {
     LibBalsaMsgCreateResult result = LIBBALSA_MESSAGE_CREATE_OK;
 
     if (message != NULL)
- 	result = libbalsa_message_queue(message, outbox, fccbox, flow);
+ 	result = libbalsa_message_queue(message, outbox, fccbox, flow, error);
     if (result == LIBBALSA_MESSAGE_CREATE_OK)
  	if (!libbalsa_process_queue(outbox, finder, debug))
  	    return LIBBALSA_MESSAGE_SEND_ERROR;
@@ -629,14 +621,14 @@ lbs_process_queue(LibBalsaMailbox * outbox, LibBalsaFccboxFinder finder,
 	const gchar *smtp_server_name;
         LibBalsaMsgCreateResult created;
 
-	msg = libbalsa_mailbox_get_message(outbox, msgno);
-        if (LIBBALSA_MESSAGE_HAS_FLAG(msg,
-                                      (LIBBALSA_MESSAGE_FLAG_FLAGGED |
-                                       LIBBALSA_MESSAGE_FLAG_DELETED))) {
-	    g_object_unref(msg);
+        /* Skip this message if it either FLAGGED or DELETED: */
+        if (!libbalsa_mailbox_msgno_has_flags
+            (outbox, msgno, 0,
+             (LIBBALSA_MESSAGE_FLAG_FLAGGED |
+              LIBBALSA_MESSAGE_FLAG_DELETED)))
             continue;
-	}
 
+	msg = libbalsa_mailbox_get_message(outbox, msgno);
         libbalsa_message_body_ref(msg, TRUE, TRUE);
         smtp_server_name =
             libbalsa_message_get_user_header(msg, "X-Balsa-SmtpServer");
@@ -658,7 +650,8 @@ lbs_process_queue(LibBalsaMailbox * outbox, LibBalsaFccboxFinder finder,
 	} else {
 	    InternetAddress *ia;
 
-	    lbs_change_flags(msg, LIBBALSA_MESSAGE_FLAG_FLAGGED, 0);
+	    libbalsa_message_change_flags(msg,
+                                          LIBBALSA_MESSAGE_FLAG_FLAGGED, 0);
 	    /*
 	       The message needs to be filtered and the newlines converted to
 	       \r\n because internally the lines foolishly terminate with the
@@ -901,7 +894,8 @@ handle_successful_send(smtp_message_t message, void *be_verbose)
       mqi->refcount--;
 
     if(mqi != NULL && mqi->orig != NULL && mqi->orig->mailbox)
-	lbs_change_flags(mqi->orig, 0, LIBBALSA_MESSAGE_FLAG_FLAGGED);
+	libbalsa_message_change_flags(mqi->orig, 0,
+                                      LIBBALSA_MESSAGE_FLAG_FLAGGED);
     else printf("mqi: %p mqi->orig: %p mqi->orig->mailbox: %p\n",
                   mqi, mqi ? mqi->orig : NULL, 
                   mqi&&mqi->orig ? mqi->orig->mailbox : NULL);
@@ -916,8 +910,9 @@ handle_successful_send(smtp_message_t message, void *be_verbose)
 	    if (mqi->orig->mailbox && fccurl) {
                 LibBalsaMailbox *fccbox = mqi->finder(fccurl);
                 GError *err = NULL;
-                lbs_change_flags(mqi->orig, 0, LIBBALSA_MESSAGE_FLAG_NEW |
-                                 LIBBALSA_MESSAGE_FLAG_FLAGGED);
+                libbalsa_message_change_flags(mqi->orig, 0,
+                                              LIBBALSA_MESSAGE_FLAG_NEW |
+                                              LIBBALSA_MESSAGE_FLAG_FLAGGED);
 		libbalsa_mailbox_sync_storage(mqi->orig->mailbox, FALSE);
                 remove = libbalsa_message_copy(mqi->orig, fccbox, &err);
                 if(!remove) {
@@ -931,9 +926,9 @@ handle_successful_send(smtp_message_t message, void *be_verbose)
             /* If copy failed, mark the message again as flagged -
                otherwise it will get resent again. And again, and
                again... */
-            lbs_change_flags(mqi->orig, remove ?
-                             LIBBALSA_MESSAGE_FLAG_DELETED :
-                             LIBBALSA_MESSAGE_FLAG_FLAGGED, 0);
+            libbalsa_message_change_flags(mqi->orig, remove ?
+                                          LIBBALSA_MESSAGE_FLAG_DELETED :
+                                          LIBBALSA_MESSAGE_FLAG_FLAGGED, 0);
 	}
     } else {
 	/* Check whether it was a problem with transfer. */
@@ -1007,7 +1002,7 @@ libbalsa_smtp_event_cb (smtp_session_t session, int event_no, void *arg, ...)
         snprintf (buf, sizeof buf, _("From %s: %d %s"), 
 	          mailbox, status->code, status->text);
         g_strchomp(buf);
-	libbalsa_information(LIBBALSA_INFORMATION_MESSAGE, buf);
+	libbalsa_information(LIBBALSA_INFORMATION_DEBUG, buf);
         break;
     case SMTP_EV_RCPTSTATUS:
         mailbox = va_arg (ap, const char *);
@@ -1022,7 +1017,7 @@ libbalsa_smtp_event_cb (smtp_session_t session, int event_no, void *arg, ...)
         snprintf (buf, sizeof buf, _("To %s: %d %s"),
 	          mailbox, status->code, status->text);
         g_strchomp(buf);
-	libbalsa_information(LIBBALSA_INFORMATION_MESSAGE, buf);
+	libbalsa_information(LIBBALSA_INFORMATION_DEBUG, buf);
         break;
     case SMTP_EV_MESSAGEDATA:
         message = va_arg (ap, smtp_message_t);
@@ -1047,7 +1042,7 @@ libbalsa_smtp_event_cb (smtp_session_t session, int event_no, void *arg, ...)
 	snprintf (buf, sizeof buf, "%d %s", status->code, status->text);
         g_strchomp(buf);
 	MSGSENDTHREAD(threadmsg, MSGSENDTHREADPROGRESS, buf, NULL, NULL, 0);
-	libbalsa_information(LIBBALSA_INFORMATION_MESSAGE, buf);
+	libbalsa_information(LIBBALSA_INFORMATION_DEBUG, buf);
         /* Reset 'mqi->sent' for the next message (i.e. bcc copy) */
         mqi = smtp_message_get_application_data (message);
         if (mqi != NULL) {
@@ -1181,13 +1176,14 @@ libbalsa_process_queue(LibBalsaMailbox* outbox, LibBalsaFccboxFinder finder,
     for (msgno = libbalsa_mailbox_total_messages(outbox);
 	 msgno > 0; msgno--) {
         LibBalsaMsgCreateResult created;
-	msg = libbalsa_mailbox_get_message(outbox, msgno);
 
-        if (LIBBALSA_MESSAGE_HAS_FLAG(msg,
-                                      (LIBBALSA_MESSAGE_FLAG_FLAGGED |
-                                       LIBBALSA_MESSAGE_FLAG_DELETED)))
+        if (libbalsa_mailbox_msgno_has_flags(outbox, msgno, 
+                                             (LIBBALSA_MESSAGE_FLAG_FLAGGED |
+                                              LIBBALSA_MESSAGE_FLAG_DELETED),
+                                             0))
             continue;
 
+	msg = libbalsa_mailbox_get_message(outbox, msgno);
         libbalsa_message_body_ref(msg, TRUE, TRUE); /* FIXME: do we need
                                                       * all headers? */
 	new_message = msg_queue_item_new(finder);
@@ -1197,7 +1193,8 @@ libbalsa_process_queue(LibBalsaMailbox* outbox, LibBalsaFccboxFinder finder,
 	if (created != LIBBALSA_MESSAGE_CREATE_OK) {
 	    msg_queue_item_destroy(new_message);
 	} else {
-	    lbs_change_flags(msg, LIBBALSA_MESSAGE_FLAG_FLAGGED, 0);
+	    libbalsa_message_change_flags(msg,
+                                          LIBBALSA_MESSAGE_FLAG_FLAGGED, 0);
 	    if (mqi)
 		mqi->next_message = new_message;
 	    else
@@ -1234,8 +1231,9 @@ handle_successful_send(MessageQueueItem *mqi, LibBalsaFccboxFinder finder)
         const gchar *fccurl =
             libbalsa_message_get_user_header(mqi->orig, "X-Balsa-Fcc");
 
-        lbs_change_flags(mqi->orig, 0, LIBBALSA_MESSAGE_FLAG_NEW |
-                         LIBBALSA_MESSAGE_FLAG_FLAGGED);
+        libbalsa_message_change_flags(mqi->orig, 0,
+                                      LIBBALSA_MESSAGE_FLAG_NEW |
+                                      LIBBALSA_MESSAGE_FLAG_FLAGGED);
 	libbalsa_mailbox_sync_storage(mqi->orig->mailbox, FALSE);
 
         if (mqi->orig->mailbox && fccurl) {
@@ -1246,9 +1244,9 @@ handle_successful_send(MessageQueueItem *mqi, LibBalsaFccboxFinder finder)
         /* If copy failed, mark the message again as flagged -
            otherwise it will get resent again. And again, and
            again... */
-        lbs_change_flags(mqi->orig, remove ?
-                         LIBBALSA_MESSAGE_FLAG_DELETED :
-                         LIBBALSA_MESSAGE_FLAG_FLAGGED, 0);
+        libbalsa_message_change_flags(mqi->orig, remove ?
+                                      LIBBALSA_MESSAGE_FLAG_DELETED :
+                                      LIBBALSA_MESSAGE_FLAG_FLAGGED, 0);
     }
     mqi->status = MQI_SENT;
 }
@@ -1285,6 +1283,10 @@ monitor_cb (const char *buf, int buflen, int writing, void *arg)
     }
 
  fputs (writing ? "C: " : "S: ", fp);
+ if (writing && g_ascii_strncasecmp(buf, "auth plain", 10) == 0) {
+     fputs("AUTH (details hidden)\n", fp);
+     return;
+ }
  if (fwrite (buf, 1, buflen, fp) != (size_t) buflen)
    /* FIXME */ return;
  if (buf[buflen - 1] != '\n')
@@ -1574,7 +1576,7 @@ get_tz_offset(time_t *t)
 
 static LibBalsaMsgCreateResult
 libbalsa_message_create_mime_message(LibBalsaMessage* message, gboolean flow,
-				     gboolean postponing)
+				     gboolean postponing, GError ** error)
 {
     gchar **mime_type;
     GMimeObject *mime_root = NULL;
@@ -1704,16 +1706,16 @@ libbalsa_message_create_mime_message(LibBalsaMessage* message, gboolean flow,
 	    if (!postponing && body == body->message->body_list &&
 		message->gpg_mode > 0 &&
 		(message->gpg_mode & LIBBALSA_PROTECT_OPENPGP) != 0) {
-		gint result = 
+		LibBalsaMsgCreateResult crypt_res =
 		    libbalsa_create_rfc2440_buffer(body,
 			                           GMIME_PART(mime_part),
-						   parent);
+						   parent, error);
 
-		if (result != LIBBALSA_MESSAGE_CREATE_OK) {
+		if (crypt_res != LIBBALSA_MESSAGE_CREATE_OK) {
 		    g_object_unref(G_OBJECT(mime_part));
 		    if (mime_root)
 			g_object_unref(G_OBJECT(mime_root));
-		    return LIBBALSA_MESSAGE_CREATE_ERROR;
+		    return crypt_res;
 		}
 	    }
 #else
@@ -1735,7 +1737,7 @@ libbalsa_message_create_mime_message(LibBalsaMessage* message, gboolean flow,
 #ifdef HAVE_GPGME
     if (message->body_list != NULL && !postponing) {
 	LibBalsaMsgCreateResult crypt_res =
-	    do_multipart_crypto(message, &mime_root, parent);
+	    do_multipart_crypto(message, &mime_root, parent, error);
 	if (crypt_res != LIBBALSA_MESSAGE_CREATE_OK)
 	    return crypt_res;
     }
@@ -1839,9 +1841,11 @@ libbalsa_message_postpone(LibBalsaMessage * message,
     gboolean retval;
     GError *err = NULL;
 
+    /* in postpone mode no crypto operation is triggered, so we don't need to
+       pass the error */
     if (!message->mime_msg
         && libbalsa_message_create_mime_message(message, flow,
-                                                TRUE) !=
+                                                TRUE, NULL) !=
         LIBBALSA_MESSAGE_CREATE_OK)
         return FALSE;
 
@@ -1903,12 +1907,12 @@ libbalsa_set_message_id(GMimeMessage * mime_message)
    copies message to msg.
 */ 
 static LibBalsaMsgCreateResult
-libbalsa_create_msg(LibBalsaMessage * message, gboolean flow)
+libbalsa_create_msg(LibBalsaMessage * message, gboolean flow, GError ** error)
 {
     if (!message->mime_msg) {
 	LibBalsaMsgCreateResult res =
 	    libbalsa_message_create_mime_message(message, flow,
-						 FALSE);
+						 FALSE, error);
 	if (res != LIBBALSA_MESSAGE_CREATE_OK)
 	    return res;
     }
@@ -1966,9 +1970,9 @@ lb_send_from(LibBalsaMessage *message)
     return ia->value.addr;
 }
 
-static gint
+static LibBalsaMsgCreateResult
 libbalsa_create_rfc2440_buffer(LibBalsaMessageBody *body, GMimePart *mime_part,
-			       GtkWindow * parent)
+			       GtkWindow * parent, GError ** error)
 {
     LibBalsaMessage *message = body->message;
     gint mode = message->gpg_mode;
@@ -1980,7 +1984,7 @@ libbalsa_create_rfc2440_buffer(LibBalsaMessageBody *body, GMimePart *mime_part,
 	    if (!libbalsa_rfc2440_sign_encrypt(mime_part, 
 					       lb_send_from(message),
 					       NULL, FALSE,
-					       parent))
+					       parent, error))
 		return LIBBALSA_MESSAGE_SIGN_ERROR;
 	    break;
 	case LIBBALSA_PROTECT_ENCRYPT:
@@ -2008,14 +2012,14 @@ libbalsa_create_rfc2440_buffer(LibBalsaMessageBody *body, GMimePart *mime_part,
 					              lb_send_from(message),
 						      encrypt_for,
 						      always_trust,
-						      parent);
+						      parent, error);
 		else
 		    result = 
 			libbalsa_rfc2440_sign_encrypt(mime_part, 
 						      NULL,
 						      encrypt_for,
 						      always_trust,
-						      parent);
+						      parent, error);
 		g_list_foreach(encrypt_for, (GFunc) g_free, NULL);
 		g_list_free(encrypt_for);
 		if (!result)
@@ -2023,8 +2027,7 @@ libbalsa_create_rfc2440_buffer(LibBalsaMessageBody *body, GMimePart *mime_part,
 	    }
 	    break;
 	default:
-	    g_error("illegal gpg_mode %d (" __FILE__ " line %d)",
-		    mode, __LINE__);
+	    g_assert_not_reached();
 	}
 
     return LIBBALSA_MESSAGE_CREATE_OK;
@@ -2034,7 +2037,7 @@ libbalsa_create_rfc2440_buffer(LibBalsaMessageBody *body, GMimePart *mime_part,
 /* handle rfc2633 and rfc3156 signing and/or encryption of a message */
 static LibBalsaMsgCreateResult
 do_multipart_crypto(LibBalsaMessage * message, GMimeObject ** mime_root,
-		    GtkWindow * parent)
+		    GtkWindow * parent, GError ** error)
 {
     gpgme_protocol_t protocol;
     gboolean always_trust;
@@ -2062,7 +2065,7 @@ do_multipart_crypto(LibBalsaMessage * message, GMimeObject ** mime_root,
 	case LIBBALSA_PROTECT_SIGN:   /* sign message */
 	    if (!libbalsa_sign_mime_object(mime_root,
 					   lb_send_from(message),
-					   protocol, parent))
+					   protocol, parent, error))
 		return LIBBALSA_MESSAGE_SIGN_ERROR;
 	    break;
 	case LIBBALSA_PROTECT_ENCRYPT:
@@ -2089,12 +2092,13 @@ do_multipart_crypto(LibBalsaMessage * message, GMimeObject ** mime_root,
 			libbalsa_sign_encrypt_mime_object(mime_root,
 							  lb_send_from(message),
 							  encrypt_for, protocol,
-							  always_trust, parent);
+							  always_trust, parent,
+							  error);
 		else
 		    success = 
 			libbalsa_encrypt_mime_object(mime_root, encrypt_for,
 						     protocol, always_trust,
-						     parent);
+						     parent, error);
 		g_list_free(encrypt_for);
 		
 		if (!success)
