@@ -27,11 +27,13 @@
 
 #include <string.h>
 #include <libesmtp.h>
+
 #ifdef HAVE_GNOME
 #include <gnome.h>
 #endif                          /* HAVE_GNOME */
 
-
+#include "libbalsa.h"
+#include "server.h"
 #include "smtp-server.h"
 #include "libbalsa-conf.h"
 #include "misc.h"
@@ -71,7 +73,9 @@ libbalsa_smtp_server_finalize(GObject * object)
 
     auth_destroy_context(smtp_server->authctx);
     g_free(smtp_server->name);
+#if HAVE_SMTP_TLS_CLIENT_CERTIFICATE
     g_free(smtp_server->cert_passphrase);
+#endif                          /* HAVE_SMTP_TLS_CLIENT_CERTIFICATE */
 
     G_OBJECT_CLASS(parent_class)->finalize(object);
 }
@@ -255,6 +259,7 @@ libbalsa_smtp_server_get_name(LibBalsaSmtpServer * smtp_server)
     return smtp_server ? smtp_server->name : _("Default");
 }
 
+#if HAVE_SMTP_TLS_CLIENT_CERTIFICATE
 void
 libbalsa_smtp_server_set_cert_passphrase(LibBalsaSmtpServer * smtp_server,
                                          const gchar * passphrase)
@@ -268,6 +273,7 @@ libbalsa_smtp_server_get_cert_passphrase(LibBalsaSmtpServer * smtp_server)
 {
     return smtp_server->cert_passphrase;
 }
+#endif                          /* HAVE_SMTP_TLS_CLIENT_CERTIFICATE */
 
 auth_context_t
 libbalsa_smtp_server_get_authctx(LibBalsaSmtpServer * smtp_server)
@@ -323,8 +329,8 @@ struct smtp_server_dialog_info {
     GtkWidget *host;
     GtkWidget *user;
     GtkWidget *pass;
-    GtkWidget *tlsm;
 #if HAVE_SMTP_TLS_CLIENT_CERTIFICATE
+    GtkWidget *tlsm;
     GtkWidget *cert;
 #endif                          /* HAVE_SMTP_TLS_CLIENT_CERTIFICATE */
     GtkWidget *split_button;
@@ -426,6 +432,7 @@ smtp_server_response(GtkDialog * dialog, gint response,
         libbalsa_server_set_password(server,
                                      gtk_entry_get_text(GTK_ENTRY
                                                         (sdi->pass)));
+#if HAVE_SMTP_TLS_CLIENT_CERTIFICATE
         switch (gtk_combo_box_get_active(GTK_COMBO_BOX(sdi->tlsm))) {
         case 0:
             server->tls_mode = Starttls_DISABLED;
@@ -439,7 +446,6 @@ smtp_server_response(GtkDialog * dialog, gint response,
         default:
             break;
         }
-#if HAVE_SMTP_TLS_CLIENT_CERTIFICATE
         libbalsa_smtp_server_set_cert_passphrase(sdi->smtp_server,
                                                  gtk_entry_get_text
                                                  (GTK_ENTRY(sdi->cert)));
@@ -574,12 +580,12 @@ libbalsa_smtp_server_dialog(LibBalsaSmtpServer * smtp_server,
     g_signal_connect(sdi->pass, "changed", G_CALLBACK(smtp_server_changed),
                      sdi);
 
+#if HAVE_SMTP_TLS_CLIENT_CERTIFICATE
     smtp_server_add_widget(table, ++row, _("Use _TLS:"), sdi->tlsm =
                            smtp_server_tls_widget(smtp_server));
     g_signal_connect(sdi->tlsm, "changed", G_CALLBACK(smtp_server_changed),
                      sdi);
 
-#if HAVE_SMTP_TLS_CLIENT_CERTIFICATE
     smtp_server_add_widget(table, ++row, _("C_ertificate Pass Phrase:"),
                            sdi->cert = gtk_entry_new());
     gtk_entry_set_visibility(GTK_ENTRY(sdi->cert), FALSE);
