@@ -21,13 +21,6 @@
 
 #define _XOPEN_SOURCE 500
 
-#include "balsa-druid-page-directory.h"
-
-#include "balsa-app.h"
-#include "save-restore.h"
-#include "misc.h"
-#include "url.h"
-
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -35,6 +28,16 @@
 #include <unistd.h>
 #include <errno.h>
 
+#include "balsa-druid-page-directory.h"
+
+#include "i18n.h"
+#include "balsa-app.h"
+#include "save-restore.h"
+#include "misc.h"
+#include "url.h"
+#include "server.h"
+
+
 static gchar *init_mbnames[NUM_EDs] = {
 #if defined(ENABLE_TOUCH_UI)
     "_In:", "_Out:", "_Sent:", "_Drafts:", "_Trash:"
