@@ -34,6 +34,7 @@
 
 #include "filter-funcs.h"
 #include "misc.h"
+#include "server.h"
 #include "smtp-server.h"
 #include "balsa-app.h"
 #include "save-restore.h"
@@ -104,7 +105,7 @@ ask_password_real(LibBalsaServer * server, LibBalsaMailbox * mbox)
         server->remember_passwd = 
             !!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rememb));
         libbalsa_server_set_password(server, passwd);
-        if(old_rem != server->remember_passwd)
+        if( server->remember_passwd || old_rem )
             libbalsa_server_config_changed(server);
     }
     gtk_widget_destroy(dialog);
@@ -359,12 +360,20 @@ balsa_app_init(void)
     balsa_app.date_string = g_strdup(DEFAULT_DATE_FORMAT);
 
     /* printing */
+#ifdef HAVE_GTK_PRINT
+    balsa_app.print_settings = gtk_print_settings_new();
+    balsa_app.page_setup = gtk_page_setup_new();
+#else
     balsa_app.paper_size = g_strdup(DEFAULT_PAPER_SIZE);
+#endif
 
     balsa_app.print_header_font = g_strdup(DEFAULT_PRINT_HEADER_FONT);
     balsa_app.print_footer_font = g_strdup(DEFAULT_PRINT_FOOTER_FONT);
     balsa_app.print_body_font   = g_strdup(DEFAULT_PRINT_BODY_FONT);
     balsa_app.print_highlight_cited = FALSE;
+#ifdef HAVE_GTK_PRINT
+    balsa_app.print_highlight_phrases = FALSE;
+#endif
 
     /* address book */
     balsa_app.address_book_list = NULL;
@@ -395,8 +404,10 @@ balsa_app_init(void)
     balsa_app.notify_new_mail_sound = 1;
     balsa_app.notify_new_mail_dialog = 0;
 
+#if !GTK_CHECK_VERSION(2, 11, 0)
     /* Tooltips */
     balsa_app.tooltips = gtk_tooltips_new();
+#endif                          /* !GTK_CHECK_VERSION(2, 11, 0) */
 
     /* Local and IMAP */
     balsa_app.local_scan_depth = 1;
@@ -760,7 +771,14 @@ gboolean
 balsa_find_iter_by_data(GtkTreeIter * iter , gpointer data)
 {
     struct balsa_find_iter_by_data_info bf;
-    GtkTreeModel *model = GTK_TREE_MODEL(balsa_app.mblist_tree_store);
+    GtkTreeModel *model;
+
+    /* We may call it from initial config, it's ok for
+       mblist_tree_store not to exist. */
+    if(!balsa_app.mblist_tree_store)
+        return FALSE;
+
+    model = GTK_TREE_MODEL(balsa_app.mblist_tree_store);
 
     bf.iter = iter;
     bf.data = data;
