@@ -85,13 +85,13 @@
 #define MESSAGEBOX_WIDTH 450
 #define MESSAGEBOX_HEIGHT 150
 
-#define DEFAULT_MESSAGE_FONT "monospace 10"
-#define DEFAULT_SUBJECT_FONT "helvetica Bold 10"
+#define DEFAULT_MESSAGE_FONT "Monospace 10"
+#define DEFAULT_SUBJECT_FONT "Monospace Bold 10"
 #define DEFAULT_DATE_FORMAT "%x %X"
 #define DEFAULT_PAPER_SIZE "A4"
-#define DEFAULT_PRINT_HEADER_FONT "Bitstream Charter Bold 12"
-#define DEFAULT_PRINT_BODY_FONT "Courier Regular 11"
-#define DEFAULT_PRINT_FOOTER_FONT "Bitstream Charter Bold Italic 8"
+#define DEFAULT_PRINT_HEADER_FONT "Monospace Regular 10"
+#define DEFAULT_PRINT_BODY_FONT "Monospace Regular 10"
+#define DEFAULT_PRINT_FOOTER_FONT "Sans Regular 8"
 #define DEFAULT_SELECTED_HDRS "from to date cc subject"
 #define DEFAULT_MESSAGE_TITLE_FORMAT "Message from %F: %s"
 #define DEFAULT_ENCODING ENC8BIT
@@ -121,6 +121,7 @@ enum _ShownHeaders {
 };
 
 
+#if !HAVE_GTKSPELL
 /* The different spell modules available to the program. */
 #define NUM_PSPELL_MODULES 2
 typedef enum _SpellCheckModule SpellCheckModule;
@@ -143,6 +144,7 @@ enum _SpellCheckSuggestMode {
     SPELL_CHECK_SUGGEST_BAD_SPELLERS
 };
 const gchar **spell_check_suggest_mode_name;
+#endif                          /* HAVE_GTKSPELL */
 
 
 typedef enum _BalsaMDNReply BalsaMDNReply;
@@ -152,6 +154,12 @@ enum _BalsaMDNReply {
     BALSA_MDN_REPLY_ALWAYS,
 };
 
+typedef enum _MwActionAfterMove MwActionAfterMove;
+enum _MwActionAfterMove {
+    NEXT_UNREAD,
+    NEXT,
+    CLOSE
+};
 
 /* global balsa application structure */
 extern struct BalsaApplication {
@@ -162,13 +170,9 @@ extern struct BalsaApplication {
     GSList* filters;
 
     gchar *local_mail_directory;
+
 #if ENABLE_ESMTP
-    gchar *smtp_server;
-    gchar *smtp_user;
-    gchar *smtp_passphrase;
-    auth_context_t smtp_authctx;
-    gint smtp_tls_mode;
-    gchar *smtp_certificate_passphrase;
+    GSList *smtp_servers;
 #endif
 
     /* folder scanning */
@@ -294,6 +298,9 @@ extern struct BalsaApplication {
     gboolean source_escape_specials;
     gboolean debug;
 
+    /* what to do with message window after moving the message */
+    MwActionAfterMove mw_action_after_move;
+
     /* external editor */
     gboolean edit_headers;
 
@@ -323,10 +330,17 @@ extern struct BalsaApplication {
     gchar *subject_font;
 
     /* encoding stuff */
-    guint encoding_style;
     gchar *date_string;
 
     /* printing */
+#ifdef HAVE_GTK_PRINT
+    GtkPageSetup *page_setup;
+    GtkPrintSettings *print_settings;
+    gdouble margin_left;
+    gdouble margin_top;
+    gdouble margin_right;
+    gdouble margin_bottom;
+#else
     gchar* paper_size; /* A4 or Letter */
     gchar* margin_left;
     gchar* margin_top;
@@ -336,10 +350,14 @@ extern struct BalsaApplication {
     gchar* print_layout;
     gchar* paper_orientation;
     gchar* page_orientation;
+#endif
     gchar* print_header_font;  /* font for printing headers */
     gchar* print_body_font;    /* font for printing text parts */
     gchar* print_footer_font;  /* font for printing footers */
     gboolean print_highlight_cited;
+#ifdef HAVE_GTK_PRINT
+    gboolean print_highlight_phrases;
+#endif
 
     /* compose */
     gchar *compose_headers;
@@ -355,11 +373,15 @@ extern struct BalsaApplication {
     LibBalsaAddressBook *default_address_book;
 
     /* spell checking */
+#if HAVE_GTKSPELL
+    gchar *spell_check_lang;
+#else                           /* HAVE_GTKSPELL */
     SpellCheckModule module;
     SpellCheckSuggestMode suggestion_mode;
     guint ignore_size;
     gboolean check_sig;
     gboolean check_quoted;
+#endif                          /* HAVE_GTKSPELL */
 
     /* Information messages */
     BalsaInformationShow information_message;
@@ -368,8 +390,10 @@ extern struct BalsaApplication {
     BalsaInformationShow debug_message;
     BalsaInformationShow fatal_message;
 
+#if !GTK_CHECK_VERSION(2, 11, 0)
     /* Tooltips */
     GtkTooltips *tooltips;
+#endif                          /* !GTK_CHECK_VERSION(2, 11, 0) */
 
     /* how to act if a MDN request is received */
     BalsaMDNReply mdn_reply_clean;
@@ -388,8 +412,11 @@ extern struct BalsaApplication {
     gboolean has_smime;
 #endif 
 
+    /* Most recently used lists */
     GList *folder_mru;
     GList *fcc_mru;
+    GList *pipe_cmds;
+
     gboolean expunge_on_close;
 
     /* use as default email client for GNOME */
@@ -419,11 +446,4 @@ BalsaIndex* balsa_find_index_by_mailbox(LibBalsaMailbox* mailbox);
 
 void  balsa_remove_children_mailbox_nodes(BalsaMailboxNode * mbnode);
 
-GtkWidget *create_label(const gchar * label, GtkWidget * table, gint row);
-GtkWidget *create_entry(GtkDialog *mcw, GtkWidget * table, 
-			GtkSignalFunc func, gpointer data, gint row, 
-			const gchar * initval, GtkWidget* hotlabel);
-GtkWidget *create_check(GtkDialog *mcw, const gchar * label, 
-			GtkWidget * table, gint row, gboolean initval);
-
 #endif				/* __BALSA_APP_H__ */
