@@ -112,9 +112,6 @@ static void balsa_message_init(BalsaMessage * bm);
 
 static void balsa_message_destroy(GtkObject * object);
 
-static void bm_message_weak_ref_cb(BalsaMessage * bm,
-                                   LibBalsaMessage * message);
-
 static void display_headers(BalsaMessage * bm);
 static void display_content(BalsaMessage * bm);
 
@@ -268,8 +265,10 @@ bm_header_tl_buttons(BalsaMessage * bm)
     GtkWidget *vbox;
     GtkWidget *button;
 
+#if !GTK_CHECK_VERSION(2, 11, 0)
     /* the event box is needed to set the background correctly */
     GtkTooltips *tooltips = gtk_tooltips_new();
+#endif                          /* !GTK_CHECK_VERSION(2, 11, 0) */
 
     ebox = gtk_event_box_new();
 
@@ -278,7 +277,6 @@ bm_header_tl_buttons(BalsaMessage * bm)
     gtk_container_add(GTK_CONTAINER(ebox), hbox2);
 
     vbox = gtk_vbox_new(FALSE, 0);
-    gtk_widget_hide(vbox);
     gtk_box_pack_start(GTK_BOX(hbox2), vbox, FALSE, FALSE, 0);
     g_object_set_data(G_OBJECT(bm), BALSA_MESSAGE_FACE_BOX, vbox);
 
@@ -287,8 +285,13 @@ bm_header_tl_buttons(BalsaMessage * bm)
     gtk_box_pack_start(GTK_BOX(hbox2), vbox, FALSE, FALSE, 0);
 
     button = gtk_button_new();
+#if GTK_CHECK_VERSION(2, 11, 0)
+    gtk_widget_set_tooltip_text(button,
+			        _("Check cryptographic signature"));
+#else                           /* GTK_CHECK_VERSION(2, 11, 0) */
     gtk_tooltips_set_tip(tooltips, button,
 			 _("Check cryptographic signature"), NULL);
+#endif                          /* GTK_CHECK_VERSION(2, 11, 0) */
     g_signal_connect(G_OBJECT(button), "focus_in_event",
 		     G_CALLBACK(balsa_mime_widget_limit_focus),
 		     (gpointer) bm);
@@ -308,8 +311,13 @@ bm_header_tl_buttons(BalsaMessage * bm)
     gtk_box_pack_start(GTK_BOX(hbox2), vbox, FALSE, FALSE, 0);
 
     button = gtk_button_new();
+#if GTK_CHECK_VERSION(2, 11, 0)
+    gtk_widget_set_tooltip_text(button,
+			        _("Select message part to display"));
+#else                           /* GTK_CHECK_VERSION(2, 11, 0) */
     gtk_tooltips_set_tip(tooltips, button,
 			 _("Select message part to display"), NULL);
+#endif                          /* GTK_CHECK_VERSION(2, 11, 0) */
     g_signal_connect(G_OBJECT(button), "focus_in_event",
 		     G_CALLBACK(balsa_mime_widget_limit_focus),
 		     (gpointer) bm);
@@ -487,7 +495,7 @@ balsa_message_destroy(GtkObject * object)
     BalsaMessage* bm = BALSA_MESSAGE(object);
 
     if (bm->treeview) {
-        balsa_message_set(bm, NULL);
+        balsa_message_set(bm, NULL, 0);
         gtk_widget_destroy(bm->treeview);
         bm->treeview = NULL;
     }
@@ -638,8 +646,12 @@ tree_mult_selection_popup(BalsaMessage * bm, GdkEventButton * event,
         GtkWidget *menu_item;
         
         bm->save_all_popup = gtk_menu_new ();
-	g_object_ref(bm->save_all_popup);
-	gtk_object_sink(GTK_OBJECT(bm->save_all_popup));
+#if GLIB_CHECK_VERSION(2, 10, 0)
+        g_object_ref_sink(bm->save_all_popup);
+#else                           /* GLIB_CHECK_VERSION(2, 10, 0) */
+        g_object_ref(bm->save_all_popup);
+        gtk_object_sink(GTK_OBJECT(bm->save_all_popup));
+#endif                          /* GLIB_CHECK_VERSION(2, 10, 0) */
         menu_item = 
             gtk_menu_item_new_with_label (_("Save selected as..."));
         gtk_widget_show(menu_item);
@@ -725,24 +737,11 @@ tree_button_press_cb(GtkWidget * widget, GdkEventButton * event,
     return TRUE;
 }
 
-
-static void
-bm_message_weak_ref_cb(BalsaMessage * bm, LibBalsaMessage * message)
-{
-    if (bm->message == message) {
-	bm->message = NULL;
-	if (bm->close_with_msg)
-	    gtk_widget_destroy(GTK_WIDGET(bm));
-	else
-	    balsa_message_set(bm, NULL);
-    }
-}
-
 /* balsa_message_set:
    returns TRUE on success, FALSE on failure (message content could not be
    accessed).
 
-   if message == NULL, clears the display and returns TRUE
+   if msgno == 0, clears the display and returns TRUE
 */
 
 /* Helpers:
@@ -775,42 +774,32 @@ balsa_message_clear_tree(BalsaMessage * bm)
 }
 
 gboolean
-balsa_message_set(BalsaMessage * bm, LibBalsaMessage * message)
+balsa_message_set(BalsaMessage * bm, LibBalsaMailbox * mailbox, guint msgno)
 {
     gboolean is_new;
     GtkTreeIter iter;
     BalsaPartInfo *info;
     gboolean has_focus = bm->focus_state != BALSA_MESSAGE_FOCUS_STATE_NO;
+    LibBalsaMessage *message;
 
     g_return_val_if_fail(bm != NULL, FALSE);
 
-    /* Leave this out. When settings (eg wrap) are changed it is OK to 
-       call message_set with the same messagr */
-    /*    if (bm->message == message) */
-    /*      return; */
-
     gtk_widget_hide(GTK_WIDGET(bm));
     select_part(bm, NULL);
     if (bm->message != NULL) {
-        g_object_weak_unref(G_OBJECT(bm->message),
-                            (GWeakNotify) bm_message_weak_ref_cb,
-                            (gpointer) bm);
         libbalsa_message_body_unref(bm->message);
+        g_object_unref(bm->message);
         bm->message = NULL;
     }
     balsa_message_clear_tree(bm);
 
-    if (message == NULL) {
+    if (mailbox == NULL || msgno == 0) {
         gtk_notebook_set_show_tabs(GTK_NOTEBOOK(bm), FALSE);
         gtk_notebook_set_current_page(GTK_NOTEBOOK(bm), 0);
         return TRUE;
     }
 
-    bm->message = message;
-
-    g_object_weak_ref(G_OBJECT(message),
-                      (GWeakNotify) bm_message_weak_ref_cb,
-                      (gpointer) bm);
+    bm->message = message = libbalsa_mailbox_get_message(mailbox, msgno);
 
     is_new = LIBBALSA_MESSAGE_IS_UNREAD(message);
     if(!libbalsa_message_body_ref(message, TRUE, TRUE)) {
@@ -819,13 +808,15 @@ balsa_message_set(BalsaMessage * bm, LibBalsaMessage * message)
 	balsa_information(LIBBALSA_INFORMATION_WARNING,
                           _("Could not access message %ld "
                             "in mailbox \"%s\"."),
-			  message->msgno, mailbox->name);
+			  msgno, mailbox->name);
+        g_object_unref(bm->message);
+        bm->message = NULL;
         return FALSE;
     }
 
 #ifdef HAVE_GPGME
-    balsa_message_perform_crypto(bm->message, 
-				 libbalsa_mailbox_get_crypto_mode(bm->message->mailbox),
+    balsa_message_perform_crypto(message, 
+				 libbalsa_mailbox_get_crypto_mode(mailbox),
 				 FALSE, 1);
     /* calculate the signature summary state if not set earlier */
     if(message->prot_state == LIBBALSA_MSG_PROTECT_NONE) {
@@ -837,8 +828,8 @@ balsa_message_set(BalsaMessage * bm, LibBalsaMessage * message)
     }
 #endif
 
-    libbalsa_mailbox_msgno_update_attach(bm->message->mailbox,
-    /* may update the icon */            bm->message->msgno, bm->message);
+    /* may update the icon */           
+    libbalsa_mailbox_msgno_update_attach(mailbox, msgno, message);
 
     display_headers(bm);
     display_content(bm);
@@ -876,13 +867,9 @@ balsa_message_set(BalsaMessage * bm, LibBalsaMessage * message)
     /*
      * emit read message
      */
-    if (is_new && message->mailbox && !message->mailbox->readonly) {
-	GList * messages = g_list_prepend(NULL, message);
-	
-	libbalsa_messages_change_flag(messages, LIBBALSA_MESSAGE_FLAG_NEW,
-                                      FALSE);
-	g_list_free(messages);
-    }
+    if (is_new && !mailbox->readonly)
+        libbalsa_mailbox_msgno_change_flags(mailbox, msgno, 0,
+                                            LIBBALSA_MESSAGE_FLAG_NEW);
 
     /* restore keyboard focus to the content, if it was there before */
     if (has_focus)
@@ -897,7 +884,7 @@ balsa_message_save_current_part(BalsaMessage * bm)
     g_return_if_fail(bm != NULL);
 
     if (bm->current_part)
-	balsa_mime_widget_ctx_menu_save(NULL, bm->current_part->body);
+	balsa_mime_widget_ctx_menu_save(GTK_WIDGET(bm), bm->current_part->body);
 }
 
 static gboolean
@@ -910,10 +897,10 @@ balsa_message_set_embedded_hdr(GtkTreeModel * model, GtkTreePath * path,
     gtk_tree_model_get(model, iter, PART_INFO_COLUMN, &info, -1);
     if (info) {
  	if (info->body && info->body->embhdrs && info->mime_widget)
- 	    balsa_mime_widget_message_set_headers(bm, info->mime_widget,
- 						  info->body->embhdrs,
- 						  info->body->parts,
- 						  info->body->embhdrs->subject);
+ 	    balsa_mime_widget_message_set_headers_d(bm, info->mime_widget,
+                                                    info->body->embhdrs,
+                                                    info->body->parts,
+                                                    info->body->embhdrs->subject);
 	g_object_unref(G_OBJECT(info));
     }
     
@@ -958,7 +945,7 @@ balsa_message_set_wrap(BalsaMessage * bm, gboolean wrap)
     /* This is easier than reformating all the widgets... */
     if (bm->message) {
         LibBalsaMessage *msg = bm->message;
-        balsa_message_set(bm, msg);
+        balsa_message_set(bm, msg->mailbox, msg->msgno);
     }
 }
 
@@ -966,9 +953,10 @@ balsa_message_set_wrap(BalsaMessage * bm, gboolean wrap)
 static void
 display_headers(BalsaMessage * bm)
 {
-    balsa_mime_widget_message_set_headers(bm, bm->bm_widget, 
- 					  bm->message->headers, bm->message->body_list,
- 					  LIBBALSA_MESSAGE_GET_SUBJECT(bm->message));
+    balsa_mime_widget_message_set_headers_d(bm, bm->bm_widget, 
+                                            bm->message->headers,
+                                            bm->message->body_list,
+                                            LIBBALSA_MESSAGE_GET_SUBJECT(bm->message));
 }
 
 
@@ -1030,6 +1018,34 @@ add_to_attachments_popup(GtkMenuShell * menu, const gchar * item,
     gtk_menu_shell_append(menu, menuitem);
 }
 
+static void
+toggle_all_inline_cb(GtkCheckMenuItem * item, BalsaPartInfo *info)
+{
+    BalsaMessage * bm = g_object_get_data(G_OBJECT(item), "balsa-message");
+
+    g_return_if_fail(bm);
+    g_return_if_fail(info);
+    
+    bm->force_inline = gtk_check_menu_item_get_active(item);
+    
+    gtk_notebook_set_current_page(GTK_NOTEBOOK(bm), 0);
+    select_part(bm, info);
+}
+
+static void
+add_toggle_inline_menu_item(GtkMenuShell * menu, BalsaMessage * bm,
+			    BalsaPartInfo *info)
+{
+    GtkWidget * menuitem =
+	gtk_check_menu_item_new_with_label (_("force inline for all parts"));
+    
+    g_object_set_data(G_OBJECT(menuitem), "balsa-message", bm);
+    g_signal_connect(G_OBJECT (menuitem), "activate",
+		     GTK_SIGNAL_FUNC (toggle_all_inline_cb),
+		     (gpointer) info);
+    gtk_menu_shell_append(menu, menuitem);
+}
+
 static void
 display_part(BalsaMessage * bm, LibBalsaMessageBody * body,
              GtkTreeModel * model, GtkTreeIter * iter, gchar * part_id)
@@ -1053,13 +1069,21 @@ display_part(BalsaMessage * bm, LibBalsaMessageBody * body,
         if (g_ascii_strcasecmp(content_type, "message/rfc822") == 0 &&
             body->embhdrs) {
             gchar *from = balsa_message_sender_to_gchar(body->embhdrs->from, 0);
+            gchar *subj = g_strdup(body->embhdrs->subject);
+            libbalsa_utf8_sanitize(&from, balsa_app.convert_unknown_8bit, NULL);
+            libbalsa_utf8_sanitize(&subj, balsa_app.convert_unknown_8bit, NULL);
             icon_title = 
                 g_strdup_printf(_("rfc822 message (from %s, subject \"%s\")"),
-                                from, body->embhdrs->subject);
+                                from, subj);
             g_free(from);
+            g_free(subj);
         } else if (is_multipart) {
             icon_title = mpart_content_name(content_type);
 	    if (!strcmp(part_id, "1")) {
+		add_toggle_inline_menu_item(GTK_MENU_SHELL(bm->parts_popup),
+					    bm, info);
+		gtk_menu_shell_append(GTK_MENU_SHELL(bm->parts_popup), 
+				      gtk_separator_menu_item_new ());
 		add_to_attachments_popup(GTK_MENU_SHELL(bm->parts_popup), 
 					 _("complete message"),
 					 bm, info);
@@ -1174,12 +1198,14 @@ display_face(BalsaMessage * bm)
     GtkWidget *image;
 
     face_box = g_object_get_data(G_OBJECT(bm), BALSA_MESSAGE_FACE_BOX);
+    gtk_container_foreach(GTK_CONTAINER(face_box),
+                          (GtkCallback) gtk_widget_destroy, NULL);
+
     if (!bm->message
         || !((face = libbalsa_message_get_user_header(bm->message, "Face"))
              || (x_face =
                  libbalsa_message_get_user_header(bm->message,
                                                   "X-Face")))) {
-        gtk_widget_hide(face_box);
         return;
     }
 
@@ -1189,7 +1215,6 @@ display_face(BalsaMessage * bm)
 #if HAVE_COMPFACE
         image = libbalsa_get_image_from_x_face_header(x_face, &err);
 #else                           /* HAVE_COMPFACE */
-        gtk_widget_hide(face_box);
         return;
 #endif                          /* HAVE_COMPFACE */
     }
@@ -1201,10 +1226,8 @@ display_face(BalsaMessage * bm)
         return;
     }
 
-    gtk_container_foreach(GTK_CONTAINER(face_box),
-                          (GtkCallback) gtk_widget_destroy, NULL);
     gtk_box_pack_start(GTK_BOX(face_box), image, FALSE, FALSE, 0);
-    gtk_widget_show_all(face_box);
+    gtk_widget_show(image);
 }
 
 static void
@@ -1214,8 +1237,12 @@ display_content(BalsaMessage * bm)
     if (bm->parts_popup)
 	g_object_unref(bm->parts_popup);
     bm->parts_popup = gtk_menu_new();
+#if GLIB_CHECK_VERSION(2, 10, 0)
+    g_object_ref_sink(bm->parts_popup);
+#else                           /* GLIB_CHECK_VERSION(2, 10, 0) */
     g_object_ref(bm->parts_popup);
     gtk_object_sink(GTK_OBJECT(bm->parts_popup));
+#endif                          /* GLIB_CHECK_VERSION(2, 10, 0) */
     display_parts(bm, bm->message->body_list, NULL, NULL);
     if (bm->info_count > 1) {
  	gtk_widget_show_all(bm->parts_popup);
@@ -1242,8 +1269,12 @@ part_create_menu (BalsaPartInfo* info)
     gchar* content_type;
     
     info->popup_menu = gtk_menu_new ();
+#if GLIB_CHECK_VERSION(2, 10, 0)
+    g_object_ref_sink(info->popup_menu);
+#else                           /* GLIB_CHECK_VERSION(2, 10, 0) */
     g_object_ref(info->popup_menu);
     gtk_object_sink(GTK_OBJECT(info->popup_menu));
+#endif                          /* GLIB_CHECK_VERSION(2, 10, 0) */
     
     content_type = libbalsa_message_body_get_mime_type (info->body);
     libbalsa_fill_vfs_menu_by_content_type(GTK_MENU(info->popup_menu),
@@ -1307,7 +1338,8 @@ static void
 part_context_save_all_cb(GtkWidget * menu_item, GList * info_list)
 {
     while (info_list) {
-	balsa_mime_widget_ctx_menu_save(NULL, BALSA_PART_INFO(info_list->data)->body);
+	balsa_mime_widget_ctx_menu_save(menu_item,
+                                        BALSA_PART_INFO(info_list->data)->body);
         info_list = g_list_next(info_list);
     }
 }
@@ -1328,7 +1360,7 @@ part_context_dump_all_cb(GtkWidget * menu_item, GList * info_list)
 
     dump_dialog =
         gtk_file_chooser_dialog_new(_("Select folder for saving selected parts"),
-                                    GTK_WINDOW(balsa_app.main_window),
+                                    balsa_get_parent_window(menu_item),
                                     GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                     GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
@@ -1477,9 +1509,10 @@ balsa_message_next_part(BalsaMessage * bmessage)
 gboolean
 balsa_message_has_next_part(BalsaMessage * bmessage)
 {
-    BalsaPartInfo *info = bm_next_part_info(bmessage);
+    BalsaPartInfo *info;
 
-    if (info) {
+    if (bmessage && bmessage->treeview
+        && (info = bm_next_part_info(bmessage))) {
         g_object_unref(info);
         return TRUE;
     }
@@ -1549,9 +1582,10 @@ balsa_message_previous_part(BalsaMessage * bmessage)
 gboolean
 balsa_message_has_previous_part(BalsaMessage * bmessage)
 {
-    BalsaPartInfo *info = bm_previous_part_info(bmessage);
+    BalsaPartInfo *info;
 
-    if (info) {
+    if (bmessage && bmessage->treeview
+        && (info = bm_previous_part_info(bmessage))) {
         g_object_unref(info);
         return TRUE;
     }
@@ -1702,6 +1736,7 @@ add_multipart_mixed(BalsaMessage * bm, LibBalsaMessageBody * body)
 		g_mime_content_type_new_from_string(body->content_type);
 
             if (libbalsa_message_body_is_inline(body) ||
+		bm->force_inline ||
                 libbalsa_message_body_is_multipart(body) ||
 		g_mime_content_type_is_type(type, "application", "pgp-signature") ||
 		(balsa_app.has_smime && 
@@ -1710,8 +1745,9 @@ add_multipart_mixed(BalsaMessage * bm, LibBalsaMessageBody * body)
                 add_body(bm, body);
 	    g_mime_content_type_destroy(type);
 #else
-            if (libbalsa_message_body_is_inline(body)
-                || libbalsa_message_body_is_multipart(body))
+            if (libbalsa_message_body_is_inline(body) ||
+		bm->force_inline ||
+		libbalsa_message_body_is_multipart(body)) 
                 add_body(bm, body);
 #endif
         }
@@ -1884,6 +1920,25 @@ balsa_message_current_part_widget(BalsaMessage * bmessage)
 	return NULL;
 }
 
+GtkWindow*
+balsa_get_parent_window(GtkWidget *widget)
+{
+    GtkWidget *a_widget, *parent_widget;
+
+    if(!widget)
+        return GTK_WINDOW(balsa_app.main_window);
+
+    for(a_widget = widget, parent_widget=NULL;
+        a_widget;
+        a_widget = gtk_widget_get_parent(a_widget)) 
+        parent_widget = a_widget;
+
+    g_return_val_if_fail(GTK_IS_WINDOW(parent_widget),
+                         GTK_WINDOW(balsa_app.main_window));
+
+    return GTK_WINDOW(parent_widget);
+}
+
 
 /*
  * This function informs the caller if the currently selected part 
@@ -1933,7 +1988,7 @@ bm_get_mailbox(const InternetAddressList * list)
     if (!ia || ia->type == INTERNET_ADDRESS_NONE)
 	return NULL;
 
-    while (ia->type == INTERNET_ADDRESS_GROUP)
+    while (ia->type == INTERNET_ADDRESS_GROUP && ia->value.members)
 	ia = ia->value.members->address;
     if (!ia || ia->type == INTERNET_ADDRESS_NONE)
 	return NULL;
@@ -2012,16 +2067,24 @@ handle_mdn_request(LibBalsaMessage *message)
         g_free (reply_to);
         g_free (sender);
     } else {
+	GError * error = NULL;
+	LibBalsaMsgCreateResult result;
+
 #if ENABLE_ESMTP
-        libbalsa_message_send(mdn, balsa_app.outbox, NULL,
-                              balsa_find_sentbox_by_url,
-			      balsa_app.current_ident->smtp_server,
-                              TRUE, balsa_app.debug);
+        result = libbalsa_message_send(mdn, balsa_app.outbox, NULL,
+				       balsa_find_sentbox_by_url,
+				       balsa_app.current_ident->smtp_server,
+				       TRUE, balsa_app.debug, &error);
 #else
-        libbalsa_message_send(mdn, balsa_app.outbox, NULL,
-                              balsa_find_sentbox_by_url,
-                              TRUE, balsa_app.debug);
+        result = libbalsa_message_send(mdn, balsa_app.outbox, NULL,
+				       balsa_find_sentbox_by_url,
+				       TRUE, balsa_app.debug, &error);
 #endif
+	if (result != LIBBALSA_MESSAGE_CREATE_OK)
+	    libbalsa_information(LIBBALSA_INFORMATION_ERROR,
+				 _("Sending the disposition notification failed: %s"),
+				 error ? error->message : "?");
+	g_error_free(error);
         g_object_unref(G_OBJECT(mdn));
     }
 }
@@ -2043,8 +2106,7 @@ static LibBalsaMessage *create_mdn_reply (LibBalsaMessage *for_msg,
     message->headers->from = internet_address_parse_string(dummy);
     g_free (dummy);
     message->headers->date = time(NULL);
-    LIBBALSA_MESSAGE_SET_SUBJECT(message,
-                                 g_strdup("Message Disposition Notification"));
+    libbalsa_message_set_subject(message, "Message Disposition Notification");
     message->headers->to_list =
 	internet_address_list_prepend(NULL, for_msg->headers->
 		                      dispnotify_to->address);
@@ -2088,8 +2150,9 @@ static LibBalsaMessage *create_mdn_reply (LibBalsaMessage *for_msg,
 			       original_rcpt);	
     g_string_append_printf(report, "Final-Recipient: rfc822; %s\n",
 			   balsa_app.current_ident->ia->value.addr);
-    g_string_append_printf(report, "Original-Message-ID: %s\n",
-			   for_msg->message_id);
+    if (for_msg->message_id)
+        g_string_append_printf(report, "Original-Message-ID: <%s>\n",
+                               for_msg->message_id);
     g_string_append_printf(report,
 			   "Disposition: %s-action/MDN-sent-%sly; displayed",
 			   manual ? "manual" : "automatic",
@@ -2134,20 +2197,27 @@ mdn_dialog_response(GtkWidget * dialog, gint response, gpointer user_data)
     LibBalsaMessage *send_msg =
         LIBBALSA_MESSAGE(g_object_get_data(G_OBJECT(dialog),
                                            "balsa-send-msg"));
+    GError * error = NULL;
+    LibBalsaMsgCreateResult result;
 
     if (response == GTK_RESPONSE_YES) {
 #if ENABLE_ESMTP
-        libbalsa_message_send(send_msg, balsa_app.outbox, NULL,
-                              balsa_find_sentbox_by_url,
-			      balsa_app.current_ident->smtp_server,
-                              TRUE, balsa_app.debug);
+        result = libbalsa_message_send(send_msg, balsa_app.outbox, NULL,
+				       balsa_find_sentbox_by_url,
+				       balsa_app.current_ident->smtp_server,
+				       TRUE, balsa_app.debug, &error);
 #else
-        libbalsa_message_send(send_msg, balsa_app.outbox, NULL,
-                              balsa_find_sentbox_by_url,
-                              TRUE, balsa_app.debug);
+        result = libbalsa_message_send(send_msg, balsa_app.outbox, NULL,
+				       balsa_find_sentbox_by_url,
+				       TRUE, balsa_app.debug, &error);
 #endif
+        if (result != LIBBALSA_MESSAGE_CREATE_OK)
+            libbalsa_information(LIBBALSA_INFORMATION_ERROR,
+                                 _("Sending the disposition notification failed: %s"),
+                                 error ? error->message : "?");
+        if (error)
+            g_error_free(error);
     }
-
     g_object_unref(G_OBJECT(send_msg));
     gtk_widget_destroy(dialog);
 }
@@ -2339,6 +2409,8 @@ libbalsa_msg_try_decrypt(LibBalsaMessage * message, LibBalsaMessageBody * body,
                      _("Parsing a message part failed: %s"),
                      err ? err->message : _("Possible disk space problem."));
                 g_clear_error(&err);
+                /* There is nothing more we can do... */
+                return body;
             }
 	}
 
@@ -2398,6 +2470,7 @@ libbalsa_msg_try_decrypt(LibBalsaMessage * message, LibBalsaMessageBody * body,
 /*
  * Treatment of a multipart/signed body with protocol protocol.
  */
+#define BALSA_MESSAGE_SIGNED_NOTIFIED "balsa-message-signed-notified"
 static void
 libbalsa_msg_try_mp_signed(LibBalsaMessage * message, LibBalsaMessageBody *body,
 			   chk_crypto_t * chk_crypto)
@@ -2421,6 +2494,8 @@ libbalsa_msg_try_mp_signed(LibBalsaMessage * message, LibBalsaMessageBody *body,
                  _("Parsing a message part failed: %s"),
                  err ? err->message : _("Possible disk space problem."));
             g_clear_error(&err);
+            /* There is nothing more we can do... */
+            return;
         }
     }
 	
@@ -2465,6 +2540,11 @@ libbalsa_msg_try_mp_signed(LibBalsaMessage * message, LibBalsaMessageBody *body,
 	return;
                 
     /* evaluate the result */
+    if (g_object_get_data(G_OBJECT(message), BALSA_MESSAGE_SIGNED_NOTIFIED))
+        return;
+    g_object_set_data(G_OBJECT(message), BALSA_MESSAGE_SIGNED_NOTIFIED,
+                      GUINT_TO_POINTER(TRUE));
+
     if (body->parts->next->sig_info) {
 	switch (libbalsa_message_body_protect_state(body->parts->next)) {
 	case LIBBALSA_MSG_PROTECT_SIGN_GOOD:
@@ -2536,6 +2616,7 @@ libbalsa_msg_part_2440(LibBalsaMessage * message, LibBalsaMessageBody * body,
                  _("Parsing a message part failed: %s"),
                  err ? err->message : _("Possible disk space problem."));
             g_clear_error(&err);
+            return;
         }
     }
 
@@ -2730,6 +2811,7 @@ message_recheck_crypto_cb(GtkWidget * button, BalsaMessage * bm)
         return;
     }
 
+    g_object_set_data(G_OBJECT(message), BALSA_MESSAGE_SIGNED_NOTIFIED, NULL);
     balsa_message_perform_crypto(message, LB_MAILBOX_CHK_CRYPT_ALWAYS, FALSE, 2);
 
     /* calculate the signature summary state */
