@@ -26,9 +26,11 @@
 #include "i18n.h"
 #include "quote-color.h"
 #include "sendmsg-window.h"
+#include "store-address.h"
 #include "balsa-mime-widget.h"
 #include "balsa-mime-widget-callbacks.h"
 #include "balsa-mime-widget-text.h"
+#include "balsa-cite-bar.h"
 
 #if HAVE_GTKSOURCEVIEW
 #include <gtksourceview/gtksourceview.h>
@@ -40,14 +42,18 @@
 
 static GtkWidget * create_text_widget(const char * content_type);
 static void bm_modify_font_from_string(GtkWidget * widget, const char *font);
-static GtkTextTag * quote_tag(GtkTextBuffer * buffer, gint level);
+static GtkTextTag * quote_tag(GtkTextBuffer * buffer, gint level, gint margin);
 static gboolean fix_text_widget(GtkWidget *widget, gpointer data);
 static void text_view_populate_popup(GtkTextView *textview, GtkMenu *menu,
 				     LibBalsaMessageBody * mime_body);
 
 #ifdef HAVE_GTKHTML
-static BalsaMimeWidget * bm_widget_new_html(BalsaMessage * bm, LibBalsaMessageBody * mime_body, gchar * ptr, size_t len);
+static BalsaMimeWidget * bm_widget_new_html(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
+					    gchar * ptr, size_t len);
 #endif
+static BalsaMimeWidget * bm_widget_new_vcard(BalsaMessage * bm,
+                                             LibBalsaMessageBody * mime_body,
+                                             gchar * ptr, size_t len);
 
 /* URL related stuff */
 typedef struct _message_url_t {
@@ -56,6 +62,19 @@ typedef struct _message_url_t {
     gchar *url;                  /* the link */
 } message_url_t;
 
+
+/* citation bars */
+typedef struct {
+    gint start_offs;
+    gint end_offs;
+    GtkTextIter start_iter;
+    GtkTextIter end_iter;
+    gint y_pos;
+    gint height;
+    guint depth;
+    GtkWidget * bar;
+} cite_bar_t;
+
 /* store the coordinates at which the button was pressed */
 static gint stored_x = -1, stored_y = -1;
 static GdkModifierType stored_mask = -1;
@@ -85,11 +104,15 @@ static void balsa_gtk_html_on_url(GtkWidget *html, const gchar *url);
 static void phrase_highlight(GtkTextBuffer * buffer, const gchar * id,
 			     gunichar tag_char, const gchar * property,
 			     gint value);
+static void destroy_cite_bars(GList * cite_bars);
+static gboolean draw_cite_bars(GtkWidget * widget, GdkEventExpose *event, GList * cite_bars);
 
 
 #define PHRASE_HIGHLIGHT_ON    1
 #define PHRASE_HIGHLIGHT_OFF   2
 
+#define BALSA_MIME_WIDGET_NEW_TEXT_NOTIFIED \
+    "balsa-mime-widget-text-new-notified"
 
 BalsaMimeWidget *
 balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
@@ -104,11 +127,13 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
     GList *url_list = NULL;
     const gchar *target_cs;
     GError *err = NULL;
+    gboolean is_text_plain;
 
 
     g_return_val_if_fail(mime_body != NULL, NULL);
     g_return_val_if_fail(content_type != NULL, NULL);
 
+    is_text_plain = !g_ascii_strcasecmp(content_type, "text/plain");
     alloced = libbalsa_message_body_get_content(mime_body, &ptr, &err);
     if (!ptr) {
         balsa_information(LIBBALSA_INFORMATION_ERROR,
@@ -134,15 +159,22 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
         return NULL;
 #endif
     }
+    if(g_ascii_strcasecmp(content_type, "text/x-vcard") == 0) {
+        return bm_widget_new_vcard(bm, mime_body, ptr, alloced);
+    }
 
     /* prepare a text part */
     if (!libbalsa_utf8_sanitize(&ptr, balsa_app.convert_unknown_8bit,
-				&target_cs)) {
+				&target_cs)
+        && !g_object_get_data(G_OBJECT(bm->message), 
+                              BALSA_MIME_WIDGET_NEW_TEXT_NOTIFIED)) {
 	gchar *from =
 	    balsa_message_sender_to_gchar(bm->message->headers->from, 0);
 	gchar *subject =
 	    g_strdup(LIBBALSA_MESSAGE_GET_SUBJECT(bm->message));
         
+	libbalsa_utf8_sanitize(&from,    balsa_app.convert_unknown_8bit, 
+			       NULL);
 	libbalsa_utf8_sanitize(&subject, balsa_app.convert_unknown_8bit, 
 			       NULL);
 	libbalsa_information
@@ -154,13 +186,24 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 	     target_cs ? target_cs : "\"?\"");
 	g_free(subject);
 	g_free(from);
+        /* Avoid multiple notifications: */
+        g_object_set_data(G_OBJECT(bm->message),
+                          BALSA_MIME_WIDGET_NEW_TEXT_NOTIFIED, 
+                          GUINT_TO_POINTER(TRUE));
     }
 
+    mw = g_object_new(BALSA_TYPE_MIME_WIDGET, NULL);
+    mw->widget = create_text_widget(content_type);
+
     if (libbalsa_message_body_is_flowed(mime_body)) {
 	/* Parse, but don't wrap. */
 	gboolean delsp = libbalsa_message_body_is_delsp(mime_body);
 	ptr = libbalsa_wrap_rfc2646(ptr, G_MAXINT, FALSE, TRUE, delsp);
-    } else if (bm->wrap_text)
+    } else if (bm->wrap_text
+#if HAVE_GTKSOURCEVIEW
+	       && !GTK_IS_SOURCE_VIEW(mw->widget)
+#endif
+	       )
 	libbalsa_wrap_string(ptr, balsa_app.browse_wrap_length);
 
     mw = g_object_new(BALSA_TYPE_MIME_WIDGET, NULL);
@@ -191,6 +234,15 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
     } else {
 	gchar *line_start;
 	LibBalsaUrlInsertInfo url_info;
+	GList * cite_bars_list;
+	guint cite_level;
+	guint cite_start;
+	gint margin;
+	PangoFontDescription *font;
+
+	font = pango_font_description_from_string(balsa_app.message_font);
+	margin = pango_font_description_get_size(font) / PANGO_SCALE;
+	pango_font_description_free(font);
 
 	gtk_text_buffer_create_tag(buffer, "url",
 				   "foreground-gdk",
@@ -201,14 +253,17 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 				   NULL);
 	url_info.callback = url_found_cb;
 	url_info.callback_data = &url_list;
+	url_info.buffer_is_flowed = libbalsa_message_body_is_flowed(mime_body);
 	url_info.ml_url = NULL;
 	url_info.ml_url_buffer = NULL;
 
 	line_start = ptr;
+	cite_bars_list = NULL;
+	cite_level = 0;
+	cite_start = 0;
 	while (line_start) {
 	    gchar *newline, *this_line;
-	    gint quote_level;
-	    GtkTextTag *tag;
+	    GtkTextTag *tag = NULL;
 	    int len;
 
 	    newline = strchr(line_start, '\n');
@@ -216,8 +271,43 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 		this_line = g_strndup(line_start, newline - line_start);
 	    else
 		this_line = g_strdup(line_start);
-	    quote_level = is_a_quote(this_line, &rex);
-	    tag = quote_tag(buffer, quote_level);
+
+	    if (is_text_plain) {
+		guint quote_level;
+		guint cite_idx;
+
+		/* get the cite level only for text/plain parts */
+		libbalsa_match_regex(this_line, &rex, &quote_level, &cite_idx);
+	    
+		/* check if the citation level changed */
+		if (cite_level != quote_level) {
+		    if (cite_level > 0) {
+			cite_bar_t * cite_bar = g_new0(cite_bar_t, 1);
+		    
+			cite_bar->start_offs = cite_start;
+			cite_bar->end_offs = gtk_text_buffer_get_char_count(buffer);
+			cite_bar->depth = cite_level;
+			cite_bars_list = g_list_append(cite_bars_list, cite_bar);
+		    }
+		    if (quote_level > 0)
+			cite_start = gtk_text_buffer_get_char_count(buffer);
+		    cite_level = quote_level;
+		}
+
+		/* strip off the citation prefix */
+		if (quote_level) {
+		    gchar *new;
+		
+		    new = this_line + cite_idx;
+		    if (g_unichar_isspace(g_utf8_get_char(new)))
+			new = g_utf8_next_char(new);
+		    new = g_strdup(new);
+		    g_free(this_line);
+		    this_line = new;
+		}
+		tag = quote_tag(buffer, quote_level, margin);
+	    }
+
 	    len = strlen(this_line);
 	    if (len > 0 && this_line[len - 1] == '\r')
 		this_line[len - 1] = '\0';
@@ -230,12 +320,28 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 	    g_free(this_line);
 	    line_start = newline ? newline + 1 : NULL;
 	}
+
+	/* add any pending cited part */
+	if (cite_level > 0) {
+	    cite_bar_t * cite_bar = g_new0(cite_bar_t, 1);
+		    
+	    cite_bar->start_offs = cite_start;
+	    cite_bar->end_offs = gtk_text_buffer_get_char_count(buffer);
+	    cite_bar->depth = cite_level;
+	    cite_bars_list = g_list_append(cite_bars_list, cite_bar);
+	}
+
+	/* add list of citation bars (if any) */
+	if (cite_bars_list) {
+	    g_object_set_data_full(G_OBJECT(mw->widget), "cite-bars", cite_bars_list,
+				   (GDestroyNotify) destroy_cite_bars);
+	    g_object_set_data(G_OBJECT(mw->widget), "cite-margin", GINT_TO_POINTER(margin));
+	    g_signal_connect_after(G_OBJECT(mw->widget), "expose-event",
+				   G_CALLBACK(draw_cite_bars), cite_bars_list);
+	}
 	regfree(&rex);
     }
 
-    if (libbalsa_message_body_is_flowed(mime_body))
-	libbalsa_wrap_view(GTK_TEXT_VIEW(mw->widget),
-			   balsa_app.browse_wrap_length);
     prepare_url_offsets(buffer, url_list);
     g_signal_connect_after(G_OBJECT(mw->widget), "realize",
 			   G_CALLBACK(fix_text_widget), url_list);
@@ -252,7 +358,7 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 			       (GDestroyNotify)free_url_list);
     }
 
-    if (!g_ascii_strcasecmp(content_type, "text/plain")) {
+    if (is_text_plain) {
 	/* plain-text highlighting */
 	g_object_set_data(G_OBJECT(mw->widget), "phrase-highlight",
 			  GINT_TO_POINTER(PHRASE_HIGHLIGHT_ON));
@@ -320,27 +426,29 @@ bm_modify_font_from_string(GtkWidget * widget, const char *font)
  * returns NULL if the level is 0 (unquoted)
  */
 static GtkTextTag *
-quote_tag(GtkTextBuffer * buffer, gint level)
+quote_tag(GtkTextBuffer * buffer, gint level, gint margin)
 {
     GtkTextTag *tag = NULL;
 
     if (level > 0) {
         GtkTextTagTable *table = gtk_text_buffer_get_tag_table(buffer);
         gchar *name;
+	gint q_level;
 
         /* Modulus the quote level by the max,
          * ie, always have "1 <= quote level <= MAX"
          * this allows cycling through the possible
          * quote colors over again as the quote level
          * grows arbitrarily deep. */
-        level = (level - 1) % MAX_QUOTED_COLOR;
+        q_level = (level - 1) % MAX_QUOTED_COLOR;
         name = g_strdup_printf("quote-%d", level);
         tag = gtk_text_tag_table_lookup(table, name);
 
         if (!tag) {
             tag =
                 gtk_text_buffer_create_tag(buffer, name, "foreground-gdk",
-                                           &balsa_app.quoted_color[level],
+                                           &balsa_app.quoted_color[q_level],
+					   "left-margin", margin * level,
                                            NULL);
             /* Set a low priority, so we can set both quote color and
              * URL color, and URL color will take precedence. */
@@ -371,6 +479,7 @@ fix_text_widget(GtkWidget *widget, gpointer data)
         url_cursor_over_url = gdk_cursor_new(GDK_HAND2);
     }
     gdk_window_set_cursor(w, url_cursor_normal);
+
     return FALSE;
 }
 
@@ -435,8 +544,7 @@ url_send_cb(GtkWidget * menu_item, message_url_t * uri)
 {
     BalsaSendmsg * newmsg;
 
-    newmsg = sendmsg_window_new(GTK_WIDGET(balsa_app.main_window),
-				NULL, SEND_NORMAL);
+    newmsg = sendmsg_window_compose(GTK_WIDGET(balsa_app.main_window));
     sendmsg_window_set_field(newmsg, "body", uri->url);
 }
 
@@ -728,8 +836,7 @@ handle_url(const message_url_t* url)
 {
     if (!g_ascii_strncasecmp(url->url, "mailto:", 7)) {
         BalsaSendmsg *snd = 
-            sendmsg_window_new(GTK_WIDGET(balsa_app.main_window),
-                               NULL, SEND_NORMAL);
+            sendmsg_window_compose(GTK_WIDGET(balsa_app.main_window));
         sendmsg_window_process_url(url->url + 7,
                                    sendmsg_window_set_field, snd);      
     } else {
@@ -772,7 +879,7 @@ phrase_highlight(GtkTextBuffer * buffer, const gchar * id, gunichar tag_char,
 		 const gchar * property, gint value)
 {
     GtkTextTag *tag = NULL;
-    const gchar * buf_chars;
+    gchar * buf_chars;
     gchar * utf_start;
     GtkTextIter iter_start;
     GtkTextIter iter_end;
@@ -797,16 +904,20 @@ phrase_highlight(GtkTextBuffer * buffer, const gchar * id, gunichar tag_char,
 	    gchar * e_next;
 
 	    /* found a proper start sequence - find the end or eject */
-	    if (!(utf_end = g_utf8_strchr(s_next, -1, tag_char)))
+	    if (!(utf_end = g_utf8_strchr(s_next, -1, tag_char))) {
+                g_free(buf_chars);
 		return;
+            }
 	    line_end = g_utf8_strchr(s_next, -1, '\n');
 	    e_next = g_utf8_next_char(utf_end);
 	    while (!g_unichar_isalnum(UNICHAR_PREV(utf_end)) ||
 		   !(*e_next == '\0' || 
 		     g_unichar_isspace(g_utf8_get_char(e_next)) ||
 		     g_unichar_ispunct(g_utf8_get_char(e_next)))) {
-		if (!(utf_end = g_utf8_strchr(e_next, -1, tag_char)))
+		if (!(utf_end = g_utf8_strchr(e_next, -1, tag_char))) {
+                    g_free(buf_chars);
 		    return;
+                }
 		e_next = g_utf8_next_char(utf_end);
 	    }
 	    
@@ -828,8 +939,88 @@ phrase_highlight(GtkTextBuffer * buffer, const gchar * id, gunichar tag_char,
 	    /* no start sequence, find the next start tag char */
 	    utf_start = *s_next ? g_utf8_strchr(s_next, -1, tag_char) : NULL;
     }
+    g_free(buf_chars);
 }
 
+/* --- citation bar stuff --- */
+static void
+destroy_cite_bars(GList * cite_bars)
+{
+    /* note: the widgets are destroyed by the text view */
+    g_list_foreach(cite_bars, (GFunc) g_free, NULL);
+    g_list_free(cite_bars);
+}
+
+typedef struct {
+    GtkTextView * view;
+    GtkTextBuffer * buffer;
+    gint dimension;
+} cite_bar_draw_mode_t;
+
+
+static void
+draw_cite_bar_real(cite_bar_t * bar, cite_bar_draw_mode_t * draw_mode)
+{
+    GdkRectangle location;
+    gint x_pos;
+    gint y_pos;
+    gint height;
+
+    /* initialise iters if we don't have the widget yet */
+    if (!bar->bar) {
+	gtk_text_buffer_get_iter_at_offset(draw_mode->buffer, &bar->start_iter,
+					   bar->start_offs);
+	gtk_text_buffer_get_iter_at_offset(draw_mode->buffer, &bar->end_iter,
+					   bar->end_offs);
+    }
+
+    /* get the locations */
+    gtk_text_view_get_iter_location(draw_mode->view, &bar->start_iter, &location);
+    gtk_text_view_buffer_to_window_coords(draw_mode->view, GTK_TEXT_WINDOW_TEXT,
+					  location.x, location.y,
+					  &x_pos, &y_pos);
+    gtk_text_view_get_iter_location(draw_mode->view, &bar->end_iter, &location);
+    gtk_text_view_buffer_to_window_coords(draw_mode->view, GTK_TEXT_WINDOW_TEXT,
+					  location.x, location.y,
+					  &x_pos, &height);
+    height -= y_pos;
+
+    /* add a new widget if necessary */
+    if (bar->bar == NULL) {
+	bar->bar = balsa_cite_bar_new(height, bar->depth, draw_mode->dimension);
+	gtk_widget_modify_fg(bar->bar, GTK_STATE_NORMAL,
+			     &balsa_app.quoted_color[(bar->depth - 1) % MAX_QUOTED_COLOR]);
+	gtk_widget_modify_bg(bar->bar, GTK_STATE_NORMAL,
+			     &GTK_WIDGET(draw_mode->view)->style->base[GTK_STATE_NORMAL]);
+	gtk_widget_show(bar->bar);
+	gtk_text_view_add_child_in_window(draw_mode->view, bar->bar,
+					  GTK_TEXT_WINDOW_TEXT, 0, y_pos);
+    } else if (bar->y_pos != y_pos || bar->height != height) {
+	/* shift/resize existing widget */
+	balsa_cite_bar_resize(BALSA_CITE_BAR(bar->bar), height);
+	gtk_text_view_move_child(draw_mode->view, bar->bar, 0, y_pos);
+    }
+
+    /* remember current values */
+    bar->y_pos = y_pos;
+    bar->height = height;
+}
+
+
+static gboolean
+draw_cite_bars(GtkWidget * widget, GdkEventExpose *event, GList * cite_bars)
+{
+    cite_bar_draw_mode_t draw_mode;
+
+    draw_mode.view = GTK_TEXT_VIEW(widget);
+    draw_mode.buffer = gtk_text_view_get_buffer(draw_mode.view);
+    draw_mode.dimension =
+	GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "cite-margin"));
+    g_list_foreach(cite_bars, (GFunc)draw_cite_bar_real, &draw_mode);
+    return FALSE;
+}
+
+
 /* --- HTML related functions -- */
 static void
 balsa_gtk_html_on_url(GtkWidget *html, const gchar *url)
@@ -896,8 +1087,12 @@ balsa_gtk_html_popup(GtkWidget * html, BalsaMessage * bm)
     gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 
     gtk_widget_show_all(menu);
+#if GLIB_CHECK_VERSION(2, 10, 0)
+    g_object_ref_sink(menu);
+#else                           /* GLIB_CHECK_VERSION(2, 10, 0) */
     g_object_ref(menu);
     gtk_object_sink(GTK_OBJECT(menu));
+#endif                          /* GLIB_CHECK_VERSION(2, 10, 0) */
     gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
                    0, gtk_get_current_event_time());
     g_object_unref(menu);
@@ -959,9 +1154,60 @@ bm_widget_new_html(BalsaMessage * bm, LibBalsaMessageBody * mime_body, gchar * p
                      G_CALLBACK(balsa_gtk_html_button_press_cb), bm);
     g_signal_connect(G_OBJECT(mw->widget), "key_press_event",
                      G_CALLBACK(balsa_mime_widget_key_press_event), bm);
-    g_signal_connect_swapped(G_OBJECT(mw->widget), "popup-menu",
+    g_signal_connect(G_OBJECT(mw->widget), "popup-menu",
                      G_CALLBACK(balsa_gtk_html_popup), bm);
 
     return mw;
 }
 #endif /* defined HAVE_GTKHTML */
+
+#define TABLE_ATTACH(t,str,label) \
+    if(str) { GtkWidget *lbl = gtk_label_new(label);              \
+        gtk_table_attach(t, lbl, 0, 1, row, row+1,                \
+                         GTK_FILL, GTK_FILL, 4, 2);               \
+        gtk_misc_set_alignment(GTK_MISC(lbl), 1.0, 0.0); \
+        gtk_table_attach(table, lbl=gtk_label_new(str), 1, 2, row, row+1, \
+                         GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 4, 2); \
+        gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.0); \
+        row++;                                                    \
+    }
+
+BalsaMimeWidget *
+bm_widget_new_vcard(BalsaMessage *bm, LibBalsaMessageBody *mime_body,
+                    gchar *ptr, size_t len)
+{
+    BalsaMimeWidget *mw = g_object_new(BALSA_TYPE_MIME_WIDGET, NULL);
+    LibBalsaAddress * addr =
+	libbalsa_address_new_from_vcard(ptr, mime_body->charset ? mime_body->charset : "us-ascii");
+    GtkTable *table;
+    GtkWidget *w;
+    int row = 1;
+
+    mw->widget = gtk_table_new(10, 2, FALSE);
+    table = (GtkTable*)mw->widget;
+        
+    gtk_table_attach_defaults(table, w=gtk_label_new(_("Address")),
+                              0, 1, 0, 1);
+    gtk_misc_set_alignment(GTK_MISC(w), 1.0, 0.0);
+    w = gtk_button_new_with_mnemonic(_("S_tore"));
+    /* FIXME: Connect signal */
+    gtk_table_attach(table, w, 1, 2, 0, 1, 0,0,  2, 2);
+    g_signal_connect_swapped(w, "clicked",
+                             G_CALLBACK(balsa_store_address), addr);
+    g_object_weak_ref(G_OBJECT(mw), (GWeakNotify)g_object_unref, addr);
+ 
+
+    TABLE_ATTACH(table, addr->full_name,    _("Full Name"));
+    TABLE_ATTACH(table, addr->nick_name,    _("Nick Name"));
+    TABLE_ATTACH(table, addr->first_name,   _("First Name"));
+    TABLE_ATTACH(table, addr->last_name,    _("Last Name"));
+    TABLE_ATTACH(table, addr->organization, _("Organization"));
+    if(addr->address_list) {
+        TABLE_ATTACH(table, addr->address_list->data, _("Email Address"));
+    }
+        
+    g_object_set_data(G_OBJECT(mw->widget), "mime-body", mime_body);
+    gtk_widget_show_all(mw->widget);
+    g_free(ptr);
+    return mw;
+}
