@@ -41,6 +41,8 @@
 #include "balsa-app.h"
 #include "save-restore.h"
 #include "mailbox-filter.h"
+#include "i18n.h"
+#include "libbalsa-conf.h"
 
 /* Defined in filter-edit-dialog.c*/
 extern option_list fe_search_type[4];
@@ -74,18 +76,13 @@ extern GtkWidget *fe_name_label;
 extern GtkWidget *fe_name_entry;
     
 /* widgets for the matching fields */
-GtkWidget *fe_match_frame;
 GtkWidget *fe_matching_fields_body;
 GtkWidget *fe_matching_fields_to;
 GtkWidget *fe_matching_fields_from;
 GtkWidget *fe_matching_fields_subject;
 GtkWidget *fe_matching_fields_cc;
 /* Combo list for user headers and check button*/
-#if GTK_CHECK_VERSION(2, 4, 0)
 GtkWidget * fe_user_header;
-#else
-GtkCombo * fe_user_header;
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
 GtkWidget * fe_matching_fields_us_head;
 
 /* widget for the conditions */
@@ -197,7 +194,7 @@ fe_free_associated_conditions(void)
         LibBalsaCondition *cond;
 
         gtk_tree_model_get(model, &iter, 1, &cond, -1);
-	libbalsa_condition_free(cond);
+	libbalsa_condition_unref(cond);
     }
 }
 
@@ -266,7 +263,6 @@ fe_regexs_selection_changed(GtkTreeSelection *selection,
     gtk_widget_set_sensitive(fe_regex_remove_button, selected);
 }
 
-#if GTK_CHECK_VERSION(2, 4, 0)
 /* Helper. */
 static gint
 fe_combo_box_get_value(GtkWidget * combo_box)
@@ -279,7 +275,6 @@ fe_combo_box_get_value(GtkWidget * combo_box)
     return GPOINTER_TO_INT(g_slist_nth_data(info->values, active));
 }
 
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
 /*
  * fe_typesmenu_cb()
  *
@@ -287,24 +282,27 @@ fe_combo_box_get_value(GtkWidget * combo_box)
  * When they are toggled, the notebook page must change
  */
 static void
-fe_typesmenu_cb(GtkWidget* widget, gpointer data)
+fe_typesmenu_cb(GtkWidget * widget, GtkWidget * field_frame)
 {
-#if GTK_CHECK_VERSION(2, 4, 0)
     ConditionMatchType type =
-	(ConditionMatchType) fe_combo_box_get_value(widget);
-#else /* GTK_CHECK_VERSION(2, 4, 0) */
-    ConditionMatchType type=GPOINTER_TO_INT(data);
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
+        (ConditionMatchType) fe_combo_box_get_value(widget);
 
-    condition_has_changed=TRUE;
+    condition_has_changed = TRUE;
     gtk_notebook_set_current_page(GTK_NOTEBOOK(fe_type_notebook),
                                   type - 1);
-    /* For certain types (date, flag)
-     * match fields have no meaning so we disable them
-     */
-    gtk_widget_set_sensitive(fe_match_frame,
-                             (type!=CONDITION_DATE && type!=CONDITION_FLAG));
-}                       /* end fe_typesmenu_cb() */
+
+    switch (type) {
+    case CONDITION_STRING:
+    case CONDITION_REGEX:
+        gtk_widget_show(field_frame);
+        break;
+    case CONDITION_DATE:
+    case CONDITION_FLAG:
+        gtk_widget_hide(field_frame);
+    default:
+        break;
+    }
+}                               /* end fe_typesmenu_cb() */
 
 typedef struct {
     const gchar * normal_str, *negate_str;
@@ -350,18 +348,9 @@ update_condition_list_label(void)
 static ConditionMatchType
 get_condition_type(void)
 {
-#if GTK_CHECK_VERSION(2, 4, 0)
     /* Set the type associated with the selected item */
     return (ConditionMatchType)
         fe_combo_box_get_value(fe_search_option_menu);
-#else /* GTK_CHECK_VERSION(2, 4, 0) */
-    GtkWidget * menu;
-
-    /* Retrieve the selected item in the search type menu */
-    menu=gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(fe_search_option_menu))));
-    /* Set the type associated with the selected item */
-    return GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menu),"value"));
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
 }
 
 /* fe_negate_condition :handle pressing on the "Contain/Does not
@@ -417,16 +406,11 @@ fe_match_fields_buttons_cb(GtkWidget * widget, gpointer data)
 static void
 fe_match_field_user_header_cb(GtkWidget * widget)
 {
-#if GTK_CHECK_VERSION(2, 4, 0)
     GtkToggleButton *button =
         GTK_TOGGLE_BUTTON(fe_matching_fields_us_head);
     gtk_widget_set_sensitive(fe_user_header,
                              gtk_toggle_button_get_active(button));
-#else
-    gtk_widget_set_sensitive(GTK_WIDGET(fe_user_header),
-			     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fe_matching_fields_us_head)));
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
-    condition_has_changed=TRUE;
+    condition_has_changed = TRUE;
 }
 
 void
@@ -444,7 +428,6 @@ fe_add_new_user_header(const gchar * str)
                              (GCompareFunc) g_ascii_strcasecmp);
 }
 
-#if GTK_CHECK_VERSION(2, 4, 0)
 static void
 fe_user_header_set_active(const gchar * user_header)
 {
@@ -481,7 +464,6 @@ fe_user_header_add(const gchar * user_header)
         fe_user_header_set_active(user_header);
     }
 }
-#endif                          /* GTK_CHECK_VERSION(2, 4, 0) */
 
 /* conditon_validate is responsible of validating
  * the changes to the current condition, according to the widgets
@@ -521,34 +503,19 @@ condition_validate(LibBalsaCondition* new_cnd)
             CONDITION_SETMATCH(new_cnd,CONDITION_MATCH_CC);
         if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fe_matching_fields_us_head))) {
 	    CONDITION_SETMATCH(new_cnd,CONDITION_MATCH_US_HEAD);
-#if GTK_CHECK_VERSION(2, 4, 0)
             str =
                 gtk_editable_get_chars(GTK_EDITABLE
                                        (GTK_BIN(fe_user_header)->child), 0,
                                        -1);
-#else
-            str = gtk_editable_get_chars(GTK_EDITABLE(fe_user_header->entry),
-                                         0, -1);
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
 	    if (!str[0]) {
                 balsa_information(LIBBALSA_INFORMATION_ERROR,
                                   _("You must specify the name of the "
                                     "user header to match on"));
 		return FALSE;
 	    }
-#if GTK_CHECK_VERSION(2, 4, 0)
             if (gtk_combo_box_get_active(GTK_COMBO_BOX(fe_user_header)) < 0)
 		/* User has changed the entry. */
 		fe_user_header_add(str);
-#else
-	    fe_add_new_user_header(str);
-	    /* This piece of code replaces the combo list
-	       by a new one that contains the new string the user has entered
-	       it seems that we must reset the text to the correct string,
-	    */
-	    gtk_combo_set_popdown_strings(fe_user_header,fe_user_headers_list);
-	    gtk_entry_set_text(GTK_ENTRY(fe_user_header->entry),str);
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
 	    g_free(str);
 	}
         else if (new_cnd->match.string.fields==CONDITION_EMPTY) {
@@ -617,15 +584,10 @@ condition_validate(LibBalsaCondition* new_cnd)
 
     new_cnd->negate = condition_not;
     if (CONDITION_CHKMATCH(new_cnd,CONDITION_MATCH_US_HEAD))
-#if GTK_CHECK_VERSION(2, 4, 0)
         new_cnd->match.string.user_header =
             gtk_editable_get_chars(GTK_EDITABLE
                                    (GTK_BIN(fe_user_header)->child), 0,
                                    -1);
-#else
-	new_cnd->match.string.user_header =
-            g_strdup(gtk_entry_get_text(GTK_ENTRY(fe_user_header->entry)));
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
     /* Set the type specific fields of the condition */
     switch (new_cnd->type) {
     case CONDITION_STRING:
@@ -653,7 +615,7 @@ condition_validate(LibBalsaCondition* new_cnd)
         for (row=0;row<2;row++)
             for (col=0;col<2;col++)
                 new_cnd->match.flags|=
-                    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fe_type_flag_buttons[row*2+col])) ? 1 << (row*2+col+1): 0;
+                    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fe_type_flag_buttons[row*2+col])) ? 1 << (row*2+col): 0;
 
     case CONDITION_NONE:
     case CONDITION_AND: /*FIXME: verify this! */
@@ -690,12 +652,27 @@ clear_condition_widgets()
 static void
 set_button_sensitivities(gboolean sensitive)
 {
+    GtkTreeModel *condition_model =
+        gtk_tree_view_get_model(fe_conditions_list);
+    GtkTreeSelection *filter_selection =
+        gtk_tree_view_get_selection(fe_filters_list);
+    gint n_conditions;
+    gboolean filter_selected;
+    gboolean filter_ok;
+
     gtk_widget_set_sensitive(fe_apply_button, sensitive);
     gtk_widget_set_sensitive(fe_revert_button, sensitive);
-    gtk_widget_set_sensitive(fe_new_button, !sensitive);
+
+    n_conditions = gtk_tree_model_iter_n_children(condition_model, NULL);
+    filter_selected =
+        gtk_tree_selection_get_selected(filter_selection, NULL, NULL);
+    filter_ok = !sensitive && (n_conditions > 0 || !filter_selected);
+
+    gtk_widget_set_sensitive(fe_new_button, filter_ok);
     gtk_dialog_set_response_sensitive(GTK_DIALOG(fe_window),
-                                      GTK_RESPONSE_OK, !sensitive);
-    gtk_widget_set_sensitive(GTK_WIDGET(fe_filters_list), !sensitive);
+                                      GTK_RESPONSE_OK, filter_ok);
+    gtk_widget_set_sensitive(GTK_WIDGET(fe_filters_list), filter_ok);
+    gtk_widget_set_sensitive(fe_op_codes_option_menu, n_conditions > 1);
 }
 
 /* fill_condition_widget : Fill all widget according to condition data
@@ -726,18 +703,12 @@ fill_condition_widgets(LibBalsaCondition* cnd)
     gtk_notebook_set_current_page(GTK_NOTEBOOK(fe_type_notebook),
                                   cnd->type - 1);
 
-#if GTK_CHECK_VERSION(2, 4, 0)
     gtk_combo_box_set_active(GTK_COMBO_BOX(fe_search_option_menu),
                              cnd->type - 1);
-#else /* GTK_CHECK_VERSION(2, 4, 0) */
-    gtk_option_menu_set_history(GTK_OPTION_MENU(fe_search_option_menu), 
-                                cnd->type-1);
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
 
     /* First update matching fields
      * but if type is date or flag, these are meaning less so we disable them */
     andmask = (cnd->type!=CONDITION_FLAG && cnd->type!=CONDITION_DATE);
-    gtk_widget_set_sensitive(fe_match_frame,andmask);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_matching_fields_body),
                                  CONDITION_CHKMATCH(cnd,CONDITION_MATCH_BODY) && andmask);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_matching_fields_to),
@@ -751,24 +722,12 @@ fill_condition_widgets(LibBalsaCondition* cnd)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_matching_fields_us_head),
                                  CONDITION_CHKMATCH(cnd,CONDITION_MATCH_US_HEAD) && andmask);
     if (CONDITION_CHKMATCH(cnd,CONDITION_MATCH_US_HEAD) && andmask) {
-#if GTK_CHECK_VERSION(2, 4, 0)
 	gtk_widget_set_sensitive(fe_user_header, TRUE);
 	fe_user_header_set_active(cnd->match.string.user_header);
-#else
-	gtk_widget_set_sensitive(GTK_WIDGET(fe_user_header),TRUE);
-	gtk_entry_set_text(GTK_ENTRY(fe_user_header->entry),
-                           cnd->match.string.user_header 
-                           ? cnd->match.string.user_header : "");
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
     }
     else {
-#if GTK_CHECK_VERSION(2, 4, 0)
 	gtk_widget_set_sensitive(fe_user_header,FALSE);
 	gtk_entry_set_text(GTK_ENTRY(GTK_BIN(fe_user_header)->child),"");
-#else
-	gtk_widget_set_sensitive(GTK_WIDGET(fe_user_header),FALSE);
-	gtk_entry_set_text(GTK_ENTRY(fe_user_header->entry),"");
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
     }	
     /* Next update type specific fields */
     switch (cnd->type) {
@@ -815,7 +774,7 @@ fill_condition_widgets(LibBalsaCondition* cnd)
         for (row=0;row<2;row++)
             for (col=0;col<2;col++)
                 gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_type_flag_buttons[row*2+col]),
-                                             cnd->match.flags & (1 << (row*2+col+1)));
+                                             cnd->match.flags & (1 << (row*2+col)));
         fe_update_label(fe_type_flag_label,   &flags_label);
         break;
     case CONDITION_NONE:
@@ -824,12 +783,8 @@ fill_condition_widgets(LibBalsaCondition* cnd)
         /* To avoid warnings :), we should never get there */
 	break;
     }
-#if GTK_CHECK_VERSION(2, 4, 0)
     gtk_combo_box_set_active(GTK_COMBO_BOX(fe_search_option_menu),
                              cnd->type - 1);
-#else /* GTK_CHECK_VERSION(2, 4, 0) */
-    gtk_menu_set_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(fe_search_option_menu))),cnd->type-1);
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
 }            /* end fill_condition_widget */
 
 static void
@@ -852,7 +807,7 @@ condition_dialog_response(GtkWidget * dialog, gint response,
                 return;
             }
 
-            /* No error occured, condition is valid, so change/add it
+            /* No error occurred, condition is valid, so change/add it
              * based on is_new_condition and only if something has
              * changed of course */
             selection = gtk_tree_view_get_selection(fe_conditions_list);
@@ -865,7 +820,7 @@ condition_dialog_response(GtkWidget * dialog, gint response,
                     LibBalsaCondition *cond;
 
                     gtk_tree_model_get(model, &iter, 1, &cond, -1);
-                    libbalsa_condition_free(cond);
+                    libbalsa_condition_unref(cond);
                 }
             } else {
                 /* It was a new condition, so add it to the list */
@@ -907,15 +862,85 @@ condition_dialog_response(GtkWidget * dialog, gint response,
     gtk_widget_set_sensitive(fe_window, TRUE);
 }
 
-/* build_type_notebook
- * build the notebook containing one page for each condition type (simple, regex, date, flag)
+/* build_type_notebook() builds the notebook containing one page for each
+ * condition type (simple, regex, date, flag)
  */
-static void build_type_notebook()
+static void
+add_button(GtkWidget *table, const char *label, int row,
+           GCallback action, int p)
+{
+    GtkWidget* button = gtk_button_new_with_mnemonic(label);
+    gtk_table_attach(GTK_TABLE(table),
+                     button,
+                     row, row+1, 4, 5,
+                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2); 
+    g_signal_connect(G_OBJECT(button), "clicked",
+                     action,
+                     GINT_TO_POINTER(p));
+}
+static GtkWidget*
+add_check(GtkWidget *table, const char *label, int col, int row)
+{
+    GtkWidget* res = gtk_check_button_new_with_mnemonic(label);
+    gtk_table_attach(GTK_TABLE(table), res,
+                     col, col+1, row, row+1,
+                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
+    g_signal_connect(G_OBJECT(res), "toggled",
+                     G_CALLBACK(fe_condition_changed_cb), NULL);
+    return res;
+}
+
+static GtkWidget*
+get_field_frame(void)
+{
+    GtkWidget *table;
+    GtkWidget *frame = gtk_frame_new(_("Match Fields"));
+    GList *list;
+
+    gtk_frame_set_label_align(GTK_FRAME(frame), 
+                              GTK_POS_LEFT, GTK_POS_TOP);
+    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
+    
+    table = gtk_table_new(5, 2, TRUE);
+    gtk_container_add(GTK_CONTAINER(frame), table);
+
+    add_button(table, _("_All"),  0,G_CALLBACK(fe_match_fields_buttons_cb),1);
+    add_button(table, _("C_lear"),1,G_CALLBACK(fe_match_fields_buttons_cb),3);
+
+    fe_matching_fields_body    = add_check(table, _("_Body"),    0,0);
+    fe_matching_fields_to      = add_check(table, _("_To:"),     0,1);
+    fe_matching_fields_from    = add_check(table, _("_From:"),   1,1);
+    fe_matching_fields_subject = add_check(table, _("_Subject"), 0,2);
+    fe_matching_fields_cc      = add_check(table,  _("_Cc:"),    1,2);
+    fe_matching_fields_us_head =
+        gtk_check_button_new_with_mnemonic(_("_User header:"));
+    gtk_table_attach(GTK_TABLE(table),
+                     fe_matching_fields_us_head,
+                     0, 1, 3, 4,
+                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
+    g_signal_connect(G_OBJECT(fe_matching_fields_us_head), "toggled",
+                     G_CALLBACK(fe_match_field_user_header_cb), NULL);
+
+    fe_user_header = gtk_combo_box_entry_new_text();
+    for (list = fe_user_headers_list; list; list = list->next)
+        gtk_combo_box_append_text(GTK_COMBO_BOX(fe_user_header),
+                                  list->data);
+    g_signal_connect(G_OBJECT(fe_user_header), "changed",
+                     G_CALLBACK(fe_condition_changed_cb), NULL);
+    gtk_table_attach(GTK_TABLE(table),
+                     fe_user_header,
+                     1, 2, 3, 4,
+                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
+    return frame;
+}
+
+static void
+build_type_notebook()
 {
     GtkWidget *page,*frame;
     GtkWidget *scroll;
     GtkWidget *box;
-    GtkWidget *button;
+    GtkWidget *button, *table;
     gint row,col;
     static gchar * flag_names[]=
         {N_("Unread"), N_("Deleted"), N_("Replied"), N_("Flagged")};
@@ -928,19 +953,20 @@ static void build_type_notebook()
 
     /* The simple page of the notebook */
 
-    page = gtk_table_new(5, 3, FALSE);
-    gtk_notebook_append_page(GTK_NOTEBOOK(fe_type_notebook), page, NULL);
+    box = gtk_vbox_new(FALSE, 5);
+    table = gtk_table_new(7, 2, FALSE);
+    gtk_box_pack_start_defaults(GTK_BOX(box), table);
 
     fe_type_simple_label = 
 	gtk_label_new_with_mnemonic(_("One of the specified f_ields contains"));
-    gtk_table_attach(GTK_TABLE(page),
+    gtk_table_attach(GTK_TABLE(table),
                      fe_type_simple_label,
                      0, 5, 0, 1,
                      GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 5, 5);
     fe_type_simple_entry = gtk_entry_new();
     g_signal_connect(G_OBJECT(fe_type_simple_entry), "changed",
                      G_CALLBACK(fe_condition_changed_cb), NULL);
-    gtk_table_attach(GTK_TABLE(page),
+    gtk_table_attach(GTK_TABLE(table),
                      fe_type_simple_entry,
                      0, 5, 1, 2,
                      GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 5, 5);
@@ -950,15 +976,20 @@ static void build_type_notebook()
     button = gtk_button_new_with_mnemonic(_("Contain/Does _Not Contain"));
     g_signal_connect(G_OBJECT(button), "clicked",
                      G_CALLBACK(fe_negate_condition), NULL);
-    gtk_table_attach(GTK_TABLE(page),
+    gtk_table_attach(GTK_TABLE(table),
                      button,
                      0, 5, 2, 3,
                      GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 5, 5);
+
+    gtk_notebook_append_page(GTK_NOTEBOOK(fe_type_notebook), box, NULL);
     
     /* The regex page of the type notebook */
+    box = gtk_vbox_new(FALSE, 5);
 
     page = gtk_table_new(5, 6, FALSE);
-    gtk_notebook_append_page(GTK_NOTEBOOK(fe_type_notebook), page, NULL);
+    gtk_box_pack_start_defaults(GTK_BOX(box), page);
+
+    gtk_notebook_append_page(GTK_NOTEBOOK(fe_type_notebook), box, NULL);
 
     fe_type_regex_label = 
         gtk_label_new_with_mnemonic(_("_One of the regular expressions matches"));
@@ -1013,7 +1044,7 @@ static void build_type_notebook()
     gtk_notebook_append_page(GTK_NOTEBOOK(fe_type_notebook), page, NULL);
 
     fe_type_date_label = 
-        gtk_label_new(_("Match when date is in the interval:"));
+        gtk_label_new(_("Match when message date is in the interval:"));
     gtk_table_attach(GTK_TABLE(page),
                      fe_type_date_label,
                      0, 5, 0, 1,
@@ -1092,132 +1123,27 @@ static void build_type_notebook()
 static
 void build_condition_dialog(GtkWidget * condition_dialog)
 {
-    GtkWidget * table,* label,* button,* page,* box;
-#if GTK_CHECK_VERSION(2, 4, 0)
-    GList *list;
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
+    GtkWidget *label,* box;
+    GtkWidget *field_frame = get_field_frame();
 
-    page = gtk_table_new(7, 2, FALSE);
-    /* builds the toggle buttons to specify fields concerned by the conditions of
-     * the filter */
-    
-    fe_match_frame = gtk_frame_new(_("Match In"));
-    gtk_frame_set_label_align(GTK_FRAME(fe_match_frame), 
-                              GTK_POS_LEFT, GTK_POS_TOP);
-    gtk_frame_set_shadow_type(GTK_FRAME(fe_match_frame), GTK_SHADOW_ETCHED_IN);
-    gtk_table_attach(GTK_TABLE(page),
-                     fe_match_frame,
-                     0, 2, 0, 2,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 5, 5);
-    
-    table = gtk_table_new(5, 2, TRUE);
-    gtk_container_add(GTK_CONTAINER(fe_match_frame), table);
-    
-    button = gtk_button_new_with_mnemonic(_("_All"));
-    gtk_table_attach(GTK_TABLE(table),
-                     button,
-                     0, 1, 4, 5,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2); 
-    g_signal_connect(G_OBJECT(button), "clicked",
-                     G_CALLBACK(fe_match_fields_buttons_cb),
-                     GINT_TO_POINTER(1));
-    button = gtk_button_new_with_mnemonic(_("C_lear"));
-    gtk_table_attach(GTK_TABLE(table),
-                     button,
-                     1, 2, 4, 5,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
-    g_signal_connect(G_OBJECT(button), "clicked",
-                     G_CALLBACK(fe_match_fields_buttons_cb),
-                     GINT_TO_POINTER(3));
-    
-    fe_matching_fields_body = gtk_check_button_new_with_mnemonic(_("_Body"));
-    gtk_table_attach(GTK_TABLE(table),
-                     fe_matching_fields_body,
-                     0, 1, 0, 1,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
-    g_signal_connect(G_OBJECT(fe_matching_fields_body), "toggled",
-                     G_CALLBACK(fe_condition_changed_cb), NULL);
-    fe_matching_fields_to = gtk_check_button_new_with_mnemonic(_("_To:"));
-    gtk_table_attach(GTK_TABLE(table),
-                     fe_matching_fields_to,
-                     0, 1, 1, 2,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
-    g_signal_connect(G_OBJECT(fe_matching_fields_to), "toggled",
-                     G_CALLBACK(fe_condition_changed_cb), NULL);
-    fe_matching_fields_from = gtk_check_button_new_with_mnemonic(_("_From:"));
-    gtk_table_attach(GTK_TABLE(table),
-                     fe_matching_fields_from,
-                     1, 2, 1, 2,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
-    g_signal_connect(G_OBJECT(fe_matching_fields_from), "toggled",
-                     G_CALLBACK(fe_condition_changed_cb), NULL);
-    fe_matching_fields_subject = 
-	gtk_check_button_new_with_mnemonic(_("_Subject"));
-    gtk_table_attach(GTK_TABLE(table),
-                     fe_matching_fields_subject,
-                     0, 1, 2, 3,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
-    g_signal_connect(G_OBJECT(fe_matching_fields_subject), "toggled",
-                     G_CALLBACK(fe_condition_changed_cb), NULL);
-    fe_matching_fields_cc = gtk_check_button_new_with_mnemonic(_("_Cc:"));
-    gtk_table_attach(GTK_TABLE(table),
-                     fe_matching_fields_cc,
-                     1, 2, 2, 3,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
-    g_signal_connect(G_OBJECT(fe_matching_fields_cc), "toggled",
-                     G_CALLBACK(fe_condition_changed_cb), NULL);
-    fe_matching_fields_us_head = gtk_check_button_new_with_mnemonic(_("_User header:"));
-    gtk_table_attach(GTK_TABLE(table),
-                     fe_matching_fields_us_head,
-                     0, 1, 3, 4,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
-    g_signal_connect(G_OBJECT(fe_matching_fields_us_head), "toggled",
-                     G_CALLBACK(fe_match_field_user_header_cb), NULL);
-#if GTK_CHECK_VERSION(2, 4, 0)
-    fe_user_header = gtk_combo_box_entry_new_text();
-    for (list = fe_user_headers_list; list; list = list->next)
-        gtk_combo_box_append_text(GTK_COMBO_BOX(fe_user_header),
-                                  list->data);
-    g_signal_connect(G_OBJECT(fe_user_header), "changed",
-                     G_CALLBACK(fe_condition_changed_cb), NULL);
-    gtk_table_attach(GTK_TABLE(table),
-                     fe_user_header,
-                     1, 2, 3, 4,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
-#else
-    fe_user_header = GTK_COMBO(gtk_combo_new());
-    gtk_combo_set_value_in_list(fe_user_header,FALSE,FALSE);
-    gtk_combo_set_case_sensitive(fe_user_header,FALSE);
-    gtk_combo_set_popdown_strings(fe_user_header,fe_user_headers_list);
-    g_signal_connect(G_OBJECT(fe_user_header->entry), "changed",
-                     G_CALLBACK(fe_condition_changed_cb), NULL);
-    gtk_table_attach(GTK_TABLE(table),
-                     GTK_WIDGET(fe_user_header),
-                     1, 2, 3, 4,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
-    
+    /* builds the toggle buttons to specify fields concerned by the
+     * conditions of the filter */
     box = gtk_hbox_new(FALSE, 5);
-    gtk_table_attach(GTK_TABLE(page),
-                     box,
-                     0, 2, 2, 3,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 5, 5);
-
     label = gtk_label_new_with_mnemonic(_("Search T_ype:"));
     gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 5);
     fe_search_option_menu = 
         fe_build_option_menu(fe_search_type, ELEMENTS(fe_search_type),
-                             G_CALLBACK(fe_typesmenu_cb));
+                             G_CALLBACK(fe_typesmenu_cb), field_frame);
     gtk_box_pack_start(GTK_BOX(box), fe_search_option_menu, FALSE, FALSE, 5);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), fe_search_option_menu);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(condition_dialog)->vbox),
+                       box, FALSE, FALSE, 2);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(condition_dialog)->vbox),
+                       field_frame, FALSE, FALSE, 2);
 
     build_type_notebook();
-    gtk_table_attach(GTK_TABLE(page),
-                     fe_type_notebook,
-                     0, 2, 3, 7,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND,
-                     GTK_FILL | GTK_SHRINK | GTK_EXPAND, 5, 5);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(condition_dialog)->vbox),page,FALSE,FALSE,2);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(condition_dialog)->vbox),
+                       fe_type_notebook, FALSE, FALSE, 2);
 }
 
 /*
@@ -1312,7 +1238,7 @@ fe_condition_remove_pressed(GtkWidget * widget, gpointer data)
         return;
 
     gtk_tree_model_get(model, &iter, 1, &cond, -1);
-    libbalsa_condition_free(cond);
+    libbalsa_condition_unref(cond);
     path = gtk_tree_model_get_path(model, &iter);
     gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
 
@@ -1366,14 +1292,14 @@ update_filters_mailbox(GtkTreeModel * model, GtkTreePath * path,
     if (tmp) {
 	gchar **filters_names = NULL;
 	gboolean def;
-	guint nb_filters;
+	gint nb_filters;
 
-	gnome_config_push_prefix(tmp);
-	gnome_config_get_vector_with_default(MAILBOX_FILTERS_KEY,
+	libbalsa_conf_push_group(tmp);
+	libbalsa_conf_get_vector_with_default(MAILBOX_FILTERS_KEY,
 					     &nb_filters, &filters_names,
 					     &def);
 	if (!def) {
-	    guint i;
+	    gint i;
 	    GList *lst;
 
 	    for (i = 0; i < nb_filters;) {
@@ -1392,7 +1318,7 @@ update_filters_mailbox(GtkTreeModel * model, GtkTreePath * path,
 				     new_name);
 		    } else {
 			/* Name removing */
-			guint j;
+			gint j;
 
 			for (j = i; j < nb_filters - 1; j++)
 			    filters_names[j] = filters_names[j + 1];
@@ -1407,12 +1333,12 @@ update_filters_mailbox(GtkTreeModel * model, GtkTreePath * path,
 	}
 
 	if (nb_filters) {
-	    gnome_config_set_vector(MAILBOX_FILTERS_KEY, nb_filters,
+	    libbalsa_conf_set_vector(MAILBOX_FILTERS_KEY, nb_filters,
 				    (const gchar **) filters_names);
-	    gnome_config_pop_prefix();
+	    libbalsa_conf_pop_group();
 	} else {
-	    gnome_config_pop_prefix();
-	    gnome_config_clean_section(tmp);
+	    libbalsa_conf_pop_group();
+	    libbalsa_conf_remove_group(tmp);
 	}
 	g_strfreev(filters_names);
 	g_free(tmp);
@@ -1491,7 +1417,11 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
             LibBalsaFilter *fil;
 
             gtk_tree_model_get(model, &iter, 1, &fil, -1);
-            balsa_app.filters = g_slist_prepend(balsa_app.filters, fil);
+            if (fil->condition)
+                balsa_app.filters = g_slist_prepend(balsa_app.filters, fil);
+            else
+                libbalsa_information(LIBBALSA_INFORMATION_WARNING,
+                                     _("Filter with no condition was omitted"));
         }
 
         /* Tell the clean-up functions not to free
@@ -1537,15 +1467,10 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
 void
 fe_action_selected(GtkWidget * widget, gpointer data)
 {
-#if GTK_CHECK_VERSION(2, 4, 0)
     FilterActionType type =
         (FilterActionType) fe_combo_box_get_value(widget);
     gtk_widget_set_sensitive(GTK_WIDGET(fe_mailboxes),
                              type != FILTER_TRASH);
-#else /* GTK_CHECK_VERSION(2, 4, 0) */
-    gtk_widget_set_sensitive(GTK_WIDGET(fe_mailboxes),
-                             GPOINTER_TO_INT(data)!=FILTER_TRASH);
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
     set_button_sensitivities(TRUE);
 }                       /* end fe_action_selected() */
 
@@ -1758,7 +1683,7 @@ fe_new_pressed(GtkWidget * widget, gpointer data)
 
     /* Adds "New Filter" to the list of actual new filters names */
     new_filters_names=g_list_prepend(new_filters_names,g_strdup(new_item));
-    gtk_widget_grab_focus(fe_name_label);
+    gtk_widget_grab_focus(fe_name_entry);
 }                       /* end fe_new_pressed() */
 
 /*
@@ -1794,21 +1719,25 @@ fe_delete_pressed(GtkWidget * widget, gpointer data)
         gtk_tree_selection_select_path(selection, path);
     else {
         /* ...the store is empty: */
-        /* We make the filters delete,revert,apply buttons unsensitive */
-        gtk_widget_set_sensitive(fe_delete_button,FALSE);
-        set_button_sensitivities(FALSE);
         /* We clear all widgets */
         gtk_entry_set_text(GTK_ENTRY(fe_name_entry),"");
         gtk_entry_set_text(GTK_ENTRY(fe_popup_entry),"");
         /*gtk_option_menu_set_history(GTK_OPTION_MENU(fe_mailboxes), 0); */
+#if !GTK_CHECK_VERSION(2, 6, 0)
         gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(fe_sound_entry))),"");
+#endif /* GTK_CHECK_VERSION(2, 6, 0) */
         gtk_list_store_clear(GTK_LIST_STORE
                              (gtk_tree_view_get_model
                               (fe_conditions_list)));
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_sound_button),FALSE);
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_popup_button),FALSE);
+        /* We make the filters delete,revert,apply buttons unsensitive */
+        gtk_widget_set_sensitive(fe_delete_button,FALSE);
+        set_button_sensitivities(FALSE);
     }
     gtk_tree_path_free(path);
+    gtk_dialog_set_response_sensitive(GTK_DIALOG(fe_window),
+                                      GTK_RESPONSE_OK, TRUE);
 }                       /* end fe_delete_pressed() */
 
 /*
@@ -1830,9 +1759,6 @@ fe_apply_pressed(GtkWidget * widget, gpointer data)
     GtkTreeIter cond_iter;
     LibBalsaFilter *fil,*old;
     const gchar *temp;
-#if !GTK_CHECK_VERSION(2, 4, 0)
-    GtkWidget * menu;
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
     FilterActionType action;
     ConditionMatchType condition_op;
 
@@ -1852,18 +1778,9 @@ fe_apply_pressed(GtkWidget * widget, gpointer data)
         return;
     }
     
-#if GTK_CHECK_VERSION(2, 4, 0)
     /* Set the type associated with the selected item */
     action =
         (FilterActionType) fe_combo_box_get_value(fe_action_option_menu);
-#else /* GTK_CHECK_VERSION(2, 4, 0) */
-    /* Retrieve the selected item in the action menu */
-    menu = gtk_menu_get_active(GTK_MENU
-                               (gtk_option_menu_get_menu
-                                (GTK_OPTION_MENU(fe_action_option_menu))));
-    /* Set the type associated with the selected item */
-    action=GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menu),"value"));
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
     
     if (!gtk_tree_model_get_iter_first(cond_model, &cond_iter)) {
         balsa_information(LIBBALSA_INFORMATION_ERROR,
@@ -1885,22 +1802,10 @@ fe_apply_pressed(GtkWidget * widget, gpointer data)
 
     /* Retrieve the selected item in the op codes menu */
 
-#if GTK_CHECK_VERSION(2, 4, 0)
     /* Set the op-codes associated with the selected item */
     condition_op =
         (FilterOpType) fe_combo_box_get_value(fe_op_codes_option_menu) ==
         FILTER_OP_OR ? CONDITION_OR : CONDITION_AND;
-#else /* GTK_CHECK_VERSION(2, 4, 0) */
-    menu = gtk_menu_get_active(GTK_MENU
-                               (gtk_option_menu_get_menu
-                                (GTK_OPTION_MENU
-                                 (fe_op_codes_option_menu))));
-
-    /* Set the op-codes associated with the selected item */
-    condition_op =
-	GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menu), "value")) ==
-	FILTER_OP_OR ? CONDITION_OR : CONDITION_AND;
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
 
     /* Retrieve all conditions for that filter */
 
@@ -1919,7 +1824,7 @@ fe_apply_pressed(GtkWidget * widget, gpointer data)
 
         gtk_tree_model_get(cond_model, &cond_iter, 1, &cond, -1);
         libbalsa_filter_prepend_condition(fil,
-                                          libbalsa_condition_clone(cond),
+                                          libbalsa_condition_ref(cond),
                                           condition_op);
     } while (gtk_tree_model_iter_next(cond_model, &cond_iter));
 
@@ -1946,20 +1851,27 @@ fe_apply_pressed(GtkWidget * widget, gpointer data)
                                       '\0')) ? _(defstring) : tmpstr);
     }
 
-#ifdef HAVE_LIBESD
+/* FIXME never defined?? #ifdef HAVE_LIBESD */
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fe_sound_button))) {
         gchar *tmpstr;
         
-        tmpstr = gtk_entry_get_text(GTK_ENTRY(fe_sound_entry));
+#if GTK_CHECK_VERSION(2, 6, 0)
+        tmpstr = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER
+                                               (fe_sound_entry));
+#else /* GTK_CHECK_VERSION(2, 6, 0) */
+        tmpstr =
+            gtk_editable_get_chars(GTK_EDITABLE(fe_sound_entry), 0, -1);
+#endif /* GTK_CHECK_VERSION(2, 6, 0) */
         if ((!tmpstr) || (tmpstr[0] == '\0')) {
+            g_free(tmpstr);
             libbalsa_filter_free(fil, GINT_TO_POINTER(TRUE));
 	    balsa_information(LIBBALSA_INFORMATION_ERROR,
 			      _("You must provide a sound to play"));
             return;
         }
-        fil->sound=g_strdup(tmpstr);
+        fil->sound = tmpstr;
     }
-#endif
+/* #endif *//* HAVE_LIBESD */
     /* New filter is OK, we replace the old one */
     gtk_tree_model_get(model, &iter, 1, &old, -1);
     change_filter_name(old->name, fil->name);
@@ -1969,6 +1881,8 @@ fe_apply_pressed(GtkWidget * widget, gpointer data)
     gtk_list_store_set(GTK_LIST_STORE(model), &iter,
                        0, fil->name, 1, fil, -1);
     gtk_tree_selection_select_iter(selection, &iter);
+    gtk_dialog_set_response_sensitive(GTK_DIALOG(fe_window),
+                                      GTK_RESPONSE_OK, TRUE);
 }                       /* end fe_apply_pressed */
 
 
@@ -2019,7 +1933,7 @@ fill_condition_list(GtkTreeModel *model, LibBalsaCondition *condition,
     gtk_list_store_prepend(GTK_LIST_STORE(model), &iter);
     gtk_list_store_set(GTK_LIST_STORE(model), &iter,
                        0, _(fe_search_type[condition->type-1].text),
-                       1, libbalsa_condition_clone(condition),
+                       1, libbalsa_condition_ref(condition),
                        -1);
 }
 
@@ -2049,23 +1963,20 @@ fe_filters_list_selection_changed(GtkTreeSelection * selection,
                        ? fil->popup_text : "");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_sound_button),
                                  fil->sound!=NULL);
+#if GTK_CHECK_VERSION(2, 6, 0)
+    if (fil->sound)
+        gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(fe_sound_entry),
+                                      fil->sound);
+#else /* GTK_CHECK_VERSION(2, 6, 0) */
     gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(fe_sound_entry))),
                        fil->sound!=NULL ? fil->sound : "");
+#endif /* GTK_CHECK_VERSION(2, 6, 0) */
     
-#if GTK_CHECK_VERSION(2, 4, 0)
     gtk_combo_box_set_active(GTK_COMBO_BOX(fe_action_option_menu),
                              fil->action - 1);
     pos = (fil->condition
            && fil->condition->type == CONDITION_AND) ? 1 : 0;
     gtk_combo_box_set_active(GTK_COMBO_BOX(fe_op_codes_option_menu), pos);
-#else /* GTK_CHECK_VERSION(2, 4, 0) */
-    gtk_option_menu_set_history(GTK_OPTION_MENU(fe_action_option_menu), 
-                                fil->action-1);
-    pos = (fil->condition && fil->condition->type == CONDITION_AND)
-        ? 1 : 0;
-    gtk_option_menu_set_history(GTK_OPTION_MENU(fe_op_codes_option_menu), 
-                                pos);
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
     if (fil->action!=FILTER_TRASH && fil->action_string)
         balsa_mblist_mru_option_menu_set(fe_mailboxes,
                                          fil->action_string);
@@ -2092,6 +2003,9 @@ fe_filters_list_selection_changed(GtkTreeSelection * selection,
         gtk_widget_set_sensitive(fe_op_codes_option_menu,
                                  gtk_tree_model_iter_n_children(model,
                                                                 NULL) > 1);
+    } else {
+        gtk_widget_set_sensitive(fe_condition_edit_button, FALSE);
+        gtk_widget_set_sensitive(fe_condition_delete_button, FALSE);
     }
 
     /* We set the filters delete,revert,apply buttons sensitivity */
@@ -2118,3 +2032,13 @@ fe_date_sample(void)
 
     return label;
 }                               /* end fe_date_sample */
+
+#if GTK_CHECK_VERSION(2, 6, 0)
+/* Callback for the sound file-chooser-button's dialog. */
+void
+fe_sound_response(GtkDialog * dialog, gint response)
+{
+    if (response == GTK_RESPONSE_ACCEPT)
+        set_button_sensitivities(TRUE);
+}
+#endif /* GTK_CHECK_VERSION(2, 6, 0) */
