@@ -62,6 +62,7 @@
 #include <bonobo-activation/bonobo-activation.h>
 
 #ifdef HAVE_GPGME
+#include <string.h>
 #include <gpgme.h>
 #endif
 
@@ -201,8 +202,8 @@ balsa_handle_automation_options() {
 static void
 balsa_init(int argc, char **argv)
 {
-    static char *attachment = NULL;
 #ifndef GNOME_PARAM_GOPTION_CONTEXT
+    static char *attachment = NULL;
     int opt;
     poptContext context;
     static struct poptOption options[] = {
@@ -254,13 +255,14 @@ balsa_init(int argc, char **argv)
                        NULL);
 #else /* USE GOption interface */
     static gchar **remaining_args = NULL;
+    static gchar **attach_vect = NULL;
     static GOptionEntry option_entries[] = {
 	{"checkmail", 'c', 0, G_OPTION_ARG_NONE,
 	 &(cmd_check_mail_on_startup),
 	 N_("Get new mail on startup"), NULL},
 	{"compose", 'm', 0, G_OPTION_ARG_STRING, &(opt_compose_email),
 	 N_("Compose a new email to EMAIL@ADDRESS"), "EMAIL@ADDRESS"},
-	{"attach", 'a', 0, G_OPTION_ARG_STRING, &(attachment),
+	{"attach", 'a', 0, G_OPTION_ARG_FILENAME_ARRAY, &(attach_vect),
 	 N_("Attach file at PATH"), "PATH"},
 	{"open-mailbox", 'o', 0, G_OPTION_ARG_STRING,
          &(cmd_line_open_mailboxes),
@@ -306,6 +308,16 @@ balsa_init(int argc, char **argv)
         g_strfreev (remaining_args);
         remaining_args = NULL;
     }
+    if (attach_vect != NULL) {
+        gint i, num_args;
+        
+        num_args = g_strv_length (attach_vect);
+        for (i = 0; i < num_args; ++i) {
+            opt_attach_list = g_slist_append(opt_attach_list, attach_vect[i]);
+        }
+        g_free(attach_vect);
+        attach_vect = NULL;
+    }
 #endif /* OPTION HANDLING */
     balsa_handle_automation_options();  
     
@@ -558,7 +570,7 @@ periodic_expunge_cb(void)
 #endif
 
     gdk_threads_enter();
-    libbalsa_information(LIBBALSA_INFORMATION_MESSAGE,
+    libbalsa_information(LIBBALSA_INFORMATION_DEBUG,
                          _("Compressing mail folders..."));
     gtk_tree_model_foreach(GTK_TREE_MODEL(balsa_app.mblist_tree_store),
 			   (GtkTreeModelForeachFunc)mbnode_expunge_func,
@@ -567,7 +579,8 @@ periodic_expunge_cb(void)
 
     for (l = list; l; l = l->next) {
         BalsaMailboxNode *mbnode = l->data;
-        if (mbnode->mailbox && libbalsa_mailbox_is_open(mbnode->mailbox)) {
+        if (mbnode->mailbox && libbalsa_mailbox_is_open(mbnode->mailbox)
+            && !mbnode->mailbox->readonly) {
             time_t tm = time(NULL);
             if (tm-mbnode->last_use > balsa_app.expunge_timeout)
                 libbalsa_mailbox_sync_storage(mbnode->mailbox, TRUE);
@@ -597,6 +610,12 @@ balsa_progress_set_text(LibBalsaProgress * progress, const gchar * text,
     gboolean rc = FALSE;
 
     gdk_threads_enter();
+
+    if (!balsa_app.main_window) {
+        gdk_threads_leave();
+        return;
+    }
+
     if (!text || total >= LIBBALSA_PROGRESS_MIN_COUNT)
         rc = balsa_window_setup_progress(balsa_app.main_window, text);
     prev_fraction = 0;
@@ -620,11 +639,32 @@ balsa_progress_set_fraction(LibBalsaProgress * progress, gdouble fraction)
     prev_fraction = fraction;
 
     gdk_threads_enter();
-    balsa_window_increment_progress(balsa_app.main_window, fraction,
-                                    !libbalsa_am_i_subthread());
+    if (balsa_app.main_window)
+        balsa_window_increment_progress(balsa_app.main_window, fraction,
+                                        !libbalsa_am_i_subthread());
     gdk_threads_leave();
 }
 
+#if defined(ENABLE_NLS) && defined(HAVE_GPGME)
+static const gchar *
+get_utf8_locale(int category)
+{
+    gchar * locale;
+    static gchar localebuf[64];  /* should be large enough */
+    gchar * dot;
+
+    if (!(locale = setlocale(category, NULL)))
+	return NULL;
+    strncpy(localebuf, locale, 57);
+    localebuf[57] = '\0';
+    dot = strchr(localebuf, '.');
+    if (!dot)
+	dot = localebuf + strlen(localebuf);
+    strcpy(dot, ".UTF-8");
+    return localebuf;
+}
+#endif
+
 /* -------------------------- main --------------------------------- */
 int
 main(int argc, char *argv[])
@@ -644,19 +684,33 @@ main(int argc, char *argv[])
     setlocale(LC_ALL, "");
 #endif
 
+#ifdef BALSA_USE_THREADS
+    /* initiate thread mutexs, variables */
+    threads_init();
+#endif
 
 #ifdef HAVE_GPGME
     /* initialise the gpgme library */
     g_message("init gpgme version %s", gpgme_check_version(NULL));
-#ifdef ENABLE_NLS
-    gpgme_set_locale(NULL, LC_CTYPE, setlocale(LC_CTYPE, NULL));
-    gpgme_set_locale(NULL, LC_MESSAGES, setlocale(LC_MESSAGES, NULL));
-#endif /* ENABLE_NLS */
+#ifdef HAVE_GPG
+    /* configure the GnuPG engine if a specific gpg path has been detected*/
+    gpgme_set_engine_info(GPGME_PROTOCOL_OpenPGP, GPG_PATH, NULL);
 #endif
+#ifdef ENABLE_NLS
+    gpgme_set_locale(NULL, LC_CTYPE, get_utf8_locale(LC_CTYPE));
+    gpgme_set_locale(NULL, LC_MESSAGES, get_utf8_locale(LC_MESSAGES));
+#endif /* ENABLE_NLS */
+    { /* scope */
+	gpgme_engine_info_t e;
 
-#ifdef BALSA_USE_THREADS
-    /* initiate thread mutexs, variables */
-    threads_init();
+	if (gpgme_get_engine_info(&e) == GPG_ERR_NO_ERROR)
+	    while (e) {
+		g_message("protocol %s: engine %s (home %s, version %s)",
+			  gpgme_get_protocol_name(e->protocol),
+			  e->file_name, e->home_dir, e->version);
+		e = e->next;
+	    }
+    }
 #endif
 
     /* FIXME: do we need to allow a non-GUI mode? */
@@ -739,7 +793,7 @@ main(int argc, char *argv[])
         BalsaSendmsg *snd;
         GSList *lst;
         gdk_threads_enter();
-        snd = sendmsg_window_new(window, NULL, SEND_NORMAL);
+        snd = sendmsg_window_compose(window);
         gdk_threads_leave();
         if(opt_compose_email) {
             if(g_ascii_strncasecmp(opt_compose_email, "mailto:", 7) == 0)
