@@ -34,12 +34,15 @@
 #include "mailbox-node.h"
 
 #include "libbalsa.h"
+#include "i18n.h"
 
 typedef struct _MessageWindow MessageWindow;
 
 /* callbacks */
 static void destroy_message_window(GtkWidget * widget, MessageWindow * mw);
-static void close_message_window(GtkWidget * widget, gpointer data);
+static void mw_expunged_cb(LibBalsaMailbox * mailbox, guint msgno,
+                           MessageWindow * mw);
+static void close_message_window_cb(GtkWidget * widget, gpointer data);
 
 static void replyto_message_cb(GtkWidget * widget, gpointer data);
 static void replytoall_message_cb(GtkWidget * widget, gpointer data);
@@ -61,6 +64,8 @@ static void wrap_message_cb(GtkWidget * widget, gpointer data);
 static void size_alloc_cb(GtkWidget * window, GtkAllocation * alloc);
 static void mw_set_buttons_sensitive(MessageWindow * mw);
 
+static void mw_set_selected(MessageWindow * mw, void (*select_func)(BalsaIndex *));
+
 static void copy_cb(GtkWidget * widget, MessageWindow * mw);
 static void select_all_cb(GtkWidget * widget, gpointer);
 
@@ -68,6 +73,9 @@ static void next_message_cb(GtkWidget * widget, gpointer data);
 static void previous_message_cb(GtkWidget * widget, gpointer data);
 static void next_unread_cb(GtkWidget * widget, gpointer);
 static void next_flagged_cb(GtkWidget * widget, gpointer);
+#ifdef HAVE_GTK_PRINT
+static void page_setup_cb(GtkWidget * widget, gpointer data);
+#endif
 static void print_cb(GtkWidget * widget, gpointer);
 static void trash_cb(GtkWidget * widget, gpointer);
 
@@ -76,8 +84,8 @@ static void message_window_move_message (MessageWindow * mw,
 static void reset_show_all_headers(MessageWindow *mw);
 #ifdef HAVE_GTKHTML
 static void mw_zoom_cb(GtkWidget * widget, MessageWindow * mw);
-static void mw_select_part_cb(BalsaMessage * bm, MessageWindow * mw);
 #endif                          /* HAVE_GTKHTML */
+static void mw_select_part_cb(BalsaMessage * bm, MessageWindow * mw);
 
 static GnomeUIInfo shown_hdrs_menu[] = {
     GNOMEUIINFO_RADIOITEM(N_("N_o Headers"), NULL,
@@ -90,10 +98,20 @@ static GnomeUIInfo shown_hdrs_menu[] = {
 };
 
 static GnomeUIInfo file_menu[] = {
+#ifdef HAVE_GTK_PRINT
+    { GNOME_APP_UI_ITEM, N_("Page _Setup"), 
+      N_("Set up page for printing"),
+      page_setup_cb, NULL, NULL, GNOME_APP_PIXMAP_NONE,
+      NULL, 'S', GDK_CONTROL_MASK, NULL},
+#endif
     GNOMEUIINFO_MENU_PRINT_ITEM(print_cb, NULL),
     GNOMEUIINFO_SEPARATOR,
+#ifdef HAVE_GTK_PRINT
+#define MENU_FILE_CLOSE_POS 3
+#else
 #define MENU_FILE_CLOSE_POS 2
-    GNOMEUIINFO_MENU_CLOSE_ITEM(close_message_window, NULL),
+#endif
+    GNOMEUIINFO_MENU_CLOSE_ITEM(close_message_window_cb, NULL),
     GNOMEUIINFO_END
 };
 
@@ -144,88 +162,88 @@ static GnomeUIInfo message_menu[] = {
     {
      GNOME_APP_UI_ITEM, N_("_Reply..."), N_("Reply to this message"),
      replyto_message_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_REPLY, 'R', 0, NULL},
+     BALSA_PIXMAP_REPLY, 'R', 0, NULL},
     /* A */
     {
      GNOME_APP_UI_ITEM, N_("Reply to _All..."),
      N_("Reply to all recipients of this message"),
      replytoall_message_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_REPLY_ALL, 'A', 0, NULL},
+     BALSA_PIXMAP_REPLY_ALL, 'A', 0, NULL},
     /* G */
     {
      GNOME_APP_UI_ITEM, N_("Reply to _Group..."),
      N_("Reply to mailing list"),
      replytogroup_message_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_REPLY_GROUP, 'G', 0, NULL},
+     BALSA_PIXMAP_REPLY_GROUP, 'G', 0, NULL},
     /* F */
     {
      GNOME_APP_UI_ITEM, N_("_Forward attached..."),
      N_("Forward this message as attachment"),
      forward_message_attached_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_FORWARD, 'F', 0, NULL},
+     BALSA_PIXMAP_FORWARD, 'F', 0, NULL},
     {
      GNOME_APP_UI_ITEM, N_("Forward inline..."), 
      N_("Forward this message inline"),
      forward_message_inline_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_FORWARD, 'F', GDK_CONTROL_MASK, NULL},
+     BALSA_PIXMAP_FORWARD, 'F', GDK_CONTROL_MASK, NULL},
     GNOMEUIINFO_SEPARATOR,
 #define MENU_MESSAGE_NEXT_PART_POS 6
     {
      GNOME_APP_UI_ITEM, N_("Next Part"), N_("Next part in message"),
      next_part_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_NEXT_PART, '.', GDK_CONTROL_MASK, NULL},
+     BALSA_PIXMAP_NEXT_PART, '.', GDK_CONTROL_MASK, NULL},
 #define MENU_MESSAGE_PREVIOUS_PART_POS 7
     {
      GNOME_APP_UI_ITEM, N_("Previous Part"),
      N_("Previous part in message"),
      previous_part_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_PREVIOUS_PART, ',', GDK_CONTROL_MASK, NULL},
+     BALSA_PIXMAP_PREVIOUS_PART, ',', GDK_CONTROL_MASK, NULL},
     {
      GNOME_APP_UI_ITEM, N_("Save Current Part..."),
      N_("Save current part in message"),
      save_current_part_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_SAVE, 's', GDK_CONTROL_MASK, NULL},
+     GTK_STOCK_SAVE, 's', GDK_CONTROL_MASK, NULL},
     {
      GNOME_APP_UI_ITEM, N_("_View Source..."),
      N_("View source form of the message"),
      view_msg_source_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_SAVE, 'v', GDK_CONTROL_MASK, NULL},
+     BALSA_PIXMAP_BOOK_OPEN, 'v', GDK_CONTROL_MASK, NULL},
      GNOMEUIINFO_SEPARATOR,
     /* N */
 #define MENU_MESSAGE_NEXT_POS 11
     {
      GNOME_APP_UI_ITEM, N_("_Next Message"), N_("Next message"),
      next_message_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_NEXT, 'N', 0, NULL},
+     BALSA_PIXMAP_NEXT, 'N', 0, NULL},
     /* P */
 #define MENU_MESSAGE_PREVIOUS_POS 12
     {
      GNOME_APP_UI_ITEM, N_("_Previous Message"),
      N_("Previous Message"),
      previous_message_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     BALSA_PIXMAP_MENU_PREVIOUS, 'P', 0, NULL},
+     BALSA_PIXMAP_PREVIOUS, 'P', 0, NULL},
     /* Ctrl + N */
 #define MENU_MESSAGE_NEXT_UNREAD_POS 13
     {
      GNOME_APP_UI_ITEM, N_("Next Unread Message"),
      N_("Next Unread Message"), next_unread_cb, NULL, NULL,
-     GNOME_APP_PIXMAP_STOCK, BALSA_PIXMAP_MENU_NEXT_UNREAD, 'N',
+     GNOME_APP_PIXMAP_STOCK, BALSA_PIXMAP_NEXT_UNREAD, 'N',
      GDK_CONTROL_MASK, NULL},
     /* Ctrl + Alt + F */
 #define MENU_MESSAGE_NEXT_FLAGGED_POS 14
     {
      GNOME_APP_UI_ITEM, N_("Next Flagged Message"),
      N_("Next Flagged Message"), next_flagged_cb, NULL, NULL,
-     GNOME_APP_PIXMAP_STOCK, BALSA_PIXMAP_MENU_NEXT_FLAGGED, 'F',
+     GNOME_APP_PIXMAP_STOCK, BALSA_PIXMAP_NEXT_FLAGGED, 'F',
      GDK_MOD1_MASK|GDK_CONTROL_MASK, NULL},
      GNOMEUIINFO_SEPARATOR,
      /* D */
 #define MENU_MESSAGE_TRASH_POS 16
     {
      GNOME_APP_UI_ITEM, N_("_Move to Trash"),
-     N_("Move the current message to Trash mailbox"),
+     N_("Move the message to Trash mailbox"),
      trash_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-     GNOME_STOCK_TRASH, 'D', 0, NULL},
+     GTK_STOCK_DELETE, 'D', 0, NULL},
     GNOMEUIINFO_END
 };
 
@@ -255,8 +273,13 @@ struct _MessageWindow {
     guint idle_handler_id;
 
     /* Pointers to our copies of widgets. */
+    GtkWidget *next;
+    GtkWidget *previous;
     GtkWidget *next_unread;
     GtkWidget *next_flagged;
+    GtkWidget *next_part;
+    GtkWidget *previous_part;
+    GtkWidget *trash;
 #ifdef HAVE_GTKHTML
     GtkWidget *zoom_in;
     GtkWidget *zoom_out;
@@ -277,25 +300,36 @@ static void
 message_window_move_message(MessageWindow * mw, LibBalsaMailbox * mailbox)
 {
     GArray *messages;
+    LibBalsaMessage *original = mw->message;
 
     g_return_if_fail(mailbox != NULL);
 
-   /*Transferring to same mailbox? */
+    /* Transferring to same mailbox? */
     if (mw->message->mailbox == mailbox)
-	return;
+        return;
 
     messages = g_array_new(FALSE, FALSE, sizeof(guint));
     g_array_append_val(messages, mw->message->msgno);
+
+    if (balsa_app.mw_action_after_move == NEXT_UNREAD)
+        /* Try selecting the next unread message. */
+        mw_set_selected(mw, ((void (*)(BalsaIndex *))
+                             balsa_index_select_next_unread));
+    else if (balsa_app.mw_action_after_move == NEXT)
+        mw_set_selected(mw, balsa_index_select_next);
+
     balsa_index_transfer(mw->bindex, messages, mailbox, FALSE);
     g_array_free(messages, TRUE);
-    
-    close_message_window(NULL, (gpointer) mw);
+
+    if (mw->message == original)
+        /* Either action-after-move was CLOSE, or we failed to select
+         * another message; either way, we close the window. */
+        gtk_widget_destroy(mw->window);
 }
 
 static void
-mw_set_part_buttons_sensitive(MessageWindow * mw)
+mw_set_part_buttons_sensitive(MessageWindow * mw, BalsaMessage * msg)
 {
-    BalsaMessage *msg = BALSA_MESSAGE(mw->bmessage);
     GtkWidget *toolbar;
     gboolean enable;
 
@@ -307,58 +341,31 @@ mw_set_part_buttons_sensitive(MessageWindow * mw)
     enable = balsa_message_has_next_part(msg);
     balsa_toolbar_set_button_sensitive(toolbar, BALSA_PIXMAP_NEXT_PART,
                                        enable);
-    gtk_widget_set_sensitive(message_menu[MENU_MESSAGE_NEXT_PART_POS].
-                             widget, enable);
+    gtk_widget_set_sensitive(mw->next_part, enable);
 
     enable = balsa_message_has_previous_part(msg);
     balsa_toolbar_set_button_sensitive(toolbar, BALSA_PIXMAP_PREVIOUS_PART,
                                        enable);
-    gtk_widget_set_sensitive(message_menu[MENU_MESSAGE_PREVIOUS_PART_POS].
-                             widget, enable);
+    gtk_widget_set_sensitive(mw->previous_part, enable);
 }
 
 static gboolean
 message_window_idle_handler(MessageWindow* mw)
 {
-    BalsaMessage *msg = BALSA_MESSAGE(mw->bmessage);
-    LibBalsaMessage *message = mw->message;
-    gchar *title;
-    gchar *from;
+    BalsaMessage *msg;
 
     gdk_threads_enter();
 
     mw->idle_handler_id = 0;
 
-    if (!message->mailbox) {
+    msg = BALSA_MESSAGE(mw->bmessage);
+    if (!balsa_message_set(msg, mw->message->mailbox, mw->message->msgno)) {
 	gtk_widget_destroy(mw->window);
 	gdk_threads_leave();
 	return FALSE;
     }
-
-    /* set window title */
-    from = libbalsa_address_to_gchar(message->headers->from, 0);
-    title = g_strdup_printf(_("Message from %s: %s"), from,
-                            LIBBALSA_MESSAGE_GET_SUBJECT(message));
-    g_free(from);
-    gtk_window_set_title(GTK_WINDOW(mw->window), title);
-    g_free(title);
-
-    balsa_message_set(msg, message);
-    balsa_message_grab_focus(BALSA_MESSAGE(mw->bmessage));
-    /* We may have triggered a mailbox-check, so we need to verify that
-     * the message is still good. */
-    if (!message->mailbox) {
-	gtk_widget_destroy(mw->window);
-	gdk_threads_leave();
-	return FALSE;
-    }
-
+    balsa_message_grab_focus(msg);
     balsa_message_set_close(msg, TRUE);
-    mw_set_part_buttons_sensitive(mw);
-
-    /* Update the style and message counts in the mailbox list */
-    balsa_mblist_update_mailbox(balsa_app.mblist_tree_store,
-                                message->mailbox);
 
     gdk_threads_leave();
     return FALSE;
@@ -379,10 +386,10 @@ static const struct callback_item {
     { BALSA_PIXMAP_NEXT_PART,    next_part_cb },
     { BALSA_PIXMAP_NEXT_UNREAD,  next_unread_cb },
     { BALSA_PIXMAP_NEXT_FLAGGED, next_flagged_cb },
-    { BALSA_PIXMAP_TRASH,        trash_cb },
-    { BALSA_PIXMAP_PRINT,        print_cb },
-    { BALSA_PIXMAP_SAVE,         save_current_part_cb },
-    { GTK_STOCK_CLOSE,           close_message_window },
+    { GTK_STOCK_DELETE,           trash_cb },
+    { GTK_STOCK_PRINT,           print_cb },
+    { GTK_STOCK_SAVE,            save_current_part_cb },
+    { GTK_STOCK_CLOSE,           close_message_window_cb },
     { BALSA_PIXMAP_SHOW_HEADERS, show_all_headers_tool_cb }
 };
 
@@ -395,9 +402,9 @@ static const gchar* message_toolbar[] = {
     BALSA_PIXMAP_REPLY_ALL,
     BALSA_PIXMAP_FORWARD,
     "",
-    BALSA_PIXMAP_PRINT,
+    GTK_STOCK_PRINT,
     "",
-    BALSA_PIXMAP_TRASH,
+    GTK_STOCK_DELETE,
     "",
     GTK_STOCK_CLOSE
 #else /* ENABLE_TOUCH_UI */
@@ -410,11 +417,11 @@ static const gchar* message_toolbar[] = {
     "",
     BALSA_PIXMAP_PREVIOUS_PART,
     BALSA_PIXMAP_NEXT_PART,
-    BALSA_PIXMAP_SAVE,
+    GTK_STOCK_SAVE,
     "",
-    BALSA_PIXMAP_PRINT,
+    GTK_STOCK_PRINT,
     "",
-    BALSA_PIXMAP_TRASH
+    GTK_STOCK_DELETE
 #endif /* ENEBLE_TOUCH_UI */
 };
 
@@ -450,16 +457,27 @@ message_window_get_toolbar_model(void)
 #define BALSA_MESSAGE_WINDOW_KEY "balsa-message-window"
 
 static void
-mw_set_message(MessageWindow *mw, LibBalsaMessage * message)
+mw_set_message(MessageWindow * mw, LibBalsaMessage * message)
 {
+    if (mw->idle_handler_id && !message) {
+	g_source_remove(mw->idle_handler_id);
+	mw->idle_handler_id = 0;
+    } 
+
+    if (mw->message) {
+        g_object_set_data(G_OBJECT(mw->message), BALSA_MESSAGE_WINDOW_KEY, NULL);
+        g_object_unref(mw->message);
+    }
+
     mw->message = message;
-    g_object_set_data(G_OBJECT(message), BALSA_MESSAGE_WINDOW_KEY, mw);
 
-    g_object_ref(message); /* protect from destroying */
-    mw->idle_handler_id =
-        g_idle_add((GSourceFunc) message_window_idle_handler, mw);
-
-    mw_set_buttons_sensitive(mw);
+    if (message) {
+        g_object_set_data(G_OBJECT(message), BALSA_MESSAGE_WINDOW_KEY, mw);
+        if (!mw->idle_handler_id)
+            mw->idle_handler_id =
+                g_idle_add((GSourceFunc) message_window_idle_handler, mw);
+        mw_set_buttons_sensitive(mw);
+    }
 }
 
 static void
@@ -471,35 +489,31 @@ bindex_closed_cb(gpointer data, GObject *bindex)
 }
 
 static void
-mw_disable_trash(GtkWidget * toolbar)
+mw_disable_trash(MessageWindow * mw, GtkWidget * toolbar)
 {
-    balsa_toolbar_set_button_sensitive(toolbar, BALSA_PIXMAP_TRASH, FALSE);
-    gtk_widget_set_sensitive(message_menu[MENU_MESSAGE_TRASH_POS].widget,
-                             FALSE);
+    balsa_toolbar_set_button_sensitive(toolbar, GTK_STOCK_DELETE, FALSE);
+    gtk_widget_set_sensitive(mw->trash, FALSE);
 }
 
 void
-message_window_new(LibBalsaMessage * message)
+message_window_new(LibBalsaMailbox * mailbox, guint msgno)
 {
+    LibBalsaMessage *message;
     MessageWindow *mw;
     BalsaToolbarModel *model;
     GtkWidget *toolbar;
     guint i;
     GtkWidget *move_menu, *submenu;
 
-    if (!message)
+    if (!mailbox || !msgno)
 	return;
 
-    /*
-     * Check to see if this message is already displayed
-     */
-    mw = g_object_get_data(G_OBJECT(message), BALSA_MESSAGE_WINDOW_KEY);
-    if (mw != NULL) {
-        /*
-         * The message is already displayed in a window, so just use
-         * that one.
-         */
-        gdk_window_raise(mw->window->window);
+    message = libbalsa_mailbox_get_message(mailbox, msgno);
+    if (message 
+        && (mw = g_object_get_data(G_OBJECT(message), 
+                                   BALSA_MESSAGE_WINDOW_KEY))) {
+        gtk_window_present(GTK_WINDOW(mw->window));
+        g_object_unref(message);
         return;
     }
 
@@ -528,17 +542,25 @@ message_window_new(LibBalsaMessage * message)
     g_signal_connect(G_OBJECT(mw->window), "size_allocate",
                      G_CALLBACK(size_alloc_cb), NULL);
     
-    mw->bindex = balsa_find_index_by_mailbox(message->mailbox);
+    mw->bindex = balsa_find_index_by_mailbox(mailbox);
     g_object_weak_ref(G_OBJECT(mw->bindex), bindex_closed_cb, mw);
     g_signal_connect_swapped(G_OBJECT(mw->bindex), "index-changed",
 			     G_CALLBACK(mw_set_buttons_sensitive), mw);
 
+    g_signal_connect(mailbox, "message_expunged",
+                     G_CALLBACK(mw_expunged_cb), mw);
+
     gnome_app_create_menus_with_data(GNOME_APP(mw->window), main_menu, mw);
 
     /* Save the widgets that we need to change--they'll be overwritten
      * if another message window is opened. */
-    mw->next_unread  = message_menu[MENU_MESSAGE_NEXT_UNREAD_POS].widget;
-    mw->next_flagged = message_menu[MENU_MESSAGE_NEXT_FLAGGED_POS].widget;
+    mw->next          = message_menu[MENU_MESSAGE_NEXT_POS].widget;
+    mw->previous      = message_menu[MENU_MESSAGE_PREVIOUS_POS].widget;
+    mw->next_unread   = message_menu[MENU_MESSAGE_NEXT_UNREAD_POS].widget;
+    mw->next_flagged  = message_menu[MENU_MESSAGE_NEXT_FLAGGED_POS].widget;
+    mw->next_part     = message_menu[MENU_MESSAGE_NEXT_PART_POS].widget;
+    mw->previous_part = message_menu[MENU_MESSAGE_PREVIOUS_PART_POS].widget;
+    mw->trash =         message_menu[MENU_MESSAGE_TRASH_POS].widget;
 #ifdef HAVE_GTKHTML
     mw->zoom_in  = view_menu[MENU_VIEW_ZOOM_IN].widget;
     mw->zoom_out = view_menu[MENU_VIEW_ZOOM_OUT].widget;
@@ -556,20 +578,18 @@ message_window_new(LibBalsaMessage * message)
                                     G_CALLBACK(mru_menu_cb), mw);
     move_menu = main_menu[MAIN_MENU_MOVE_POS].widget;
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(move_menu), submenu);
-    if(message->mailbox->readonly) {
+    if (mailbox->readonly) {
 	gtk_widget_set_sensitive(move_menu, FALSE);
-	mw_disable_trash(toolbar);
+	mw_disable_trash(mw, toolbar);
     }
-    if (message->mailbox == balsa_app.trash)
-	mw_disable_trash(toolbar);
+    if (mailbox == balsa_app.trash)
+	mw_disable_trash(mw, toolbar);
     mw->bmessage = balsa_message_new();
     
     gnome_app_set_contents(GNOME_APP(mw->window), mw->bmessage);
 
-#ifdef HAVE_GTKHTML
     g_signal_connect(mw->bmessage, "select-part",
 		     G_CALLBACK(mw_select_part_cb), mw);
-#endif				/* HAVE_GTKHTML */
 
     if (balsa_app.shown_headers >= HEADERS_NONE &&
 	balsa_app.shown_headers <= HEADERS_ALL)
@@ -596,37 +616,36 @@ message_window_new(LibBalsaMessage * message)
     GTK_WIDGET_UNSET_FLAGS(toolbar, GTK_CAN_FOCUS);
 }
 
-static void
-mw_clear_message(MessageWindow * mw)
-{
-    if (mw->idle_handler_id) {
-	g_source_remove(mw->idle_handler_id);
-	mw->idle_handler_id = 0;
-    } 
-    if (mw->message) {
-	g_object_set_data(G_OBJECT(mw->message), BALSA_MESSAGE_WINDOW_KEY,
-			  NULL);
-	g_object_unref(mw->message);
-	mw->message = NULL;
-    }
-}
-
 /* Handler for the "destroy" signal for mw->window. */
 static void
 destroy_message_window(GtkWidget * widget, MessageWindow * mw)
 {
-    if (mw->bindex) { /* BalsaIndex still exists */
+    if (mw->bindex) {           /* BalsaIndex still exists */
         g_object_weak_unref(G_OBJECT(mw->bindex), bindex_closed_cb, mw);
         g_signal_handlers_disconnect_matched(G_OBJECT(mw->bindex),
                                              G_SIGNAL_MATCH_DATA, 0, 0,
                                              NULL, NULL, mw);
+        mw->bindex = NULL;
     }
 
-    mw_clear_message(mw);
+    if (mw->message && mw->message->mailbox)
+        g_signal_handlers_disconnect_matched(G_OBJECT(mw->message->mailbox),
+                                             G_SIGNAL_MATCH_DATA, 0, 0,
+                                             NULL, NULL, mw);
+
+    mw_set_message(mw, NULL);
 
     g_free(mw);
 }
 
+/* Handler for the mailbox's "message-expunged" signal */
+static void
+mw_expunged_cb(LibBalsaMailbox * mailbox, guint msgno, MessageWindow * mw)
+{
+    if ((guint) mw->message->msgno == msgno)
+        gtk_widget_destroy(mw->window);
+}
+
 static void
 replyto_message_cb(GtkWidget * widget, gpointer data)
 {
@@ -635,7 +654,8 @@ replyto_message_cb(GtkWidget * widget, gpointer data)
     g_return_if_fail(widget != NULL);
     g_return_if_fail(mw != NULL);
 
-    sendmsg_window_new(widget, mw->message, SEND_REPLY);
+    sendmsg_window_reply(widget, mw->message->mailbox, 
+                         mw->message->msgno, SEND_REPLY);
 }
 
 static void
@@ -646,7 +666,8 @@ replytoall_message_cb(GtkWidget * widget, gpointer data)
     g_return_if_fail(widget != NULL);
     g_return_if_fail(mw != NULL);
 
-    sendmsg_window_new(widget, mw->message, SEND_REPLY_ALL);
+    sendmsg_window_reply(widget, mw->message->mailbox, 
+                         mw->message->msgno, SEND_REPLY_ALL);
 }
 
 static void
@@ -657,7 +678,8 @@ replytogroup_message_cb(GtkWidget * widget, gpointer data)
     g_return_if_fail(widget != NULL);
     g_return_if_fail(mw != NULL);
 
-    sendmsg_window_new(widget, mw->message, SEND_REPLY_GROUP);
+    sendmsg_window_reply(widget, mw->message->mailbox,
+                         mw->message->msgno, SEND_REPLY_GROUP);
 }
 
 static void
@@ -668,7 +690,8 @@ forward_message_attached_cb(GtkWidget * widget, gpointer data)
     g_return_if_fail(widget != NULL);
     g_return_if_fail(mw != NULL);
 
-    sendmsg_window_new(widget, mw->message, SEND_FORWARD_ATTACH);
+    sendmsg_window_forward(widget, mw->message->mailbox,
+                           mw->message->msgno, TRUE);
 }
 
 static void
@@ -679,7 +702,8 @@ forward_message_inline_cb(GtkWidget * widget, gpointer data)
     g_return_if_fail(widget != NULL);
     g_return_if_fail(mw != NULL);
 
-    sendmsg_window_new(widget, mw->message, SEND_FORWARD_INLINE);
+    sendmsg_window_forward(widget, mw->message->mailbox,
+                           mw->message->msgno, FALSE);
 }
 
 static void
@@ -690,8 +714,8 @@ forward_message_default_cb(GtkWidget * widget, gpointer data)
     g_return_if_fail(widget != NULL);
     g_return_if_fail(mw != NULL);
 
-    sendmsg_window_new(widget, mw->message, balsa_app.forward_attached 
-		       ? SEND_FORWARD_ATTACH : SEND_FORWARD_INLINE);
+    sendmsg_window_forward(widget, mw->message->mailbox,
+                           mw->message->msgno, balsa_app.forward_attached);
 }
 
 static void
@@ -700,7 +724,6 @@ next_part_cb(GtkWidget * widget, gpointer data)
     MessageWindow *mw = (MessageWindow *) data;
 
     balsa_message_next_part(BALSA_MESSAGE(mw->bmessage));
-    mw_set_part_buttons_sensitive(mw);
 }
 
 static void
@@ -709,7 +732,6 @@ previous_part_cb(GtkWidget * widget, gpointer data)
     MessageWindow *mw = (MessageWindow *) data;
 
     balsa_message_previous_part(BALSA_MESSAGE(mw->bmessage));
-    mw_set_part_buttons_sensitive(mw);
 }
 
 static void
@@ -729,7 +751,7 @@ view_msg_source_cb(GtkWidget * widget, gpointer data)
 }
 
 static void
-close_message_window(GtkWidget * widget, gpointer data)
+close_message_window_cb(GtkWidget * widget, gpointer data)
 {
     MessageWindow *mw = (MessageWindow *) data;
     gtk_widget_destroy(mw->window);
@@ -793,8 +815,8 @@ mw_set_buttons_sensitive(MessageWindow * mw)
     GtkWidget *toolbar =
         balsa_toolbar_get_from_gnome_app(GNOME_APP(mw->window));
     LibBalsaMailbox *mailbox = mw->message->mailbox;
-    guint current_msgno = mw->message->msgno;
     BalsaIndex *index = mw->bindex;
+    guint current_msgno = mw->message->msgno;
     gboolean enable;
 
     if (!mailbox) {
@@ -804,23 +826,20 @@ mw_set_buttons_sensitive(MessageWindow * mw)
 
     enable = index && balsa_index_next_msgno(index, current_msgno) > 0;
     balsa_toolbar_set_button_sensitive(toolbar, BALSA_PIXMAP_NEXT, enable);
-    gtk_widget_set_sensitive(message_menu[MENU_MESSAGE_NEXT_POS].widget,
-                             enable);
+    gtk_widget_set_sensitive(mw->next, enable);
 
     enable = index && balsa_index_previous_msgno(index, current_msgno) > 0;
     balsa_toolbar_set_button_sensitive(toolbar, BALSA_PIXMAP_PREVIOUS,
                                        enable);
-    gtk_widget_set_sensitive(message_menu[MENU_MESSAGE_PREVIOUS_POS].
-                             widget, enable);
+    gtk_widget_set_sensitive(mw->previous, enable);
 
-    enable = index
-        && balsa_index_next_unread_msgno(index, current_msgno) > 0;
+    enable = index && index->mailbox_node->mailbox->unread_messages > 0;
     balsa_toolbar_set_button_sensitive(toolbar, BALSA_PIXMAP_NEXT_UNREAD,
                                        enable);
     gtk_widget_set_sensitive(mw->next_unread, enable);
 
     enable = index
-        && balsa_index_next_flagged_msgno(index, current_msgno) > 0;
+        && libbalsa_mailbox_total_messages(index->mailbox_node->mailbox) > 0;
     balsa_toolbar_set_button_sensitive(toolbar, BALSA_PIXMAP_NEXT_FLAGGED,
                                        enable);
     gtk_widget_set_sensitive(mw->next_flagged, enable);
@@ -843,76 +862,75 @@ select_all_cb(GtkWidget * widget, gpointer data)
 {
     MessageWindow *mw = (MessageWindow *) (data);
 
-    libbalsa_window_select_all(GTK_WINDOW(mw->window));
+    balsa_window_select_all(GTK_WINDOW(mw->window));
 }
 
 static void
-mw_set_msgno(MessageWindow * mw, guint msgno)
+mw_set_selected(MessageWindow * mw, void (*select_func) (BalsaIndex *))
 {
+    guint msgno;
     LibBalsaMessage *message;
     MessageWindow *tmp;
 
-    message =
-        libbalsa_mailbox_get_message(mw->bindex->mailbox_node->mailbox,
-                                     msgno);
-    tmp = g_object_get_data(G_OBJECT(message), BALSA_MESSAGE_WINDOW_KEY);
-    if (tmp) {
-        /*
-         * The message is already displayed in a window, so just use
-         * that one.
-         */
-        g_object_unref(message);
-        gdk_window_raise(tmp->window->window);
-        return;
+    balsa_index_set_next_msgno(mw->bindex, mw->message->msgno);
+    select_func(mw->bindex);
+    msgno = balsa_index_get_next_msgno(mw->bindex);
+    message = libbalsa_mailbox_get_message(mw->message->mailbox, msgno);
+
+    if ((tmp = g_object_get_data(G_OBJECT(message), 
+                                 BALSA_MESSAGE_WINDOW_KEY))) {
+        if (tmp == mw) {
+            gtk_window_present(GTK_WINDOW(tmp->window));
+            g_object_unref(message);
+            return;
+        }
+        /* Close the other window */
+	gtk_widget_destroy(tmp->window);
     }
 
     /* Temporarily tell the BalsaMessage not to close when its message
-     * is finalized, so we can safely unref it in mw_clear_message.
+     * is finalized, so we can safely unref it in mw_set_message.
      * We'll restore the usual close-with-message behavior after setting
      * the new message. */
     balsa_message_set_close(BALSA_MESSAGE(mw->bmessage), FALSE);
-    mw_clear_message(mw);
     mw_set_message(mw, message);
-    g_object_unref(message);
 }
 
 static void
 next_message_cb(GtkWidget * widget, gpointer data)
 {
-    MessageWindow *mw = (MessageWindow *) (data);
-
-    mw_set_msgno(mw,
-                 balsa_index_next_msgno(mw->bindex, mw->message->msgno));
+    mw_set_selected((MessageWindow *) data, balsa_index_select_next);
 }
 
 static void
 previous_message_cb(GtkWidget * widget, gpointer data)
 {
-    MessageWindow *mw = (MessageWindow *) (data);
-
-    mw_set_msgno(mw,
-                 balsa_index_previous_msgno(mw->bindex,
-                                            mw->message->msgno));
+    mw_set_selected((MessageWindow *) data, balsa_index_select_previous);
 }
 
 static void
 next_unread_cb(GtkWidget * widget, gpointer data)
 {
-    MessageWindow *mw = (MessageWindow *) (data);
-
-    mw_set_msgno(mw,
-                 balsa_index_next_unread_msgno(mw->bindex,
-                                               mw->message->msgno));
+    mw_set_selected((MessageWindow *) data,
+                    ((void (*)(BalsaIndex *))
+                     balsa_index_select_next_unread));
 }
 
-static void next_flagged_cb(GtkWidget * widget, gpointer data)
+static void
+next_flagged_cb(GtkWidget * widget, gpointer data)
+{
+    mw_set_selected((MessageWindow *) data, balsa_index_select_next_flagged);
+}
+
+
+#ifdef HAVE_GTK_PRINT
+static void page_setup_cb(GtkWidget * widget, gpointer data)
 {
     MessageWindow *mw = (MessageWindow *) (data);
-
-    mw_set_msgno(mw,
-                 balsa_index_next_flagged_msgno(mw->bindex,
-                                                mw->message->msgno));
+    
+    message_print_page_setup(GTK_WINDOW(mw->window));
 }
+#endif
 
 
 static void print_cb(GtkWidget * widget, gpointer data)
@@ -925,7 +943,6 @@ static void print_cb(GtkWidget * widget, gpointer data)
 static void trash_cb(GtkWidget * widget, gpointer data)
 {
     MessageWindow *mw = (MessageWindow *) (data);
-
     message_window_move_message(mw, balsa_app.trash);
 }
 
@@ -968,13 +985,28 @@ mw_zoom_cb(GtkWidget * widget, MessageWindow * mw)
 			(G_OBJECT(widget), GNOMEUIINFO_KEY_UIDATA));
     balsa_message_zoom(BALSA_MESSAGE(bm), in_out);
 }
+#endif				/* HAVE_GTKHTML */
 
 static void
 mw_select_part_cb(BalsaMessage * bm, MessageWindow * mw)
 {
+    gchar *title;
+    gchar *from;
+#ifdef HAVE_GTKHTML
     gboolean enable = bm && balsa_message_can_zoom(bm);
+
     gtk_widget_set_sensitive(mw->zoom_in,  enable);
     gtk_widget_set_sensitive(mw->zoom_out, enable);
     gtk_widget_set_sensitive(mw->zoom_100, enable);
-}
 #endif				/* HAVE_GTKHTML */
+
+    /* set window title */
+    from = 
+        internet_address_list_to_string(bm->message->headers->from, FALSE);
+    title = g_strdup_printf(_("Message from %s: %s"), from,
+                            LIBBALSA_MESSAGE_GET_SUBJECT(bm->message));
+    g_free(from);
+    gtk_window_set_title(GTK_WINDOW(mw->window), title);
+    g_free(title);
+    mw_set_part_buttons_sensitive(mw, bm);
+}
