@@ -27,15 +27,20 @@ extern "C" {
 #endif				/* __cplusplus */
 
     typedef enum {
-	SEND_NORMAL,		/* initialized by Compose */
-	SEND_REPLY,		/* by Reply               */
-	SEND_REPLY_ALL,		/* by Reply All           */
-	SEND_REPLY_GROUP,       /* by Reply to Group      */
-	SEND_FORWARD_ATTACH,    /* by Forward attached    */
-	SEND_FORWARD_INLINE,    /* by Forward inline      */
-	SEND_CONTINUE		/* by Continue postponed  */
+       SEND_NORMAL,            /* initialized by Compose */
+       SEND_REPLY,             /* by Reply               */
+       SEND_REPLY_ALL,         /* by Reply All           */
+       SEND_REPLY_GROUP,       /* by Reply to Group      */
+       SEND_FORWARD_ATTACH,    /* by Forward attached    */
+       SEND_FORWARD_INLINE,    /* by Forward inline      */
+       SEND_CONTINUE           /* by Continue postponed  */
     } SendType;
 
+    typedef enum {
+        SENDMSG_STATE_CLEAN,
+        SENDMSG_STATE_MODIFIED,
+        SENDMSG_STATE_AUTO_SAVED
+    } SendmsgState;
 
 #if defined(ENABLE_TOUCH_UI)
 #define VIEW_MENU_LENGTH 4
@@ -55,25 +60,29 @@ extern "C" {
     struct _BalsaSendmsg {
 	GtkWidget *window;
 	GtkWidget *to[3], *from[3], *subject[2], *cc[3], *bcc[3], *fcc[3];
-#define OLD_FROM (!GTK_CHECK_VERSION(2, 4, 0))
-#if OLD_FROM
-        BalsaSendmsgAddress to_info, from_info, cc_info, bcc_info;
-#else
         BalsaSendmsgAddress to_info, cc_info, bcc_info;
-#endif /* OLD_FROM */
 #if !defined(ENABLE_TOUCH_UI)        
         GtkWidget *reply_to[3];
         BalsaSendmsgAddress reply_to_info;
 #endif
 	GtkWidget *attachments[4];
+        gchar *in_reply_to;
+        GList *references;
 	GtkWidget *text;
+#if !HAVE_GTKSPELL
 	GtkWidget *spell_checker;
+#else
+	GtkWidget *spell_check_menu_item;
+        gboolean   spell_check_error;
+#endif                          /* HAVE_GTKSPELL */
 	GtkWidget *notebook;
-	LibBalsaMessage *orig_message;
+	LibBalsaMessage *parent_message; /* to which we're replying     */
+	LibBalsaMessage *draft_message;  /* where the message was saved */
 	SendType type;
 	/* language selection related data */
 	gchar *charset;
-	const gchar *locale;
+	GSList *charsets;
+	const gchar *spell_check_lang;
 	GtkWidget *current_language_menu;
 	/* identity related data */
 	LibBalsaIdentity* ident;
@@ -87,11 +96,16 @@ extern "C" {
 	                        /* the config */
 	gulong delete_sig_id;
         gulong changed_sig_id;
+#if !HAVE_GTKSOURCEVIEW
         gulong delete_range_sig_id;
+#endif                          /* HAVE_GTKSOURCEVIEW */
+        gulong insert_text_sig_id;
         guint wrap_timeout_id;
+        guint autosave_timeout_id;
+        SendmsgState state;
         gulong identities_changed_id;
-	gboolean modified;
 	gboolean flow;          /* send format=flowed */ 
+	GtkWidget *flow_widget;
 	gboolean req_dispnotify; /* send a MDN */ 
 	gboolean quit_on_close; /* quit balsa after the compose window */
 	                        /* is closed.                          */
@@ -105,13 +119,28 @@ extern "C" {
 #endif
         GtkWidget *header_table;
 
+#if !HAVE_GTKSOURCEVIEW
         GtkTextBuffer *buffer2;       /* Undo buffer. */
+#endif                          /* HAVE_GTKSOURCEVIEW */
         GtkWidget *undo_widget;
         GtkWidget *redo_widget;
+
+        /* To update cursor after text is inserted. */
+        GtkTextMark *insert_mark;
     };
 
-    BalsaSendmsg *sendmsg_window_new(GtkWidget *, LibBalsaMessage *,
-                                     SendType);
+    BalsaSendmsg *sendmsg_window_compose(GtkWidget *);
+    BalsaSendmsg *sendmsg_window_reply(GtkWidget *, LibBalsaMailbox *,
+                                       guint msgno, SendType rt);
+    BalsaSendmsg *sendmsg_window_reply_embedded(GtkWidget *w,
+                                                LibBalsaMessageBody *part,
+                                                SendType reply_type);
+
+    BalsaSendmsg *sendmsg_window_forward(GtkWidget *, LibBalsaMailbox *,
+                                         guint msgno, gboolean attach);
+    BalsaSendmsg *sendmsg_window_continue(GtkWidget *, LibBalsaMailbox *,
+                                          guint msgno);
+
     void sendmsg_window_set_field(BalsaSendmsg *bsmsg, const gchar* key,
                                   const gchar* val);
 
@@ -124,7 +153,8 @@ extern "C" {
     void sendmsg_window_process_url(const char *url, field_setter func,
 				    void *data);
     BalsaSendmsg *sendmsg_window_new_from_list(GtkWidget * w,
-                                               GList * message_list,
+                                               LibBalsaMailbox * mailbox,
+                                               GArray * selected,
                                                SendType type);
     BalsaToolbarModel *sendmsg_window_get_toolbar_model(void);
 
